///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ATMOSPHERE_H
#define BNOTOOL_ATMOSPHERE_H
///////////////////////////////////////////////////////////////////////////////
#include "IModel.h"
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
class StaticAtmosphere : public IAtmosphereModel
  {
  public:
      //  .      
      //  
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun)
      const;
      //  
    virtual TAtmosphereId GetID() const
      { return atmIdStatic;};
      //       ?
    virtual bool IsSunPosNeed() const
      { return false;};
      //        ?
    virtual bool IsTableNeed() const
      { return false;};
      //    
      //        
    static void GetStaticAtmKoeff(double altitude, double* hi, double* k1,
                                  double* k2, double *a = NULL);
      //      
    static double Density(double h);
  };
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
class IDynAtmosphere : public IAtmosphereModel
  {
  public:
      //   
    IDynAtmosphere(ITableGeoSolarFlux *g = NULL) : IAtmosphereModel()
      { Gsf = g;};
      //       ?
    virtual bool IsSunPosNeed() const
      { return true;};
      //        ?
    virtual bool IsTableNeed() const
      { return true;};
    ITableGeoSolarFlux *Gsf;
  };
///////////////////////////////////////////////////////////////////////////////
//     -77
///////////////////////////////////////////////////////////////////////////////
class Gost77Atmosphere : public IDynAtmosphere
  {
  public:
      //   
    Gost77Atmosphere(ITableGeoSolarFlux *g = NULL) : IDynAtmosphere(g)
      {};
      //  .      
      //  
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun)
      const;
      //  
    virtual TAtmosphereId GetID() const
      { return atmIdDMA77;};
  };
///////////////////////////////////////////////////////////////////////////////
//     -90
///////////////////////////////////////////////////////////////////////////////
class Gost90Atmosphere : public IDynAtmosphere
  {
  public:
      //   
    Gost90Atmosphere(ITableGeoSolarFlux *g = NULL, bool flag3h = false) : IDynAtmosphere(g)
      {is3HourInd = flag3h;};
      //  .      
      //  
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun)
      const;
      //  
    virtual TAtmosphereId GetID() const
      { return atmIdDMA90;};

    bool is3HourInd;
  };
///////////////////////////////////////////////////////////////////////////////
//     -2000
///////////////////////////////////////////////////////////////////////////////
class Gost2000Atmosphere : public IDynAtmosphere
  {
  public:
      //   
    Gost2000Atmosphere(ITableGeoSolarFlux *g = NULL, bool flag3h = false) : IDynAtmosphere(g)
      {is3HourInd = flag3h;};
      //  .      
      //  
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun)
      const;
      //  
    virtual TAtmosphereId GetID() const
      { return atmIdDMA2000;};

    bool is3HourInd;
  };
///////////////////////////////////////////////////////////////////////////////
//     -2004
///////////////////////////////////////////////////////////////////////////////
class Gost2004Atmosphere : public IDynAtmosphere
  {
  public:
      // .
      // g -       .
      // flag3h  -    .
      // flagMod -    -2004.
    Gost2004Atmosphere(ITableGeoSolarFlux *g = NULL, bool flag3h = false, bool flagMod = false) 
      : IDynAtmosphere(g)
      { is3HourInd = flag3h; fmod = flagMod; };
      //    [/^3].   
      //     .
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun) const;
      //   .
    virtual TAtmosphereId GetID() const
      { return ( fmod ? atmIdDMA2004m : atmIdDMA2004 ); };
      //     [/^3].
      // h   -      [.].
      // f81 -      81 .
    double NightDensity(double h, double f81) const;
      //      
      // true  -   ,
      // false -   .
    bool is3HourInd;
      //    -2004, 
      //     
      //    .
      // true  -  -2004,
      // false -  "" -2004.
    bool fmod;
  };
///////////////////////////////////////////////////////////////////////////////
#endif
