///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//    ,   
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_BNOASTROBASE_H
#define BNOTOOL_BNOASTROBASE_H
///////////////////////////////////////////////////////////////////////////////
#include "IAstroBase.h"
#include "AstroTimeTables.h"
#include "GeoSolarFlux.h"
#include "Planet.h"
#include "TextConfig.h"
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
class BNOAstroBase : public IAstroBase
  {
  public:
      //   
    BNOAstroBase();

      //    
    void SetCfgFile(const char *fn)
      { CfgFName = fn; };

      //  
    bool Init(bool noAtm = false, bool loadPlanets = false);
      //        
    bool Init(const TimeGD &s, const TimeGD &e, bool noAtm = false, bool loadPlanets = false);
      //        
    bool Init(const TimeJD &s, const TimeJD &e, bool noAtm = false, bool loadPlanets = false);

      //    
    virtual void ConvertTimeScaleJD(TmScale src, TmScale dst, int &d, 
                                    double &f) const;
      //     J2000  METE
    virtual void MtxJ20002METE(const TimeJD &jdMETE, TransMtx &mtx) const;
      //    
    virtual double SaTime(const TimeJD &jd) const;
      //       
    virtual double SaTime(const TimeJD &jd, double ds) const;
      //    
    virtual double Smean(const TimeJD &dt) const;
      //     ECI   
    virtual void MtxECI2GFCS(const TimeJD &jdECI, const TimeJD &jdGFCS, 
      TransMtx &mtx) const;
      //  ,     
    virtual double MtxPrecession(const TimeJD &dtStart, const TimeJD &dtEnd, 
                         TransMtx &mtx) const;
      //     METE  
    virtual void MtxMETE2GCS(const TimeJD &t, TransMtx &mtx) const;

      //     
    virtual IPlanet *GetPlanet(TPlanetId pl);
    virtual const IPlanet *GetPlanet(TPlanetId pl) const;
      //       
    TTableGeoSolarFlux * GetTableGeoSolarFlux()
      { return &TableGeoSolarFlux;};
    const TTableGeoSolarFlux * GetTableGeoSolarFlux() const
      { return &TableGeoSolarFlux;};

      //       
    virtual void GetPosGelio(TPlanetId pl, const TimeJD &t, Vect3 *crd, Vect3 *vel = NULL, 
                     Vect3 *acc = NULL) const;
      //       
    virtual void GetPosGeo(TPlanetId pl, const TimeJD &t, Vect3 *crd, Vect3 *vel = NULL, 
                     Vect3 *acc = NULL) const;

      //      
    bool LoadEphData(TPlanetId pl);
      //      
    void UnLoadEphData(TPlanetId pl);

      //   
    void SetAtmosphereModel(TAtmosphereId mi, bool flag3h = false)
      {Earth.SetAtmModel(mi, &TableGeoSolarFlux, flag3h);};
      //     
    bool SetEarthGravityModel(TGravityModelId mg);
	
	void GetWGS84Geoid(double& EarthRadius, double& EarthOblatness) const 
	{
		EarthRadius = 6.3781369999999997;
		EarthOblatness = 0.0033528106647474805;
	}

	void GetPZ9002Geoid(double& EarthRadius, double& EarthOblatness) const 
	{
		EarthRadius = 6.378136;
		EarthOblatness = 1.0 / 0.29825784E+03;
	}

  protected:
    TString CfgFName;
    TimeJD StartTime;
    TimeJD EndTime;
    TextConfig Config;

    TTableUTC2UT1 TableUTC2UT1;
    TTableTAI2UTC TableTAI2UTC;
    TTablePolus TablePolus;
    TTableGeoSolarFlux TableGeoSolarFlux;

    SPlanet Planets[8];
    MoonPlanet Moon;
    SunPlanet Sun;
    EarthPlanet Earth;
    PlanetEphemeride EarthMoon;

      //   TAI -> UTC
    bool LoadTAITable(const char *fname)
      { return TableTAI2UTC.Init(fname);};
      //   UTC -> UT1
    bool LoadUT1Table(const char *fname)
      { return TableUTC2UT1.Init(fname);};
      //    
    bool LoadPolusTable(const char *fname)
      { return TablePolus.Init(fname);};
      //    
    bool LoadGeoSolarFlux(const char *fname)
      { return TableGeoSolarFlux.Init(fname);};

      //  ,     
    double MtxNutation(const TimeJD &dt, TransMtx &mtx) const;
      //   ,     
      //      
    void MtxPolus(double x, double y, TransMtx &mtx) const; 
      //   ,     
      //      
    void MtxPolus (const TimeJD &t, TransMtx &mtx) const; 
  };
///////////////////////////////////////////////////////////////////////////////
#endif
