///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,     
///////////////////////////////////////////////////////////////////////////////
#ifndef CLOSE_APPR_H
#define CLOSE_APPR_H

#include "PhasePoint.h"
#include "Matrix.h"
#include "twll.h"
///////////////////////////////////////////////////////////////////////////////
//   ,     
///////////////////////////////////////////////////////////////////////////////
class ICASpaceObject
  {
  public:
      // 
    virtual ~ICASpaceObject(); 

      //   
    virtual unsigned GetID() const = 0;
      //  .   ( )
    virtual double GetSize() const = 0;
      //     
    virtual void GetTimeRange(TimeJD &tmin, TimeJD &tmax) const;
      //   ,    
      // (..  ,     
      // )
    virtual void GetHeightRange(double &hp, double &ha) const;

      //      
      //    
      //   false,     
      //        
      //  ,     
    virtual bool GetPos(const TimeJD &t, PhasePoint6D &rv,
                        DSymmMatrix *cm = NULL);     
  };
///////////////////////////////////////////////////////////////////////////////
//   ,     ,
//   
// !   ,    
//     
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<ICASpaceObject> ICASpaceObjectEntry;
///////////////////////////////////////////////////////////////////////////////
//  ,     
///////////////////////////////////////////////////////////////////////////////
class CASpaceObjectList : public Twll_List_Class<ICASpaceObjectEntry>
  {
  public:
      //   ,     
    void GetHeightRange(double &hp, double &ha) const;

      //     
      // (   ,     
      //  )
    void FilterByHeight(double hp, double ha);
      //     
      // (   ,     
      //    )
    void FilterByTime(const TimeJD &tmin, const TimeJD &tmax);
  };
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
class CAParams
  {
  public:
      //--------------------------------------------------------
      //  
      //--------------------------------------------------------
      //  
    int SOID1, SOID2;
      //  
    TimeJD Tca;
      //      
    double Dr;
      //      RNB 
    Vect3 DrRNB1, DrRNB2;
      //    
    double SigDr;
    Vect3 SigDrRNB1, SigDrRNB2;
      //       RNB 
    Vect3 SigDpRNB1, SigDpRNB2;
      //  .  
    double Dv;
      //   .    RNB 
    Vect3 DvRNB1, DvRNB2;
      //   
    double MinOrbDistance;
      //  
    double Alpha1, Beta1, Alpha2, Beta2;
      //  
    double Probab;
      //      
    PhasePoint6D Pos1, Pos2;
      //    
    DSymmMatrix Cm1, Cm2;

      //--------------------------------------------------------
      // 
      //--------------------------------------------------------
      //      
      //    
    bool Set(ICASpaceObject &ko1, ICASpaceObject &ko2, const TimeJD tca);
   
  protected:
      //--------------------------------------------------------
      //  ,   
      //--------------------------------------------------------
      //  
    static void UnitVect(const Vect3 &vect, Vect3 &unit);
      //      RNB
    static void BuildRNB(const PhasePoint6D &rp, Vect3 &r, Vect3 &n, Vect3 &b);
      //      RNB
    static void BuildCM_RNB(const Vect3 &r, const Vect3 &n, const Vect3 &b, 
                            const DSymmMatrix &ksrc, DSymmMatrix &krnb);
      //     33
    static double SMDet(const DSymmMatrix &p);
  };
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<CAParams> CAParamsEntry;
class CAParamsList : public Twll_List_Class<CAParamsEntry>
  {
  public:
      //  
      // dt -  ""   ( . )
    void KillDoubles(double dt = 1.e-3);
  };
///////////////////////////////////////////////////////////////////////////////
//  ,     
//  
//       
///////////////////////////////////////////////////////////////////////////////
class CAOrbitNodes
  {
  public:
    CAOrbitNodes()
      {
      NodeFlag = false;
      };

    virtual ~CAOrbitNodes()
      {
      }

      //    
    ICASpaceObject *Ko;

      //  
    bool  Init(ICASpaceObject &ko)
      { Ko = &ko;};

      //     
      //   
    void SetNode(const TimeJD &jd);
      //   
    bool IsNodeValid() const
      { return NodeFlag; };
      //     
    void FastPredict(const TimeJD &jd);
      //    
    double GetSortFCoord() const
      { return F_r[0]; };
      //    
    const Vect3 &GetFCoords() const
      { return F_r; };
      //    
    const Vect3 &GetFVelocity() const
      { return F_v; };
      // ,    
    const TimeJD &FPredictTime() const
      { return F_time; };

      //  .  true,    
    bool GetCoordVelocity(const TimeJD &jd, Vect3 &coord, Vect3 &velocity);

      //       
      //  
    bool IsInsideInterval(const TimeJD &jd) const;
      //     
    //const TimeJD &GetStartTime() const;
      //     
    //const TimeJD &GetEndTime() const;

  private:
    //   
      //  
    TimeJD  Node_Time;
    bool    NodeFlag;
    Vect3   Node_R;
    Vect3   Node_V;
    Vect3   Node_A;

      //   
    TimeJD  F_time;
    Vect3   F_r;
    Vect3   F_v;
    };
///////////////////////////////////////////////////////////////
//  ,    
///////////////////////////////////////////////////////////////
class CASeacher
  {
  public:
      // -----------------------------------------------------
      //  ,   
      //      
      //  
      // -----------------------------------------------------
    double MinPermissDistRad;
    double MinPermissDistNorm;
    double MinPermissDistBinorm;
    double MinPermissDistRadAbs;
    double MinPermissDistNormAbs;
    double MinPermissDistBinormAbs;
    double MinProbability;
    double TimeProbStep;
    double CoordDiff;
    double VelocityDiff;
    double LinearInt;
    double RVectorDiff2;
    double RVectorDiff3;

    CASeacher(void);
    virtual ~CASeacher(void)
      {};

      //    "  "
    bool Run(CASpaceObjectList &ListKO1, CASpaceObjectList &ListKO2, 
             const TimeJD &tstart, const TimeJD &tend);
      //   "  "
    bool Run(ICASpaceObject &dko1, ICASpaceObject &dko2, 
             const TimeJD &tstart, const TimeJD &te);
      //   
    const CAParamsList &GetResult() const
      { return ListCA;};
      //   
    void Clear()
      { ListCA.clear();};

  protected:
      //   
    CAParamsList ListCA;
      //     
      //  left_t  left_t + TimeProbStep
      //   -    
    void AddCA(CAOrbitNodes &do2, CAOrbitNodes &do1, 
               const TimeJD &left_t, const TimeJD &tend);
      //      
      //  
    static void MinDistance(const Vect3 &r1, const Vect3 &v1, const Vect3 &r2, const Vect3 &v2,
                            double &min_gap, double &time);
      //     ?
    virtual bool IsDangerous(const CAParams &ca, 
                             const ICASpaceObject &ko1, const ICASpaceObject &ko2);
  };
///////////////////////////////////////////////////////////////////////////////
#endif
