///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,   
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"

#include <math.h>
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
void DDiagMatrix::Init(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  DLineMem::Init(mt.Dimension);
 
  memcpy(Vector, mt.Vector, mt.Dimension * sizeof(double));
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DDiagMatrix & DDiagMatrix::operator+=(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(mt.Dim() == Dimension);
  TAssert(Vector != NULL);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Vector[i] += mt.Vector[i];

  return *this;
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DDiagMatrix & DDiagMatrix::operator-=(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(mt.Dim() == Dimension);
  TAssert(Vector != NULL);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Vector[i] -= mt.Vector[i];

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DDiagMatrix & DDiagMatrix::MulRight(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(mt.Dim() == Dimension);
  TAssert(Vector != NULL);
  TAssert(Vector != mt.Vector);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Vector[i] *= mt.Vector[i];

  return *this;
  }
//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
bool DDiagMatrix::Expand(DUppMatrix &s, DDiagMatrix &d) const
  {
  TAssert(Vector != NULL);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    if (fabs(Vector[i]) < ZERO_ACCURACY)
      return false;

  d = *this;
  s.Init(Dimension);

  s.Init(0.0);
  for (i = 0; i < Dimension; i++)
    s.Buffer[s.Index(i, i)] = 1.0;

  return true;
  }
//-----------------------------------------------------------------------------
//  (  )
//-----------------------------------------------------------------------------
bool DDiagMatrix::Inverse()
  {
  TAssert(Vector != NULL);

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    if (fabs(Vector[i]) < ZERO_ACCURACY)
      {
      for (j = 0; j < i; j++)
        Vector[j] = 1.0 / Vector[j];
      return false;
      }
    Vector[i] = 1.0 / Vector[i];
    }

  return true;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
void DDiagMatrix::GetLine(unsigned index, double *line) const
  {
  TAssert(Vector != NULL);
  TAssert(line != NULL);
  TAssert(index < Dimension);

  memset(line, 0, Dimension * sizeof(double));
  line[index] = Vector[index];  
  }
//-----------------------------------------------------------------------------
//    o
//-----------------------------------------------------------------------------
void DDiagMatrix::GetCol(unsigned index, double *col) const
  {
  GetLine(index, col);
  }
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
void DDiagMatrix::GetArray(double *array) const
  {
  TAssert(Vector != NULL);
  TAssert(array != NULL);

  memset(array, 0, Dimension * Dimension * sizeof(double));

  register unsigned i;
  for (i = 0; i < Dimension; i++)
    {
    *array = Vector[i];
    array += Dimension + 1;
    }
  }
///////////////////////////////////////////////////////////////////////////////
