///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,    
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
void DUppMatrix::Init(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  DTriagMem::Init(mt.Dim());

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] = mt.Buffer[mt.Index(i, j)];
    }

  }
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
void DUppMatrix::Init(const DDiagMatrix &mt)
  {
  DTriagMem::Init(mt.Dim());

  DTriagMem::Init(0.0);
  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Buffer[Index(i, i)] = mt.Vector[i];
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::operator+=(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(mt.Dim() == Dimension);
  TAssert(Buffer != NULL);

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] += mt.Buffer[mt.Index(i, j)];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::operator-=(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(mt.Dim() == Dimension);
  TAssert(Buffer != NULL);

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] -= mt.Buffer[mt.Index(i, j)];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::operator+=(const DDiagMatrix &mt)
  {
  TAssert(mt.Dim() == Dimension);
  TAssert(Buffer != NULL);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Buffer[Index(i, i)] += mt.Vector[i];

  return *this;
  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::operator-=(const DDiagMatrix &mt)
  {
  TAssert(mt.Dim() == Dimension);
  TAssert(Buffer != NULL);

  register unsigned i;

  for (i = 0; i < Dimension; i++)
    Buffer[Index(i, i)] -= mt.Vector[i];

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::MulRight(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(Buffer != NULL);
  TAssert(Buffer != mt.Buffer);
  TAssert(Dimension == mt.Dimension);

  DUppMatrix src(Dimension);
  XChng(src);

  register unsigned i, j, k;
  double *p;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      {
      p = Buffer + Index(i, j);
      *p = 0.0;
      for (k = i; k < j + 1; k++)
        *p += src.Buffer[src.Index(i, k)] * mt.Buffer[mt.Index(k, j)];
      }
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::MulLeft(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(Buffer != NULL);
  TAssert(Buffer != mt.Buffer);
  TAssert(Dimension == mt.Dimension);

  DUppMatrix src(Dimension);
  XChng(src);

  register unsigned i, j, k;
  double *p;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      {
      p = Buffer + Index(i, j);
      *p = 0.0;
      for (k = i; k < j + 1; k++)
        *p += mt.Buffer[mt.Index(i, k)] * src.Buffer[src.Index(k, j)];
      }
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::MulRight(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(Buffer != NULL);
  TAssert(Dimension == mt.Dim());

  DUppMatrix src(Dimension);
  XChng(src);

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] = src.Buffer[src.Index(i, j)] * mt.Vector[j];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::MulLeft(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(Buffer != NULL);
  TAssert(Dimension == mt.Dim());

  DUppMatrix src(Dimension);
  XChng(src);

  register unsigned i, j;

  for (i = 0; i < Dimension; i++)
    {
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] = mt.Vector[i] * src.Buffer[src.Index(i, j)];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
DUppMatrix & DUppMatrix::Transpose(const DLowMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);

  DTriagMem::Init(mt.Dim());

  register unsigned i, j;
   
  for (i = 0; i < Dimension; i++)
    for (j = i; j < Dimension; j++)
      Buffer[Index(i, j)] = mt.Buffer[mt.Index(i, j)];

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
void DUppMatrix::GetLine(unsigned index, double *line) const
  {
  TAssert(Buffer != NULL);
  TAssert(line != NULL);
  TAssert(index < Dimension);

  if (index > 0)
    memset(line, 0, index * sizeof(double));
  memcpy(line + index, Buffer + Index(index, index), (Dimension - index) * sizeof(double));
  
  }
//-----------------------------------------------------------------------------
//    o
//-----------------------------------------------------------------------------
void DUppMatrix::GetCol(unsigned index, double *col) const
  {
  TAssert(Buffer != NULL);
  TAssert(col != NULL);
  TAssert(index < Dimension);

  register unsigned i;

  for (i = 0; i <= index; i++)
    col[i] = *(Buffer + Index(i, index));
  if (index < Dimension - 1)
    memset(col + index + 1, 0, (Dimension - index - 1) * sizeof(double));
  }
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
void DUppMatrix::GetArray(double *array) const
  {
  TAssert(Buffer != NULL);
  TAssert(array != NULL);

  register unsigned i;
  for (i = 0; i < Dimension; i++)
    {
    GetLine(i, array);
    array += Dimension;
    }
  }
///////////////////////////////////////////////////////////////////////////////

