///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,  -
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"

#include <math.h>
///////////////////////////////////////////////////////////////////////////////
//  
//-----------------------------------------------------------------------------
DVector & DVector::operator=(const CDMatrix &mt)
  {
  TAssert(mt.Cols() == 1);

  Init(mt.Rows());
  mt.GetCol(0, Vector);

  return *this;
  }
//-----------------------------------------------------------------------------
//   
//-----------------------------------------------------------------------------
double DVector::Abs(void) const
  {
  TAssert(Vector != NULL);

  double s = 0;
  for (register unsigned i = 0; i < Dimension; i++)
    s += Vector[i] * Vector[i];

  return sqrt(s);
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DVector & DVector::MulLeft(const DUppMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(Dimension == mt.Dim());

  DVector ptr;
  ptr.Init(mt.Dimension);
  XChng(ptr);

  register unsigned i, j;

  for (i = 0; i < mt.Dimension; i++)
    {
    Vector[i] = 0;
    for (j = i; j < Dimension; j++)
      Vector[i] += mt.Buffer[mt.Index(i, j)] * ptr[j];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DVector & DVector::MulLeft(const DLowMatrix &mt)
  {
  TAssert(mt.Buffer != NULL);
  TAssert(Dimension == mt.Dim());

  DVector ptr;
  ptr.Init(mt.Dimension);
  XChng(ptr);

  register unsigned i, j;

  for (i = 0; i < mt.Dimension; i++)
    {
    Vector[i] = 0;
    for (j = 0; j < i + 1; j++)
      Vector[i] += mt.Buffer[mt.Index(j, i)] * ptr[j];
    }

  return *this;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
DVector & DVector::MulLeft(const DDiagMatrix &mt)
  {
  TAssert(mt.Vector != NULL);
  TAssert(Dimension == mt.Dim());

  register unsigned i;
  for (i = 0; i < Dimension; i++)
    Vector[i] *= mt.Vector[i];
  return *this;
  }
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
DVector & DVector::TRectRectVector(const CDMatrix &a, const CDMatrix &p, const DVector &b)
  {
  TAssert(a.Rows() == p.Rows());
  TAssert(p.Cols() == b.Rows());
  
  Init(a.Cols());
  
  register unsigned i, j, k;
  
  for (i = 0; i < Dimension; i++)
    {
    Vector[i] = 0.0;
    for (k = 0; k < b.Dimension; k++)
      {
      double s = 0.0;
      for (j = 0; j < a.Rows(); j++)
        s += a(j, i) * p(j, k);
      Vector[i] += s * b.Vector[k];	
      }
    }
    
  return *this;    
  }						    
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
void DVector::GetLine(unsigned index, double *line) const
  {
  TAssert(Vector != NULL);
  TAssert(index < Dimension);
  TAssert(line != NULL);

  *line = Vector[index];
  }
//-----------------------------------------------------------------------------
//    o
//-----------------------------------------------------------------------------
void DVector::GetCol(unsigned index, double *col) const
  {
  TAssert(Vector != NULL);
  TAssert(index == 0);
  TAssert(col != NULL);

  memcpy(col, Vector, Dimension * sizeof(double));
  }
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
void DVector::GetArray(double *array) const
  {
  TAssert(Vector != NULL);
  TAssert(array != NULL);

  GetCol(0, array);
  }
///////////////////////////////////////////////////////////////////////////////
