///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  3-  6-     
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL__DVECTORS_H
#define BNOTOOL__DVECTORS_H
////////////////////////////////////////////////////////////////////////////////
#include <string.h>
#include <stdio.h>
#include <math.h>
////////////////////////////////////////////////////////////////////////////////
  //    3x3
class TransMtx;
////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
class Vect3
  {
  public:
    Vect3(double x_ = 0, double y_ = 0, double z_ = 0)
      { Init(x_, y_, z_);};

      //  
    Vect3 &Init(double x_ = 0, double y_ = 0, double z_ = 0)
      {x = x_; y = y_; z = z_ ; return *this; };
      
      //   
    double Len() const
      { return sqrt(x * x + y * y + z * z);};
     
      //   
    double Len2() const
      { return x * x + y * y + z * z;};

      //     
      // (..  - -)
    Vect3 &MulRight(const TransMtx &mtx);
      //     
      // (..  - -)
    Vect3 &MulLeft(const TransMtx &mtx)
      { *this *= mtx; return *this; };
    Vect3 &operator*=(const TransMtx &mtx);

      // 
      //    
    Vect3 & operator-()
      { x = -x; y = -y; z = -z; return *this;};
      //   
    bool operator==(const Vect3 &p) const
      { return (x == p.x) && (y == p.y) && (z == p.z);};
      //     
    Vect3 &operator=(double t) 
      { x = t; y = t; z = t; return *this; };
      //     
    double & operator[](int i) 
      { return (i == 0 ? x : (i==1 ? y : z)); };
      //      i
    double operator[](int i) const
      { return (i == 0 ? x : (i==1 ? y : z)); };
      //   
    Vect3 &operator+=(const Vect3 &v0) 
      { x += v0.x; y += v0.y; z += v0.z; return *this; };

      //   
    Vect3 &operator-=(const Vect3 &v0) 
      { x -= v0.x; y -= v0.y; z -= v0.z; return *this; };
      //    
    Vect3 &operator*=(double mul) 
      { x *= mul; y *= mul; z *= mul; return *this; };
      //    
    Vect3 &operator/=(double div) 
      { x /= div; y /= div; z /= div; return *this; };
      //  
    double operator*(const Vect3 &v0) const 
      { return x * v0.x + v2 * v0.y + z * v0.z; };

      //  
    Vect3 &MulVect(const Vect3 &v0) 
      {
      double x_= x, y_ = y, z_= z;
      x = y_ * v0.z - z_ * v0.y;
      y = z_ * v0.x - x_ * v0.z;
      z = x_ * v0.y - y_ * v0.x;
      return *this;
      };

      //   
    void print(const char *s, FILE *out = stdout) const
      { fprintf(out,"%s%23.16e,%23.16e,%23.16e\n", s, x, y, z); };

      //  
    void Normalize()
      {
      double a = 1.0 / Len();
      x *= a;
      y *= a;
      z *= a;
      }

      //  
    union 
      { double x; double v1; };
    union 
      { double y; double v2; };
    union 
      { double z; double v3; };
  };
////////////////////////////////////////////////////////////////////////////////
//      ( + )
////////////////////////////////////////////////////////////////////////////////
class Vect6 : public Vect3
  {
  public:
      // 
    Vect6(double x_ = 0, double y_ = 0, double z_ = 0, 
          double vx_ = 0, double vy_ = 0, double vz_ = 0)
      { Init(x_, y_, z_, vx_, vy_, vz_); };
    Vect6(const Vect3 &pt, double vx_, double vy_, double vz_)
      { Init(pt, vx_, vy_, vz_); };
    Vect6(const Vect3 &pt, const Vect3 &v)
      { Init(pt, v); };

      //  
    Vect6 & Init(double x_ = 0, double y_ = 0, double z_ = 0, 
                 double vx_ = 0, double vy_ = 0, double vz_ = 0)
      { Vect3::Init(x_,y_,z_); vx = vx_; vy = vy_; vz = vz_; return *this;};
    Vect6 & Init(const Vect3 &pt, double vx_, double vy_, double vz_)
      { Vect3::Init(pt.x, pt.y, pt.z); vx = vx_; vy = vy_; vz = vz_; return *this;};
    Vect6 & Init(const Vect3 &pt, const Vect3 &v)
      { Vect3::Init(pt.x, pt.y, pt.z); vx = v.x; vy = v.y; vz = v.z; return *this;};
    
      //   
    double LenV() const
      { return sqrt(vx * vx + vy * vy + vz * vz);};
      //    
    double LenV2() const
      { return vx * vx + vy * vy + vz * vz;};

      //    
    Vect6 & operator-()
      { x = -x; y = -y; z = -z; vx = -vx; vy = -vy; vz = - vz; return *this;};
      //   
    bool operator==(const Vect6 &p) const
      { return (x == p.x) && (y == p.y) && (z == p.z) && (vx == p.vx) && (vy == p.vy) 
                && (vz == p.vz);};
      //     
    double & operator[](int i) 
      { return i == 0 ? x : i == 1 ? y : i == 2 ? z : i == 3 ? vx : i ==4 ? vy : vz; };
      //      i
    double operator[](int i) const
      { return i == 0 ? x : i == 1 ? y : i == 2 ? z : i == 3 ? vx : i ==4 ? vy : vz; };

      //  ,   
    Vect3 & GetVelocity(Vect3 &v) const
      { v.Init(vx, vy, vz); return v; };
      //      
    void SetVelocity(const Vect3 &v)
      { vx = v.x; vy = v.y; vz = v.z;};

      //   
    void print(const char *s, FILE *out = stdout) const;

      //  
    double vx, vy, vz;
  };
////////////////////////////////////////////////////////////////////////////////
class TransMtx 
  {
  public:
      // 
    TransMtx() 
      {};
      //   
    TransMtx(const double *p) 
      { Init(p); };

      //  (   p   
    TransMtx & Init(const double *p) 
      { memcpy(mtx, p, sizeof(double) * 9); return *this; };
      //   
    TransMtx & Init1() 
      { 
      mtx[0]=1.0; mtx[1]=0.0; mtx[2]=0.0;
      mtx[3]=0.0; mtx[4]=1.0; mtx[5]=0.0;
      mtx[6]=0.0; mtx[7]=0.0; mtx[8]=1.0;
      return *this;
      };

      //      
    double operator()(int row, int col) const 
      { return mtx[row * 3 + col]; };
      //     
    double & operator()(int row, int col) 
      { return mtx[row * 3 + col]; };
      //     
      // (   )
    double & operator[](int pos) 
      { return mtx[pos]; };
    double operator[](int pos) const 
      { return mtx[pos]; };

      // 
    TransMtx &Transp();
      // ,   m
      //    
    void Transp(TransMtx &m) const;

       //    : A = A * B
    TransMtx &operator*=(double a);
      //    : A = A * B
    TransMtx &operator*=(const TransMtx &mRight);
    TransMtx &MulRight(const TransMtx &mRight)
      { *this *= mRight; return *this; };
      //    : A = B * A
    TransMtx &MulLeft(const TransMtx &mLeft);
      //     m
    void mul(const TransMtx &m, TransMtx &res) const;
      //  
    double Det() const;

      //  
    TransMtx &operator+=(const TransMtx &mtx);
      //  
    TransMtx &operator-=(const TransMtx &mtx);
      //     
    TransMtx &operator = (double val);
      //     
    bool operator==(const TransMtx &mtx) const;

      //   
    void print(const char *name,FILE *out=stdout) const;

      //      ( )
    double *GetPtr() 
      { return mtx; };
      //      ( )
    const double *GetConstPtr() const
      { return mtx; };
      //      
    void Save(double *p)  const
      { memcpy(p, mtx, sizeof(double) * 9); };

  private:
    double mtx[9];
  };
////////////////////////////////////////////////////////////////////////////////
#endif
