///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//      
///////////////////////////////////////////////////////////////////////////////
#include "DigPredict.h"
///////////////////////////////////////////////////////////////////////////////
//   
//-----------------------------------------------------------------------------
DigPredict::DigPredict()
{
	Integrator = NULL;
	Adapter = NULL;
	KOProp = NULL;
	FlagInit = false;
}
//-----------------------------------------------------------------------------
//   
//-----------------------------------------------------------------------------
void DigPredict::SetIntegrator(IODESolver *slv)
{
	Integrator = slv;
	Integrator->SetSystem(&System);
}
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
void DigPredict::SetKOProp(IKOProp *prop)
{ 
	KOProp = prop;
	KOProp->Init(MMode);
};
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
void DigPredict::SetInitPoint(const PhasePoint6D &src)
{
	TAssert(Integrator != NULL);
	TAssert(Adapter != NULL);
	TAssert(KOProp != NULL);

	//   
	T0 = src.T;
	System.Init(T0, MMode, *KOProp, *Adapter);

	//  .   J2000
	PhasePoint6D j2000;
	J2000Frame f2000;
	j2000.SetFrame(f2000);
	j2000.Assign(src);


	double Y0[6];

	Y0[0] = j2000.CoordsVel.x; 
	Y0[1] = j2000.CoordsVel.y; 
	Y0[2] = j2000.CoordsVel.z; 
	Y0[3] = j2000.CoordsVel.vx; 
	Y0[4] = j2000.CoordsVel.vy; 
	Y0[5] = j2000.CoordsVel.vz; 

	Integrator->SetInitial(0.0, Y0); 

	FlagInit = true;
}
//-----------------------------------------------------------------------------
// ""    
//-----------------------------------------------------------------------------
void DigPredict::ReRun()
{
	PhasePoint6D tmp;
	GetDestPoint(tmp);
	SetInitPoint(tmp);
}
//-----------------------------------------------------------------------------
//   
//-----------------------------------------------------------------------------
//2012 void DigPredict::ReInit()
//  {
//  Integrator->SetInitial(0.0, Y0);
//  }
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
bool DigPredict::Run(const TimeJD &t)
{
	TAssert(FlagInit);
	//  ,     
	// (  )
	double dt = (t - T0).GetDouble() * 86.4;
	return Integrator->Run(dt);
}
//-----------------------------------------------------------------------------
//   
//-----------------------------------------------------------------------------
void DigPredict::GetDestPoint(PhasePoint6D &dest)
{
	TAssert(FlagInit);

	dest.T = T0;
	dest.T += Integrator->GetX() / 86.4;

	J2000Frame f2000;
	dest.SetFrame(f2000);

	const double *y = Integrator->GetSolution();

	dest.CoordsVel.x = y[0];
	dest.CoordsVel.y = y[1];
	dest.CoordsVel.z = y[2];
	dest.CoordsVel.vx = y[3];
	dest.CoordsVel.vy = y[4];
	dest.CoordsVel.vz = y[5];
}
///////////////////////////////////////////////////////////////////////////////
//   :
// -  ..
// -  ,     
// -    .   .  
///////////////////////////////////////////////////////////////////////////////
//      (. , . )
//-----------------------------------------------------------------------------
//static double Eps[] =
//  {
//  1.e-9, 1.e-9, 1.e-9, 1.e-9, 1.e-9, 1.e-9
//  };
//-----------------------------------------------------------------------------
//   
//-----------------------------------------------------------------------------
DigStPredict::DigStPredict()
:
DigPredict(),
m_eps(1E-9), 
m_stepinit(10E-3), 
m_stepmin(.1E-3)
{
	//  
	DigPredict::SetIntegrator(&SIntegrator);
	//  
	DigPredict::SetAdapter(&SAdapter);
	//   
	DigPredict::SetKOProp(&KOModel);
	//    
	SetAccuracy(m_eps, m_stepinit, m_stepmin, false);
}

DigStPredict::DigStPredict(double eps, double stepinit, double stepmin, bool precItrf)
:
DigPredict(),
m_eps(eps), 
m_stepinit(stepinit), 
m_stepmin(stepmin)
{
	//  
	DigPredict::SetIntegrator(&SIntegrator);
	//  
	DigPredict::SetAdapter(&SAdapter);
	//   
	DigPredict::SetKOProp(&KOModel);
	//    
	SetAccuracy(m_eps, m_stepinit, m_stepmin, false);
}

void DigStPredict::SetKOParams(double kb, double kappa)
{
	KOModel.Kb = kb;
	KOModel.Kappa = kappa;
} 

void DigStPredict::SetAccuracy(double eps, double stepinit, double stepmin, bool precItrf)
{
	double epsd[6];

	for (int i = 0; i < 6; i++){
		epsd[i] = eps;
	}

	SIntegrator.SetParams(epsd, stepinit /* .  */, stepmin /* . */);

	System.SetPrecItrf(precItrf);
}
