///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//       
//    
////////////////////////////////////////////////////////////////////////////////
#include "FrameConverter.h"
////////////////////////////////////////////////////////////////////////////////
  //       
  //       .
void FrameConverter3D::Convert(const Vect3 &source, Vect3 &target) const
  {
  target = source;
  target.MulLeft(Rotation);
  target += Shift;
  }
//------------------------------------------------------------------------------
  //    result  
void FrameConverter3D::Invert(FrameConverter3D &result) const
  {
  Rotation.Transp(result.Rotation);
  result.Shift = Shift;
  result.Shift.MulLeft(result.Rotation);

  result.Shift.x = -result.Shift.x;
  result.Shift.y = -result.Shift.y;
  result.Shift.z = -result.Shift.z;
  }
//------------------------------------------------------------------------------
  //     
FrameConverter3D & FrameConverter3D::Invert()
  {
  FrameConverter3D m;
  Invert(m);

  *this = m;

  return *this ;
  }
//------------------------------------------------------------------------------
  //   result   .
  //     -  
  //    -  c.
void FrameConverter3D::Multiply(const FrameConverter3D &c, 
                                FrameConverter3D & result) const
  {
  c.Rotation.mul(Rotation, result.Rotation);
  result.Shift = Shift;
  result.Shift.MulLeft(c.Rotation);
  result.Shift += c.Shift;
  }
//------------------------------------------------------------------------------
  //         
FrameConverter3D & FrameConverter3D::Multiply(const FrameConverter3D & c)
  {
  FrameConverter3D m;
  Multiply(c, m);
  *this = m;
  return *this ;
  }
////////////////////////////////////////////////////////////////////////////////
  //        
FrameConverter6D & FrameConverter6D::Init(const TransMtx &mc, const TransMtx &mv,
                                          const Vect6 &rv)
  {
  Converter3D.Init(mc, rv);
  VRotation = mv;
  rv.GetVelocity(VShift);
  return *this;
  }
//------------------------------------------------------------------------------
  //       
  //       .
void FrameConverter6D::Convert(const Vect6 &source, Vect6 &target) const
  {
  Converter3D.Convert(source, target);

  Vect3 vt, vv = source;

  source.GetVelocity(vt);
  vt.MulLeft(Converter3D.Rotation);
  vv.MulLeft(VRotation);
  vt += vv;
  vt += VShift;
  target.SetVelocity(vt);
  }
//------------------------------------------------------------------------------
  //    result  
void FrameConverter6D::Invert(FrameConverter6D & result) const
  {
  Converter3D.Invert(result.Converter3D);

  VRotation.Transp(result.VRotation);

  Vect3 s;
  s = Converter3D.Shift;
  s.MulLeft(result.VRotation);
  result.VShift = VShift;
  result.VShift.MulLeft(result.Converter3D.Rotation);
  result.VShift += s;

  result.VShift.x = -result.VShift.x;
  result.VShift.y = -result.VShift.y;
  result.VShift.z = -result.VShift.z;
  }
//------------------------------------------------------------------------------
  //     
FrameConverter6D & FrameConverter6D::Invert()
  {
  FrameConverter6D m;
  Invert(m);
  *this = m;
  return *this ;
  }
//------------------------------------------------------------------------------
  //   result   .
  //     -  
  //    -  c.
void FrameConverter6D::Multiply(const FrameConverter6D &c, 
                                FrameConverter6D & result) const
  {
  TransMtx q;

  Converter3D.Multiply(c.Converter3D, result.Converter3D);

  c.Converter3D.Rotation.mul(VRotation, result.VRotation);
  c.VRotation.mul(Converter3D.Rotation, q);
  result.VRotation += q;

  Vect3 s;

  s = VShift;
  s.MulLeft(c.Converter3D.Rotation);
  result.VShift = Converter3D.Shift;
  result.VShift.MulLeft(c.VRotation);
  result.VShift += s;
  result.VShift += c.VShift;
  }
//------------------------------------------------------------------------------
  //         
FrameConverter6D & FrameConverter6D::Multiply(const FrameConverter6D & c)
  {
  FrameConverter6D m;
  Multiply(c, m);
  *this = m;
  return *this ;
  }
////////////////////////////////////////////////////////////////////////////////
