///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IASTROBASE_H
#define BNOTOOL_IASTROBASE_H
///////////////////////////////////////////////////////////////////////////////
#include "TM.h"
#include "DVectors.h"
#include "Consts.h"
#include "Ids.h"
#include "IModel.h"
#include "TMScale.h"

//    
///////////////////////////////////////////////////////////////////////////////
class IAstroBase
  {
  public:
    virtual ~IAstroBase()
      {};

      //    
    virtual void ConvertTimeScaleJD(TmScale src, TmScale dst, int &d, 
                                    double &f) const = 0;

      //     J2000  METE
    virtual void MtxJ20002METE(const TimeJD &jdMETE, TransMtx &mtx) const = 0;
      //    
    virtual double SaTime(const TimeJD &jd) const = 0;
      //       
    virtual double SaTime(const TimeJD &jd, double ds) const = 0;
      //    
    virtual double Smean(const TimeJD &dt) const = 0;
      //     ECI   
    virtual void MtxECI2GFCS(const TimeJD &jdECI, const TimeJD &jdGFCS, 
      TransMtx &mtx) const = 0;
      //  ,     
    virtual double MtxPrecession(const TimeJD &dtStart, const TimeJD &dtEnd, 
                                 TransMtx &mtx) const = 0;
      //     METE  
    virtual void MtxMETE2GCS(const TimeJD &t, TransMtx &mtx) const = 0;

      //     
    virtual IPlanet *GetPlanet(TPlanetId pl) = 0;
    virtual const IPlanet *GetPlanet(TPlanetId pl) const = 0;

      //       
    virtual void GetPosGelio(TPlanetId pl, const TimeJD &t, Vect3 *crd, Vect3 *vel = NULL, 
                     Vect3 *acc = NULL) const = 0;
      //       
    virtual void GetPosGeo(TPlanetId pl, const TimeJD &t, Vect3 *crd, Vect3 *vel = NULL, 
                     Vect3 *acc = NULL) const = 0;
      //        
      //  
    void GetGelio2GeoShift(const Vect3 &vEarthMoon, const Vect3 &vMoon, Vect3 &result) const;

      //     J2000   
    void MtxJ20002GFCS(const TimeJD &jdGFCS, TransMtx &mtx) const
      { TimeJD tf(0, doubleJ2000); MtxECI2GFCS(tf, jdGFCS, mtx);};
      //        
      //  
    void GCSVelocityMatrix(TransMtx &m) const;
      //        NEZ
    void MtxGCS2NZE(const Vect3 &p, TransMtx &mtx) const;

      //    
      //   [ / . .]
    double GetEarthRotation() const
      { return GetPlanet(plIdEarth)->omega;};
      //  
    double GetEarthOblatness() const
      { return GetPlanet(plIdEarth)->Geometry.alpha;};
      //  
    double GetEarthRadius() const
      { return GetPlanet(plIdEarth)->Geometry.Re;};
      //  
    double GetEarthMu() const
      { return GetPlanet(plIdEarth)->GetGravityModel()->mu;};
      //  
    const IAtmosphereModel *GetEarthAtmosphere() const
      { return GetPlanet(plIdEarth)->GetAtmosphere();};

  protected:
      //        
      //     dt
    void MtxRotation(double dt, TransMtx &mtx) const;
      //     
      //       [] 
      //       []
    double Sa(double Smean, double dSmean) 
      {return Smean + dSmean;}
  };
///////////////////////////////////////////////////////////////////////////////
//        
///////////////////////////////////////////////////////////////////////////////
  // 
void SetAstroBase(IAstroBase &base); 
  //  
IAstroBase *GetAstroBase();
///////////////////////////////////////////////////////////////////////////////
#endif
