///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IMEASUREMENT_H
#define BNOTOOL_IMEASUREMENT_H
///////////////////////////////////////////////////////////////////////////////
#include "ITraject.h"
#include <string>

///////////////////////////////////////////////////////////////////////////////
//  .
///////////////////////////////////////////////////////////////////////////////
//  .
#define mtUnknown 0
// 6-      
//   .
#define mtOrt 1
//  .
#define mtOptic 2
//      .
#define mtKApriori 3
//  .
#define mtRLocation 4
//   
#define mtRTKama 5
//      
#define mtRTKamaDt 6
//    TLE
#define mtTLE 7
//      
#define mtUser 1000
///////////////////////////////////////////////////////////////////////////////
//    
//        
///////////////////////////////////////////////////////////////////////////////
#include "ObjectId.h"
#include "MeasId.h"
#include "SiteId.h"

class IMeasurement
{
public:
	//   
	IMeasurement(); 
	// .
	virtual ~IMeasurement() 
	{};
	//     
	virtual unsigned GetSize() const = 0;
	//    
	virtual int GetType() const
	{ return Type;};
	//    
	virtual const TimeJD &GetTime() const
	{ return MTime;};
	//     v  
	virtual void GetRealVector(DVector& v) const = 0 ;
	//     v   
	virtual void GetMSEVector(DVector& v) const = 0 ;
	//    
	virtual TMeasId GetID() const
	{ return Id;};
	//     ,
	//    .
	//   0,    .
	virtual TSiteId GetStationID() const
	{ return StId; };

	virtual TNightId GetNightID() const
	{ return NightId; };

	//     ,
	//    .
	//   0,       .
	virtual TObjectId GetObjectID() const
	{ return ObjId;};
	virtual TPacketId GetPacketID() const
	{ return PacketId;};
	//    
	virtual void SetTime(const TimeJD &t)
	{ MTime = t; };
	//    
	virtual void SetID(TMeasId id) 
	{ Id = id; };
	//     ,
	//    
	virtual void SetStationID(TSiteId id) 
	{ StId = id;};
	virtual void SetNightID(TNightId id) 
	{ NightId = id;};
	//     ,
	//    
	virtual void SetObjectID(TObjectId id)
	{ ObjId = id;};
	//     p   ,
	virtual void SetPacketID(TPacketId id)
	{ PacketId = id;};
	//     p   ,
	//     
	virtual void GetWeightMatrix(DSymmMatrix &p) const;
	//       ,
	//       trj.
	//    :
	// t	-	,     
	//           (    ,
	//          )
	// fr	-	 ,       
	//        ,      (.   f  m)
	// v	-	   .     0,
	//          
	// m	-	   Nx6,  N -   
	//      (..    N   6 ).   
	//        Rx Ry Rz Vx  Vy Vz,    -   .
	//          0,     
	// del	-	   .     0,
	//           
	virtual bool Simulate(int matrixSize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f = 0,
		DVector *v = 0, DMatrix *m = 0, DVector *del = 0) const = 0 ;

	//  m1  m2 == NULL,   
	virtual void Diff(DVector &d, const DVector *m1 = NULL, const DVector *m2 = NULL) const = 0;

	//      new()
	virtual IMeasurement* Clone() const = 0;

	virtual bool GetIan(unsigned k) const
	{ return false;};
	//   
	virtual void SetIan(unsigned k, bool f = true)
	{ return;};

	bool IsRearranged() const {return m_rearranged;}
	bool SetRearranged(bool value) {return m_rearranged = value;}
	bool IsDroppable() const {return m_droppable;}
	bool SetDroppable(bool value) {return m_droppable = value;}

	int counter;

	bool m_skip;
protected:
	//  
	TMeasId Id;
	//   
	TSiteId StId;
	//   
	TNightId NightId;
	//   ,
	//    
	TObjectId ObjId;
	
	TPacketId PacketId;

	int Type;

	//  
	TimeJD MTime;
	
	bool m_rearranged;
	bool m_droppable;
};
///////////////////////////////////////////////////////////////////////////////
// ,        
///////////////////////////////////////////////////////////////////////////////
class AMeasurement : public IMeasurement
{
public:
	//   
	AMeasurement();
	// .
	virtual ~AMeasurement(); 
	//    
	virtual const IFrame *GetSrcFrame() const = 0;
	//       ,
	//       trj.
	//    :
	// t	-	,     
	//           (    ,
	//          )
	// fr	-	 ,       
	//        ,      (.   f  m)
	// v	-	   .     0,
	//          
	// m	-	   Nx6,  N -   
	//      (..    N   6 ).   
	//        Rx Ry Rz Vx  Vy Vz,    -   .
	//          0,     
	// del	-	   .     0,
	//           
	Vect3 GetRecPos() const {return RecPos;}
	const FrameConverter6D& GetFrameConverter() const {return FrameConverter;}
protected:
	//    (    
	//  simulate()
//	mutable IFrame const *WrkFrame;
	//        
	mutable FrameConverter6D FrameConverter;
	//    t,     
	//   ,          
	virtual bool DetectPosition(ITraject &trj, TimeJD &t, const IFrame *fr = 0, Vect6 *f = 0) const = 0;
	//     
	void SetWrkFrame(const IFrame &fr) const;

	Vect3 RecPos;

private:

};
///////////////////////////////////////////////////////////////////////////////
#endif

