#include "IMinMnk.h"
///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
#include "IMinMnk.h"
#include "Defines.h"
#include <math.h>
#include <stdio.h>
#include <iostream>
#include "Lagrange.h"
//#include "LagrTrajectVarM.h"
#include "KepData.h"

///////////////////////////////////////////////////////////////////////////////
// ,     
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
DMinMnk::DMinMnk()
{
	MaxIt = 18;
	MaxItD = 6;
	MaxN05 = 6;
	Cn1 = 500.0;
	Cn2 = 1000.0;
	AlphaS = 0.1;
	RegPrint = NONE;

	Reset();
}
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
DMinMnk::~DMinMnk()
{
}
//-------------------------------------------------------------
// :    
//    
//-------------------------------------------------------------
void DMinMnk::Reset()
{
	IsMin = false;
	Path = UNDEFINED_PATH;
	IsDesc = true;
	Nit = NitD = 0;
}
//-------------------------------------------------------------
//  
// u0 -  
// f -  
//-------------------------------------------------------------
void DMinMnk::Minimize(const DVector &u0, IMinFunct &fc)
{
	Reset();

	//    
	unsigned n = u0.Dim();
	TAssert(n == fc.Size());

	//  
	// 
	DVector df(n), du(n);
	// :   
	DSymmMatrix fm(n);

	U = u0;
	Sm.Init(n);

	if (!fc.Fun(U, F0, &df, &fm) || !InvertMatrix(fm, Sm))
	{
		//  -   
		if (TRACE <= RegPrint)
			printf("   \n");
		Res = FUNC_RES;
		return;
	}

	Fk = F0;

	while (Nit < MaxIt)
	{
		//  
		if (TRACE <= RegPrint)
		{
			printf(": ");
			PrintVector(U);
			printf("\n>>> : %d  %10.4e\n", Nit, Fk);
		}

		if (Path == DESC_PATH)
		{
			//  
			IsDesc = Descent(fc, Fk, U, df, fm, Sm);
			if (IsDesc)
			{
				Nit++;
				NitD++;
				Path = UNDEFINED_PATH;
				continue;
			}
			Res = STOP_RES;
			if (TRACE <= RegPrint)
				printf(">>>  \n");
			break;
		}
		else
		{
			//   -
			unsigned i, j;
			double kalpha = 1.0;;

			for (i = 0; i < n; i++)
			{
				du[i] = 0.0;
				for (j = 0; j < n; j++)
					du[i] += Sm(i, j) * df[j];
				}
					
			//   
			double dun = 0.0;
			for (i = 0; i < n; i++)
				dun += du[i] * du[i] / Sm(i, i);

			dun = sqrt(dun);

			if (dun > Cn1)
			{
				kalpha = Cn1 / dun;
				if (kalpha < 0.3)
					kalpha = 0.3;
				for (i = 0; i < n; i++)
					du[i] = kalpha * du[i];
				if (TRACE <= RegPrint)
					printf("   K=%10.4e\n", kalpha);
			}

			if (Path == UNDEFINED_PATH && NitD < MaxItD && dun > Cn2)
			{
				//  
				if (TRACE <= RegPrint)
					printf("  \n");
				IsDesc = Descent(fc, Fk, U, df, fm, Sm);
				Nit++;
				NitD++;
				if (IsDesc)
				{
					Path = UNDEFINED_PATH;
					continue;
				}
				Path = GN_PATH;
				continue;
			}
			else
			{
				//  -
				if(!GNStep(fc, Fk, U, df, fm, Sm, du)){
					Res = FAIL_RES;
					break;
				}
				if (dun > Cn1)
					IsMin = false;
				if (IsMin && Path != DESC_PATH)
				{
					if (TRACE <= RegPrint)
					{
						printf(": ");
						PrintVector(U);
						printf(">>>  : %d  %10.4e\n", Nit, Fk);
					}
					Res = SUCCESS_RES;
					break;
				}
				Nit++;
			}
		}
	}

	if (Nit == MaxIt)
	{
		Res = NITER_RES;
		if (TRACE <= RegPrint)
			printf(">>>   \n");
	}

	if ((Res == NITER_RES || Res == STOP_RES) && fc.CanBeAccepted(U) || Res == NITER_RES && AlphaS < 0)
	{
		if (TRACE <= RegPrint)
		{
			printf(": ");
			PrintVector(U);
			printf(">>>  : %d  %10.4e\n", Nit, Fk);
		}
		Res = SUCCESS_RES;
	}
}
//-------------------------------------------------------------
//  TA 
//-------------------------------------------------------------
void DMinMnk::Parabola(double f0, double f1, double fp0, double fp1, double kc0,
					   double &umin, double &fmin)
{
	if (fp0 > 0)
	{
		umin = 0.0;
		fmin = f0;
		return;
	}

	double z = fp0 + fp1 + 3.0 * (f0 - f1);
	double d = 1.0 - fp0 * fp1 / (z * z);

	if (d < 0.0)
	{
		umin = 0.0;
		fmin = f0;
		return;
	}

	double a2 = -z - fp0;
	double a3 = (2.0 * z + fp0 + fp1) / 3.0;

	umin = (fabs(z) * sqrt(d) - a2) / (3.0 * a3);
	fmin = f0 + umin * (fp0 + umin * (a2 + umin * a3));

	if (fmin > kc0 * f0)
	{
		umin = 0.0;
		fmin = f0;
	}
}
//-------------------------------------------------------------
//    TA 
//-------------------------------------------------------------
bool DMinMnk::DescOneStep(IMinFunct &fc, double &f, DVector &u, DVector &df,
						  DSymmMatrix &fm, DSymmMatrix &s)
{
	unsigned n, i;
	double tres = 0.95;

	n = fc.Size();

	DVector dfn(n);
	DSymmMatrix sm(n);

	for (i = 0; i < n; i++)
		dfn[i] = df[i] * s(i, i);

	double a = 0.0;
	for (i = 0; i < n; i++)
		a += dfn[i] * df[i];

	double b = VtMSV(dfn, fm);
	a /= b;

	DVector du(n), u1(n), df1(n);
	DSymmMatrix fm1(n);
	double f1;
	for (i = 0; i < n; i++)
		du[i] = a * dfn[i];

	fc.AddU(u, du, u1);

	if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, sm))
	{
		if (TRACE <= RegPrint)
			printf("   \n");
		return false;
	}

	if (f1 < tres * f)
	{
		f = f1;
		u = u1;
		df = df1;
		fm = fm1;
		s = sm;
		if (TRACE <= RegPrint)
		{
			printf(" \n");
			printf(" %10.4e\n", f);
			printf("  \n");
			PrintVector(du);
		}
		return true;
	}

	///////////////////////////////////////////////////////////////////////////////////
	if (f1 < f)
	{
		DVector dfn1(n);

		double w = 0.0, w1 = 0.0;
		for (i = 0; i < n; i++)
		{
			dfn1[i] = df1[i] * sm(i, i);
			w += dfn[i] * dfn[i];
			w1 += dfn1[i] * dfn1[i];
		}

		for (i = 0; i < n; i++)
			dfn1[i] += w1 / w * dfn[i];

		a = 0.0;
		for (i = 0; i < n; i++)
			a += dfn1[i] * df1[i];

		b = VtMSV(dfn1, fm1);
		a /= b;

		for (i = 0; i < n; i++)
			du[i] = a * dfn1[i];

		u = u1;
		f = f1;
		fc.AddU(u, du, u1);

		if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, sm))
		{
			if (TRACE <= RegPrint)
				printf("   \n");
			return false;
		}
		if (f1 < tres * f)
		{
			f = f1;
			u = u1;
			df = df1;
			fm = fm1;
			s = sm;
			if (TRACE <= RegPrint)
			{
				printf("    . \n");
				printf(" %10.4e\n", f);
				printf("  \n");
				PrintVector(du);
			}
			return true;
		}
	}
	///////////////////////////////////////////////////////////////////////////////////

	if (TRACE <= RegPrint)
		printf(" ,  \n");

	//    
	double fp0 = 0.0, fp1 = 0.0;
	for (i = 0; i < n; i++)
	{
		fp0 -= 2.0 * du[i] * df[i];
		fp1 -= 2.0 * du[i] * df1[i];
	}

	double emin, fmin;
	Parabola(f, f1, fp0, fp1, tres, emin, fmin);

	if (emin == 0)
	{
		if (TRACE <= RegPrint)
			printf("   \n");
		return false;
	}

	//      
	for (i = 0; i < n; i++)
		du[i] *= emin;

	fc.AddU(u, du, u1);
	if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, sm))
	{
		if (TRACE <= RegPrint)
			printf("   \n");
		return false;
	}

	if (f1 < tres * f)
	{
		f = f1;
		u = u1;
		df = df1;
		fm = fm1;
		s = sm;
		if (TRACE <= RegPrint)
		{
			printf("   \n");
			printf(" %10.4e\n", f);
			printf("  \n");
			PrintVector(du);
		}
		return true;
	}


	fp1 = 0.0;
	for (i = 0; i < n; i++)
		fp1 -= 2.0 * du[i] * df1[i];

	Parabola(f, f1, fp0, fp1, tres, emin, fmin);

	if (emin == 0)
	{
		if (TRACE <= RegPrint)
			printf("    \n");
		return false;
	}

	for (i = 0; i < n; i++)
		du[i] *= emin;

	fc.AddU(u, du, u1);
	if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, sm) || f1 > tres * f)
	{
		if (TRACE <= RegPrint)
			printf("    \n");
		return false;
	}

	f = f1;
	u = u1;
	df = df1;
	fm = fm1;
	s = sm;

	if (TRACE <= RegPrint)
	{
		printf("      \n");
		printf(" %10.4e\n", f);
		printf("  \n");
		PrintVector(du);
	}

	return true;
}
//-------------------------------------------------------------
//   
//-------------------------------------------------------------
bool DMinMnk::Descent(IMinFunct &fc, double &f, DVector &u, DVector &df,
					  DSymmMatrix &fm, DSymmMatrix &s)
{
	if (!DescOneStep(fc, f, u, df, fm, s))
		return false;
	if (!DescOneStep(fc, f, u, df, fm, s))
		return false;
	return true;
}

//-------------------------------------------------------------
//    -
//-------------------------------------------------------------
bool DMinMnk::GNStep(IMinFunct &fc, double &f, DVector &u, DVector &df,
					 DSymmMatrix &fm, DSymmMatrix &s, const DVector &du)
{
	Path = UNDEFINED_PATH;
	double kn = (f > 1.e6) ? 0.8 : 0.999;
	unsigned n05 = 0;

	unsigned n = fc.Size(), i;
	DVector du1(n), u1(n), df1(n), dus(n);
	DSymmMatrix fm1(n), sm1(n);
	double f1;

	du1 = du;
//	PrintVector(du1);

	//  
	for (;;)
	{
		if (n05 == MaxN05)
		{
			Path = DESC_PATH;
			IsMin = false;
			return true;
		}

		unsigned npar = 0;
		fc.AddU(u, du1, u1);
				
		if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, sm1))
		{
			if (TRACE <= RegPrint)
			{
				printf("   \n");
				printf("  \n");
				printf("  \n");
				PrintVector(du1);
			}
			for (i = 0; i < n; i++)
				du1[i] *= 0.5;

			n05++;
			continue;
		}
		if (npar == 0 && n05 == 0)
			IsMin = fc.IsMin(u, f, u1, f1, sm1, AlphaS, Nit, Path);
		else
			IsMin = false;
		if (IsMin || f1 < kn * f)
		{
			if (Path != DESC_PATH)
			{
				u = u1;
				df = df1;
				f = f1;
				fm = fm1;
				s = sm1;
			}
			if (TRACE <= RegPrint)
			{
				printf("  -\n");
				printf("  \n");
				PrintVector(du1);
			}
			return true;
		}
		dus = du1;
		if (TRACE <= RegPrint)
			printf("  \n");
		while (npar < 3)
		{
			double fp0 = 0.0, fp1 = 0.0;
			for (i = 0; i < n; i++)
			{
				fp0 -= 2.0 * du1[i] * df[i];
				fp1 -= 2.0 * du1[i] * df1[i];
			}
			double kmin, fmin;
			Parabola(f, f1, fp0, fp1, 0.95, kmin, fmin);
			if (kmin < 0.02)
			{
				if (TRACE <= RegPrint)
					printf("  -  \n");
				break;
			}
			for (i = 0; i < n; i++)
				du1[i] = kmin * du1[i];
			
			fc.AddU(u, du1, u1);
			
			if (!fc.Fun(u1, f1, &df1, &fm1) || !InvertMatrix(fm1, s))
			{
				if (TRACE <= RegPrint)
					printf("   \n");

				return false;
				//break;
			}

			if (f1 < f)
			{
				u = u1;
				df = df1;
				f = f1;
				fm = fm1;
				if (TRACE <= RegPrint)
				{
					printf("  -   \n");
					printf("  \n");
					PrintVector(du1);
				}
				return true;
			}
			npar++;
		}
		if (f1 < f)
		{
			u = u1;
			df = df1;
			f = f1;
			fm = fm1;
			if (TRACE <= RegPrint)
			{
				printf("  -    \n");
				printf("  \n");
				PrintVector(du1);
			}
			return true;
		}
		for (i = 0; i < n; i++)
			du1[i] = dus[i] * 0.5;
		n05++;
		if (TRACE <= RegPrint)
			printf("  \n");
	}
}
//-------------------------------------------------------------
//    Vt*S*V,  V-,
// S -  
//-------------------------------------------------------------
double DMinMnk::VtMSV(const DVector &v, const DSymmMatrix &s)
{
	TAssert(v.Dim() == s.Dim());

	unsigned n = v.Dim();

	double *p_mt3= new double[n];

	double element;

	register unsigned j, k;

	for ( j = 0; j < n; j++)
	{
		element = 0.0;
		for (k = 0; k < n; k++)
			element += v[k] * s(j, k);
		p_mt3[j] = element;
	}

	element=0.0;
	for (k = 0; k < n; k++)
		element += p_mt3[k] * v[k];

	delete[] p_mt3;

	return element;
}
//-------------------------------------------------------------
//  
//-------------------------------------------------------------
bool DMinMnk::InvertMatrix(const DSymmMatrix &fm, DSymmMatrix &s)
{
	unsigned i, n = fm.Dim(), k;

	//   
	//     . 
	for (i = 0; i < n; i++)
	{
		if (fm(i, i) <= 0)
			return false;
	}

	for (k = 0; k < 6; k++)
	{
		double r;
		if (k == 0)
			r = 0.0;
		else
			r = 1 / pow(10.0,(double)(7 - k));
		if (!InvertByReg(fm, r, s))
			continue;
		for (i = 0; i < n; i++)
		{
			if (s(i, i) <= 0.0)
				break;
		}
		if (i == n)
			return true;
	}

	return false;

}
//-------------------------------------------------------------
//       
//-------------------------------------------------------------
bool DMinMnk::InvertByReg(const DSymmMatrix &fm, double r, DSymmMatrix &s)
{
	unsigned n = fm.Dim(), i, j, k;

	DSymmMatrix a0(n), a1(n);
	DVector z(n);

	a0 = fm;

	if (r > 0)
	{
		for (i = 0; i < n; i++)
			a0(i, i) = a0(i, i) * (1 + r);
	}

	for (i = 0; i < n; i++)
		z[i] = 1.0 / sqrt(a0(i, i));

	for (i = 0; i < n; i++)
	{
		for (j = i; j < n; j++)
		{
			double a = (i == j) ? 1.0 : a0(i, j) * z[i] * z[j];
			if (fabs(a) > 1.0)
				return false;
			a0(i, j) = a;
		}
	}

	for (k = 0; k < n; k++)
	{
		for (i = 0; i < n - 1; i++)
			a1(i, n - 1) = -a0(0, i + 1) / a0(0, 0);
		for (i = 0; i < n - 1; i++)
		{
			for (j = i; j < n - 1; j++)
				a1(i, j) = a0(i + 1, j + 1) - a0(0, i + 1) * a0(0, j + 1) / a0(0, 0);
		}
		a1(n - 1, n - 1) = -1.0 / a0(0, 0);
		a0 = a1;
	}

	for (i = 0; i < n; i++)
	{
		for (j = 0; j < n; j++)
			s(i, j) = -a0(i, j) * z[i] * z[j];
	}

	return true;
}
//-------------------------------------------------------------
//   
//-------------------------------------------------------------
void DMinMnk::PrintVector(const DVector &u) const
{
	unsigned j;
	for (j = 0; j < u.Dim(); j++)
		printf(" %14.8e", u[j]);
	printf("\n");
}
///////////////////////////////////////////////////////////////

