///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
#ifndef IMINMNK_H
#define IMINMNK_H
///////////////////////////////////////////////////////////////////////////////
//     
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
typedef enum
{
	UNDEFINED_PATH = 0, //   
	DESC_PATH = 1,      //  
	GN_PATH = 2         //    -
} EnMinMnkPath;
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
typedef enum
{
	NONE = -1, //  
	TRACE = 1, //  :    
} EnMinMnkPrintReg;
///////////////////////////////////////////////////////////////////////////////
//    ,  
// 
///////////////////////////////////////////////////////////////////////////////
class IMinFunct
{
public:
	// 
	virtual ~IMinFunct()
	{};

	//    
	virtual unsigned Size() const = 0;
	//    ,  
	//   
	//   false,    
	//     
	virtual bool Fun(const DVector &u, double &f, DVector *df = NULL,
		DSymmMatrix *fm = NULL) = 0;
	//    
	virtual bool CanBeAccepted(const DVector &u) = 0;
	//      
	virtual void AddU(const DVector &u0, const DVector &du,
		DVector &u) const = 0;
	//    
	//  true,  
	virtual bool IsMin(const DVector &u, double f,
		const DVector &u1, double f1,
		const DSymmMatrix &s, double alpha, unsigned nit,
		EnMinMnkPath &path) = 0;
};
///////////////////////////////////////////////////////////////////////////////
// ,     
///////////////////////////////////////////////////////////////////////////////
class DMinMnk
{
public:
	//  /
	typedef enum
	{
		SUCCESS_RES = 0,    //  
		NITER_RES = 1,      //     
		STOP_RES = 2,       //   
		FUNC_RES = 3,       //    
		BAD_IANS = 4,
		FAIL_RES = 5
	} EnRes;


	// 
	DMinMnk();
	// 
	virtual ~DMinMnk();

	// :    
	//    
	void Reset();

	//      
	//    
	unsigned GetMaxIter() const
	{ return MaxIt;};
	//    
	void SetMaxIter(unsigned n)
	{ MaxIt = n;};
	//     
	unsigned GetMaxIterD() const
	{ return MaxItD;};
	//     
	void SetMaxIterD(unsigned n)
	{ MaxItD = n;};
	//      
	unsigned GetMaxN05() const
	{ return MaxN05;};
	//      
	void SetMaxN05(unsigned n)
	{ MaxN05 = n;};
	//     
	double GetCn1() const
	{ return Cn1;};
	//     
	void SetCn1(double c)
	{ Cn1 = c;};
	//     
	double GetCn2() const
	{ return Cn2;};
	//     
	void SetCn2(double c)
	{ Cn2 = c;};
	//   
	double GetAlphaS() const
	{ return AlphaS;};
	//     
	void SetAlphaS(double a)
	{ AlphaS = a;};

	//  
	// u0 -  
	// f -  
	void Minimize(const DVector &u0, IMinFunct &f);

	//    
	//    
	EnRes GetRes() const
	{ return Res;};
	//   
	const DVector &GetVector() const
	{ return U;};
	//   
	const double GetFk() const
	{ return Fk;};
	//   
	const DSymmMatrix &GetSMatrix() const
	{ return Sm;};
	//  
	const unsigned GetNit() const
	{ return Nit;};

	//    
	EnMinMnkPrintReg RegPrint;

protected:
	//  
	//    
	unsigned MaxIt;
	//     
	unsigned MaxItD;
	//      
	unsigned MaxN05;
	//  
	double Cn1, Cn2;
	//  
	double AlphaS;

	//  
	//   
	bool IsMin;
	//   
	EnMinMnkPath Path;
	//   
	bool IsDesc;
	//   
	unsigned Nit;
	//   
	unsigned NitD;
	//   
	EnRes Res;
	//     
	double F0, Fk;
	//  
	DSymmMatrix Sm;
	//    
	DVector U;
		
	//  
	//  
	bool InvertMatrix(const DSymmMatrix &fm, DSymmMatrix &s);
	//       
	bool InvertByReg(const DSymmMatrix &fm, double r, DSymmMatrix &s);
	//  TA 
	void Parabola(double f0, double f1, double fp0, double fp1, double kc0,
		double &umin, double &fmin);
	//    TA 
	bool DescOneStep(IMinFunct &fc, double &f, DVector &u, DVector &df,
		DSymmMatrix &fm, DSymmMatrix &s);
	//   
	bool Descent(IMinFunct &fc, double &f, DVector &u, DVector &df,
		DSymmMatrix &fm, DSymmMatrix &s);
	//    -
	bool GNStep(IMinFunct &fc, double &f, DVector &u, DVector &df,
		DSymmMatrix &fm, DSymmMatrix &s, const DVector &du);
	//    Vt*S*V,  V-,
	// S -  
	double VtMSV(const DVector &v, const DSymmMatrix &s);
	//   
	void PrintVector(const DVector &u) const;
};
///////////////////////////////////////////////////////////////////////////////
#endif

