///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IMODELS_H
#define BNOTOOL_IMODELS_H
///////////////////////////////////////////////////////////////////////////////
#include "TM.h"
#include "DVectors.h"
#include "Ids.h"
///////////////////////////////////////////////////////////////////////////////
//    ,     
//  
class IEphemeride
{
public:
    //     
    virtual ~IEphemeride()
    {};
    //     
    virtual bool LoadData(const TimeJD &s, const TimeJD &e) = 0;
    //  
    virtual void UnLoadData() = 0;
    //   .
    virtual void GetPosition(const TimeJD &t, Vect3 *pos = NULL,
        Vect3 *vel = NULL, Vect3 *acc = NULL) const = 0;
    //       
    virtual TCoordSysId GetCS() const = 0;
};
///////////////////////////////////////////////////////////////////////////////
//      
class TGeometryModel
{
public:
    double Re;    //  
    double alpha; // 
};
///////////////////////////////////////////////////////////////////////////////
//   
class IAtmosphereModel
{
public:
    //  .      
    virtual ~IAtmosphereModel()
    {};
    //  .      
    //  
    virtual double Density(const TimeJD &t, const Vect3 &v, const Vect3 &vSun)
        const = 0;
    //  
    virtual TAtmosphereId GetID() const = 0;
    //       ?
    virtual bool IsSunPosNeed() const = 0;
    //        ?
    virtual bool IsTableNeed() const = 0;
};
///////////////////////////////////////////////////////////////////////////////
//  ,    
class TGeoSolarFlux
{
public:
    double f135;   //      135   -  77
    double f90;    //      90    - ???
    double f81;    //      81    - ???
    double f135w;  //      135 
    double f81w;   //      81  -  90
    double f01;    //      1      -  
    double kp;    //      (   )
    int    ap;    //     
    double three_hours_kp[8]; //  
    bool   iskp3;
};
///////////////////////////////////////////////////////////////////////////////
//        
class ITableGeoSolarFlux
{
public:
    //     
    virtual ~ITableGeoSolarFlux()
    {};
    //     
    virtual const TGeoSolarFlux *Get_F107(const TimeJD &tm) const = 0;
    //      
    virtual const double *Get_Kp3hour(const TimeJD &tm) const = 0;
    //    
    virtual double Get_Kp3hourMod(const TimeJD &tm) const = 0;
};
///////////////////////////////////////////////////////////////////////////////
//   
class IGravityModel
{
public:
    //   
    virtual ~IGravityModel()
    {};
    //   
    virtual double c(unsigned int n, unsigned int m) const = 0;
    virtual double d(unsigned int n, unsigned int m) const = 0;
    //   
    virtual unsigned int N() const = 0;
    virtual unsigned int M() const = 0;
    //  
    virtual TGravityModelId GetID() const = 0;
    virtual const double* const* get_cc() const { return 0; }
    virtual const double* const* get_dd() const { return 0; }

    double mu;  //  
    double Re;  //  
    double alpha; //   
};
///////////////////////////////////////////////////////////////////////////////
//       
class IPlanet
{
public:
    //   
    IPlanet()
    {};
    //
    virtual ~IPlanet()
    {};

    //  
    virtual const char *Name() const = 0;
    //  
    virtual TPlanetId Id() const = 0;
    //    
    virtual const IGravityModel *GetGravityModel() const = 0;
    //    
    virtual const IAtmosphereModel *GetAtmosphere() const = 0;
    //   
    virtual IEphemeride *GetEphemeride() = 0;
    virtual const IEphemeride *GetEphemeride() const = 0;

    double omega; //   
    double RShad; //  
    TGeometryModel Geometry;
};
///////////////////////////////////////////////////////////////////////////////
#endif
