///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//        
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ITRAJECT_H
#define BNOTOOL_ITRAJECT_H
///////////////////////////////////////////////////////////////////////////////
#include "TM.h"
#include "Matrix.h"
#include "DVectors.h"
#include "IFrames.h"
#include "IPredict.h"

///////////////////////////////////////////////////////////////////////////////
//         
///////////////////////////////////////////////////////////////////////////////
class ITraject
{
public:
	ITraject():m_cache(false){}
	// .
	virtual ~ITraject() 
	{};
	//      
	virtual unsigned Size() const = 0;
	//      
	virtual bool Init(const DVector &u0) = 0 ;
	//      
	virtual void SetTime(const TimeJD& t) = 0 ;
	//      
	virtual const TimeJD& GetTime() const = 0 ;
	//         t. 
	//  fr   ,      
	//   f (..   )
	virtual bool Predict(const TimeJD &t, const IFrame &fr, Vect6 &f) = 0 ;
	virtual bool PredictA(const TimeJD &t, Vect6 &f){return 0;};
	virtual bool PredictMatrix(const TimeJD &t, const IFrame& fr, Vect6 &f, DMatrix &m) = 0 ;
	//       u   
	//   t.
	virtual bool Move(const TimeJD &t, DVector& u) = 0 ;

//	virtual IPredict* GetPredictorEx() {return 0;}
	virtual IPredict* GetPredictor() = 0;

	bool PredictEx( const TimeJD &t, Vect6 &f )
	{
		if (m_cache && m_cacheTime == t){
			f = m_cachePos;
			return true;
		}

		IPredict* predict = GetPredictor();
		if (!predict){return false;}

		if (!predict->Run(t)){
			return false;
		}
		
		predict->GetDestPointEX(f);

		m_cacheTime = t;
		m_cache = true;
		m_cachePos = f;

		return true;
	}

private:
	TimeJD m_cacheTime;
	Vect6 m_cachePos;
	bool m_cache;
};
///////////////////////////////////////////////////////////////////////////////
//         
//        
///////////////////////////////////////////////////////////////////////////////
class ITrajectVar : public ITraject
{
public:
	//         t 
	//         .
	//  fr   ,     
	//   f (..   )   m.
	virtual bool PredictMatrix(const TimeJD &t, const IFrame& fr, Vect6 &f, DMatrix &m) = 0 ;
	//      (  )
//	virtual IPredictVar *GetPredictor() = 0;
};
///////////////////////////////////////////////////////////////////////////////
#endif
