///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//   ""
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_KAMAMEAS_H
#define BNOTOOL_KAMAMEAS_H
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"
///////////////////////////////////////////////////////////////////////////////
//   ""
///////////////////////////////////////////////////////////////////////////////
class KAMAMeasurement : public IMeasurement
  {
  public:
      //   
    KAMAMeasurement(); 
      // .
    virtual ~KAMAMeasurement() 
      {};
      //     
    virtual unsigned GetSize() const
      { return 1;};
      //     v  
    virtual void GetRealVector(DVector& v) const
      { v.Init(1U); v(0) = D;};
      //     v   
    virtual void GetMSEVector(DVector& v) const
      { v.Init(1U); v(0) = SigD;};
      //       ,
      //       trj.
      //    :
      // t	-	,     
      //           (    ,
      //          )
      // fr	-	 ,       
      //        ,      (.   f  m)
      // v	-	   .     0,
      //          
      // m	-	   Nx6,  N -   
      //      (..    N   6 ).   
      //        Rx Ry Rz Vx  Vy Vz,    -   .
      //          0,     
      // del	-	   .     0,
      //           
    virtual bool Simulate(int matrixSize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f = 0,
                          DVector *v = 0, DMatrix *m = 0, DVector *del = 0) const;
      //      new()
      //  m1  m2 == NULL,   
    virtual void Diff(DVector &d, const DVector *m1 = NULL, const DVector *m2 = NULL) const;
      //      new()
    virtual IMeasurement* Clone() const;

      //   
    void Init(double d, double sigd, const Vect3 &psend, const Vect3 &prec)
      { D = d; SigD = sigd; PSend = psend; PRec = prec; };

	virtual void SetIan(unsigned k, bool f = true)
	{ m_isBad = f;};

	virtual bool GetIan(unsigned k) const
	{ return m_isBad;};

  protected:
      //  
    double D;
      //  
    double SigD;
      //     
    Vect3 PSend;
      //     
    Vect3 PRec;

	bool m_isBad;
  };
///////////////////////////////////////////////////////////////////////////////
//   Ddot
///////////////////////////////////////////////////////////////////////////////
class KAMADtMeasurement : public IMeasurement
  {
  public:
      //   
    KAMADtMeasurement() : IMeasurement()
      { Type = mtRTKamaDt;}; 
      // .
    virtual ~KAMADtMeasurement() 
      {};
      //     
    virtual unsigned GetSize() const
      { return 1;};
      //     v  
    virtual void GetRealVector(DVector& v) const
      { v.Init(1U); v(0) = Dt;};
      //     v   
    virtual void GetMSEVector(DVector& v) const
      { v.Init(1U); v(0) = SigDt;};
      //       ,
      //       trj.
      //    :
      // t	-	,     
      //           (    ,
      //          )
      // fr	-	 ,       
      //        ,      (.   f  m)
      // v	-	   .     0,
      //          
      // m	-	   Nx6,  N -   
      //      (..    N   6 ).   
      //        Rx Ry Rz Vx  Vy Vz,    -   .
      //          0,     
      // del	-	   .     0,
      //           
    virtual bool Simulate(int matrixSize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f = 0,
                          DVector *v = 0, DMatrix *m = 0, DVector *del = 0) const;
      //      new()
      //  m1  m2 == NULL,   
    virtual void Diff(DVector &d, const DVector *m1 = NULL, const DVector *m2 = NULL) const;
      //      new()
      //      new()
    virtual IMeasurement* Clone() const;

      //   
    void Init(double dt, double sigdt, const Vect3 &psend, const Vect3 &prec)
      { Dt = dt; SigDt = sigdt; PSend = psend; PRec = prec; };

  protected:
      //    
    double Dt;
      //    
    double SigDt;
      //     
    Vect3 PSend;
      //     
    Vect3 PRec;
  };
///////////////////////////////////////////////////////////////////////////////
#endif

