///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_LAGRTRAJECTVAR_H
#define BNOTOOL_LAGRTRAJECTVAR_H
///////////////////////////////////////////////////////////////////////////////
#include "IMinMnkTr.h"
#include "TrjNumVar.h"
#include "Lagrange.h"
///////////////////////////////////////////////////////////////////////////////
class LagrTrajectVar : public IMinTrFun
  {
  public:
      // 
    LagrTrajectVar();
      // 
    virtual ~LagrTrajectVar();

      //  
    void SetFlags(const bool *f);
      //  
    const bool *GetFlags() const
      { return Flags;};
      //     
    void SetParams(const TimeJD &t, const Lagrange &lagr, double kb, double kappa);
      //      
    void GetParams(Lagrange &lagr, double &kb, double &kappa, const DVector &u) const;

      //     
    virtual unsigned Size() const;
      //      
    virtual bool Init(const DVector &u0);
      //      
    virtual void SetTime(const TimeJD& t)
      { GetTrj().SetTime(t); };
      //      
    virtual const TimeJD& GetTime() const
      { return GetTrj().GetTime();}
      //         t. 
      //  fr   ,      
      //   f (..   )
    virtual bool Predict(const TimeJD &t, const IFrame &fr, Vect6 &f)
      { return GetTrj().Predict(t, fr, f); };
      //       u   
      //   t.
    virtual bool Move(const TimeJD &t, DVector& u);
      //         t 
      //         .
      //  fr   ,     
      //   f (..   )   m.
    virtual bool PredictMatrix(const TimeJD &t, const IFrame& fr, Vect6 &f, DMatrix &m);
      //    
    virtual bool CanBeAccepted(const DVector &u);
      //    
      //  true,  
    virtual bool IsMin(const DVector &u, double f,
                       const DVector &u1, double f1,
                       const DSymmMatrix &s, double alpha, unsigned nit,
                       EnMinMnkPath &path);
      //      
    virtual void AddU(const DVector &u0, const DVector &du,
                      DVector &u) const;
      //   
    virtual ITrajectVar &GetTrj()
      { return Trj;};
	virtual const ITrajectVar &GetTrj() const
	{ return Trj;};
      //      (  )
    virtual IPredictVar *GetPredictor()
      { return (IPredictVar*)GetTrj().GetPredictor();};

	bool allowPoorQuality;

  protected:
      //   
    bool Flags[8];
      //  
    TrjNumVar Trj;
      //   ( )
    Lagrange Lagr;
      // . 
    double Kb;
      //  . 
    double Kappa;
      //     . 
    DSqrMatrix DrDl;
	
      //      (0-7),
      // k-   
    virtual unsigned LIndex(unsigned k) const;
      // ,       
    virtual void GetEps(DVector &eps) const;
      //     
    virtual void FormPDM(const DMatrix &mr, DMatrix &m);
      //   
    virtual bool IsValid(const DVector &u) const;

  };
///////////////////////////////////////////////////////////////////////////////
#endif
