///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//      
////////////////////////////////////////////////////////////////////////////////
#include "LocalRLFrame.h"
#include "GreenwichFrame.h"
#include "FGreenwichFrame.h"
#include "METEFrame.h"
#include "IAstroBase.h"
////////////////////////////////////////////////////////////////////////////////
//   
//------------------------------------------------------------------------------
void LocalRLFrame::SetParameters(const IFrameParameters &p)
  {
  TAssert(p.GetFrameType() == csIdLocalRL);
  const LocalRLFrameParameters &pm = (const LocalRLFrameParameters &)p;
  Pars = pm;
  }
//------------------------------------------------------------------------------
//  ,        
//    J2000    t0 ( 3- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get3DJ2000Converter(const TimeJD &t0, FrameConverter3D &result) const
  {
  TAssert(Pars.IsInit());
  FrameConverter3D lg, gj;

  Get3DGreenwichConverter(lg);

  GreenwichFrame g;
  g.Get3DJ2000Converter(t0, gj);
  lg.Multiply(gj, result);
  }
//------------------------------------------------------------------------------
//  ,        
//    J2000    t0 ( 6- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get6DJ2000Converter(const TimeJD &t0, FrameConverter6D &result) const
  {
  TAssert(Pars.IsInit());
  FrameConverter6D lg, gj;

  Get6DGreenwichConverter(lg);

  GreenwichFrame g;
  g.Get6DJ2000Converter(t0, gj);
  lg.Multiply(gj, result);
  }
//------------------------------------------------------------------------------
//  ,        
//    rf    t0 ( 3- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get3DConverter(const IFrame &rf, const TimeJD &t0, 
                          FrameConverter3D &result) const
  {
  TAssert(Pars.IsInit());

  if (rf.GetType() == csIdGFCS)
    {
    const FGreenwichFrame &gf = (const FGreenwichFrame &)rf;
    if (t0 == gf.GetFixTime())
      {
      Get3DGreenwichConverter(result);
      return;
      }
    }

  if (rf.GetType() == csIdGCS)
    {
    Get3DGreenwichConverter(result);
    return;
    }

  if (rf.GetType() == csIdMETE)
    {
    const METEFrame &mf = (const METEFrame &)rf;
    if (t0 == mf.GetFixTime())
      {
      FrameConverter3D lg, gm;
      Get3DGreenwichConverter(lg);
      gm.Init();
      GetAstroBase()->MtxMETE2GCS(t0, gm.Rotation);
      gm.Rotation.Transp();
      lg.Multiply(gm, result);
      }
    }

  IFrame::Get3DConverter(rf, t0, result);
  }
//------------------------------------------------------------------------------
//  ,        
//    rf    t0 ( 6- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get6DConverter(const IFrame &rf, const TimeJD &t0, 
                          FrameConverter6D &result) const
  {
  TAssert(Pars.IsInit());

  if (rf.GetType() == csIdGFCS)
    {
    const FGreenwichFrame &gf = (const FGreenwichFrame &)rf;
    if (t0 == gf.GetFixTime())
      {
      FrameConverter6D lg, gf;
      Get6DGreenwichConverter(lg);
      gf.Init();
      GetAstroBase()->GCSVelocityMatrix(gf.VRotation);
      gf.VRotation.Transp();
      lg.Multiply(gf, result);
      return;
      }
    }

  if (rf.GetType() == csIdGCS)
    {
    Get6DGreenwichConverter(result);
    return;
    }

  if (rf.GetType() == csIdMETE)
    {
    const METEFrame &mf = (const METEFrame &)rf;
    if (t0 == mf.GetFixTime())
      {
      FrameConverter6D lg, gf, gm;
      Get6DGreenwichConverter(lg);
      gf.Init();
      GetAstroBase()->GCSVelocityMatrix(gf.VRotation);
      gf.VRotation.Transp();
      lg.Multiply(gf);
      gm.Init();
      GetAstroBase()->MtxMETE2GCS(t0, gm.Converter3D.Rotation);
      gm.Converter3D.Rotation.Transp();
      lg.Multiply(gm, result);
      return;
      }
    }

  IFrame::Get6DConverter(rf, t0, result);
  }
//------------------------------------------------------------------------------
//  ,        
//      t0 ( 3- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get3DGreenwichConverter(FrameConverter3D &result) const 
  {
  TAssert(Pars.IsInit());

  GeoCoordsPrec g;
  double az;

  g = Pars.GetPoint();
  az = Pars.GetAz();

  double cf = cos(g.Phi);
  double sf = sin(g.Phi);
  double cl = cos(g.Lam);
  double sl = sin(g.Lam);
  double ca = cos(az);
  double sa = sin(az);

  g.GetXYZ(result.Shift);

  result.Rotation[6] = -sl * ca + cl * sf * sa;
  result.Rotation[7] =  cl * ca + sl * sf * sa;
  result.Rotation[8] = -cf * sa;

  result.Rotation[3] =  cl * cf;
  result.Rotation[4] =  sl * cf;
  result.Rotation[5] =  sf;

  result.Rotation[0] = -sl * sa - cl * sf * ca;
  result.Rotation[1] =  cl * sa - sl * sf * ca;
  result.Rotation[2] =  cf * ca;

  result.Rotation.Transp();

  }
//------------------------------------------------------------------------------
//  ,        
//      t0 ( 6- )
//------------------------------------------------------------------------------
void LocalRLFrame::Get6DGreenwichConverter(FrameConverter6D &result) const 
  {
  result.Init();
  Get3DGreenwichConverter(result.Converter3D);
  }
////////////////////////////////////////////////////////////////////////////////

