///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_MATRIX_H
#define BNOTOOL_MATRIX_H
///////////////////////////////////////////////////////////////////////////////
#include "Defines.h"

#include <string.h>
///////////////////////////////////////////////////////////////////////////////
//  
///////////////////////////////////////////////////////////////////////////////
class DMatrix;
class DVector;
class DSqrMatrix;
class DSymmMatrix;
class DUppMatrix;
class DLowMatrix;
class DDiagMatrix;
///////////////////////////////////////////////////////////////////////////////
//    ,    
///////////////////////////////////////////////////////////////////////////////
class CDMatrix
  {
  public:
      //   
    virtual ~CDMatrix()
      {} ;
      //   
    virtual unsigned Rows() const = 0;
      //   
    virtual unsigned Cols() const = 0;
      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //      
      // (   )
    virtual void GetArray(double *array) const = 0;
      //   
    virtual void GetLine(unsigned index, double *line) const = 0;
      //   
    virtual void GetCol(unsigned index, double *col) const = 0;

  protected:
    virtual double Val(unsigned r, unsigned c) const = 0;  
  };
///////////////////////////////////////////////////////////////////////////////
//    ,    
///////////////////////////////////////////////////////////////////////////////
class CDSqrMatrix : public CDMatrix
  {
  public:
    unsigned Dim(void) const
      {return Rows();};
  };
///////////////////////////////////////////////////////////////////////////////
//    ,    
///////////////////////////////////////////////////////////////////////////////
class CDSymmMatrix : public CDSqrMatrix
  {
  public:
      //      
    virtual bool Expand(DUppMatrix &s, DDiagMatrix &d) const = 0;
  };
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
class DRectMem
  {
    //  
  friend class DTriagMem;
  friend class DLineMem;
  friend class DSymmMatrix;

    //      
  friend void MultRect(DRectMem &dst, const DRectMem &src1, const DRectMem &src2);
  friend void MultRect(DRectMem &dst, const DRectMem &src1, const DUppMatrix &src2);
  friend void MultRect(DRectMem &dst, const DUppMatrix &src1, const DRectMem &src2);
  friend void MultRect(DRectMem &dst, const DRectMem &src1, const DLowMatrix &src2);
  friend void MultRect(DRectMem &dst, const DLowMatrix &src1, const DRectMem &src2);
  friend void MultRect(DRectMem &dst, const DRectMem &src1, const DDiagMatrix &src2);
  friend void MultRect(DRectMem &dst, const DDiagMatrix &src1, const DRectMem &src2);

  public:
      // 
    DRectMem();
      // 
    virtual ~DRectMem();

      //      
    void Init(double value);

  protected:
      //  
    unsigned NRows;
      //  
    unsigned NCols;
      //    
    unsigned NRowsMem;
      //    
    unsigned NColsMem;
      //  
    unsigned BufSize;
      // 
    double **Matrix;

      //    
    static unsigned ReqSize(unsigned n, unsigned m)
      {return sizeof(double) * m * n + sizeof(double *) * n;};
      //  
    virtual double **CreatePtr(unsigned n, unsigned m);
      //  
    virtual void SetLinesPtrs(double **ptr, unsigned n, unsigned m);

      //  
    void XChng(DRectMem &m);
      //  
    void Init(unsigned r, unsigned c);
      //  
    void ResetDimension(unsigned r, unsigned c);

      // 
    void Init(const CDMatrix &mt);
    void InitRect(const DRectMem &mt);
    void Init(const DSymmMatrix &mt);
    void Init(const DUppMatrix &mt);
    void Init(const DLowMatrix &mt);
    void Init(const DDiagMatrix &mt);

      //     
    void Mult(double mltp);

      //    
    void Add(const CDMatrix &mt);
    void Sub(const CDMatrix &mt);
    void AddRect(const DRectMem &mt);
    void SubRect(const DRectMem &mt);
    void Add(const DSymmMatrix &mt);
    void Sub(const DSymmMatrix &mt);
    void Add(const DUppMatrix &mt);
    void Sub(const DUppMatrix &mt);
    void Add(const DLowMatrix &mt);
    void Sub(const DLowMatrix &mt);
    void Add(const DDiagMatrix &mt);
    void Sub(const DDiagMatrix &mt);

      //   
    void Mult(const DRectMem &m1, const DRectMem &m2);
    void Mult(const DRectMem &m, const DUppMatrix &mt);
    void Mult(const DUppMatrix &mt, const DRectMem &m);
    void Mult(const DRectMem &m, const DLowMatrix &mt);
    void Mult(const DLowMatrix &mt, const DRectMem &m);
    void Mult(const DRectMem &m, const DDiagMatrix &mt);
    void Mult(const DDiagMatrix &mt, const DRectMem &m);
 
      //   
    void MulRightRect(const DRectMem &mt);
    void MulLeftRect(const DRectMem &mt);
    void MulRight(const DUppMatrix &mt);
    void MulLeft(const DUppMatrix &mt);
    void MulRight(const DLowMatrix &mt);
    void MulLeft(const DLowMatrix &mt);
    void MulRight(const DDiagMatrix &mt);
    void MulLeft(const DDiagMatrix &mt);
private:
	DRectMem(const DRectMem&);
  };
///////////////////////////////////////////////////////////////////////////////
//      
///////////////////////////////////////////////////////////////////////////////
class DLineMem
  {
    //  
  friend class DRectMem;
  friend class DTriagMem;
  friend class DSymmMatrix;
  friend class DUppMatrix;
  friend class DLowMatrix;

    //      
  friend DVector &Mult(DVector &dst, const DVector &src1, const CDMatrix &src2);
  friend DVector &Mult(DVector &dst, const CDMatrix &src1, const DVector &src2);
  friend DVector &Mult(DVector &dst, const DVector &src1, const DMatrix &src2);
  friend DVector &Mult(DVector &dst, const DMatrix &src1, const DVector &src2);
  friend DVector &Mult(DVector &dst, const DVector &src1, const DSqrMatrix &src2);
  friend DVector &Mult(DVector &dst, const DSqrMatrix &src1, const DVector &src2);

  public:
      // 
    DLineMem();
      // 
    virtual ~DLineMem();

  protected:
    unsigned Dimension, DimMem;
    unsigned MemSize;
    double *Vector;

      // 
    void Init(unsigned dim);
    void Init(const double *vec, unsigned dim);
      //  
    void ResetDimension(unsigned dim);

      //  
    void XChng(DLineMem &m);

      //      
    double GetMaxElementAbs() const;
    double GetMinElementAbs() const;
    double GetMaxElement() const;
    double GetMinElement() const;

      //      
    void Init(double value);
      //     
    void Mult(double mltp);

      //    
    void Add(const CDMatrix &mt);
    void Sub(const CDMatrix &mt);
    void AddVect(const DLineMem &mt);
    void SubVect(const DLineMem &mt);

      //  
    void MulLeftRect(const DRectMem &mt);
    void MulRect(const DRectMem &mt, const DLineMem &vc);
   private:
    DLineMem(const DLineMem& );
    };
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
class DTriagMem
  {
    //  
  friend class DRectMem;
  friend class DLineMem;

  public:
      // 
    DTriagMem();
      // 
    virtual ~DTriagMem();

  protected:
    unsigned Dimension, DimMem, MemSize;
    double *Buffer;

      //  
    void XChng(DTriagMem &m);
      // 
    void Init(unsigned dim);
      //  
    void ResetDimension(unsigned dim);

      //      
    void Init(double value);
      //     
    void Mult(double mltp);

      //    
    static inline unsigned ReqMemSize(unsigned n) 
      { return (n * (n - 1) >> 1) + n;};
      //     
    unsigned Index(unsigned x, unsigned y) const
      { return ((x * ((DimMem << 1) - x - 1)) >> 1) + y;};
private:
	DTriagMem(const DTriagMem&);
  };
///////////////////////////////////////////////////////////////////////////////
// ,    
///////////////////////////////////////////////////////////////////////////////
class DMatrix : public CDMatrix, public DRectMem
  {
  friend class DSqrMatrix;

  public:
      //   
    DMatrix() : DRectMem()
      {};
      //    
    DMatrix(unsigned r, unsigned c)
      { DRectMem::Init(r, c);};
      //  
    DMatrix(const DMatrix &mt) : CDMatrix() , DRectMem()
      { DRectMem::InitRect(mt);};
    DMatrix(const DSqrMatrix &mt);
    DMatrix(const CDMatrix &mt)
      { DRectMem::Init(mt);};
    DMatrix(const CDSymmMatrix &mt)
      { DRectMem::Init(mt);};
    DMatrix(const DSymmMatrix &mt)
      { DRectMem::Init(mt);};
    DMatrix(const DUppMatrix &mt)
      { DRectMem::Init(mt);};
    DMatrix(const DLowMatrix &mt)
      { DRectMem::Init(mt);};
    DMatrix(const DDiagMatrix &mt)
      { DRectMem::Init(mt);};

      // 
    virtual ~DMatrix(void) 
      {};

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < NRows && c < NCols);
      TAssert(Matrix != NULL);
      return Matrix[r][c];
      };

      //  
    DMatrix & operator=(const DMatrix &mt)
      {InitRect(mt); return *this;};
    DMatrix &operator=(const DSqrMatrix &mt);
    DMatrix & operator=(const CDMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DMatrix & operator=(const CDSymmMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DMatrix & operator=(const DSymmMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DMatrix & operator=(const DUppMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DMatrix & operator=(const DLowMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DMatrix & operator=(const DDiagMatrix &mt)
      { DRectMem::Init(mt); return *this;};

      //  
    void Init(unsigned r, unsigned c)
      { DRectMem::Init(r, c);};
      //      
    void Init(double value)
      { DRectMem::Init(value);};

      //  
    void ResetDimension(unsigned r, unsigned c)
      { DRectMem::ResetDimension(r, c);};

      //   
    DMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;}

      //    
    DMatrix & operator+=(const CDMatrix &mt)
      {Add(mt); return *this;};
    DMatrix & operator+=(const DMatrix &mt)
      {AddRect(mt); return *this;};
    DMatrix & operator+=(const DSqrMatrix &mt);
    DMatrix & operator-=(const CDMatrix &mt)
      {Sub(mt); return *this;};
    DMatrix & operator-=(const DMatrix &mt)
      {SubRect(mt); return *this;};
    DMatrix & operator-=(const DSqrMatrix &mt);
    DMatrix & operator+=(const CDSymmMatrix &mt)
      {Add(mt); return *this;};
    DMatrix & operator+=(const DSymmMatrix &mt)
      {Add(mt); return *this;};
    DMatrix & operator-=(const CDSymmMatrix &mt)
      {Sub(mt); return *this;};
    DMatrix & operator-=(const DSymmMatrix &mt)
      {Sub(mt); return *this;};
    DMatrix & operator+=(const DUppMatrix &mt);
    DMatrix & operator-=(const DUppMatrix &mt);
    DMatrix & operator+=(const DLowMatrix &mt);
    DMatrix & operator-=(const DLowMatrix &mt);
    DMatrix & operator+=(const DDiagMatrix &mt);
    DMatrix & operator-=(const DDiagMatrix &mt);

      //  
    DMatrix & MulRight(const DMatrix &mt)
      {MulRightRect(mt); return *this;};
    inline DMatrix & operator*=(const DMatrix &mt)
      { return MulRight(mt);};
    DMatrix & MulRight(const DSqrMatrix &mt);
    inline DMatrix & operator*=(const DSqrMatrix &mt)
      { return MulRight(mt);};
    DMatrix & MulRight(const CDMatrix &mt)
      { MulRightRect(DMatrix(mt)); return *this;};
    inline DMatrix & operator*=(const CDMatrix &mt)
      { return MulRight(mt);};

    DMatrix & MulLeft(const DMatrix &mt)
      { MulLeftRect(mt); return *this;};
    DMatrix & MulLeft(const DSqrMatrix &mt);
    DMatrix & MulLeft(const CDMatrix &mt)
      { MulLeftRect(DMatrix(mt)); return *this;};

    DMatrix & MulRight(const DUppMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DMatrix & MulLeft(const DUppMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DMatrix & operator*=(const DUppMatrix &mt)
      { return MulRight(mt);};

    DMatrix & MulRight(const DLowMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DMatrix & MulLeft(const DLowMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DMatrix & operator*=(const DLowMatrix &mt)
      { return MulRight(mt);};

    DMatrix & MulRight(const DDiagMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DMatrix & MulLeft(const DDiagMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DMatrix & operator*=(const DDiagMatrix &mt)
      { return MulRight(mt);};

      // 
    void Transpose();

      //    
    virtual unsigned Rows() const
      { return NRows;};
      //    
    virtual unsigned Cols() const
      { return NCols;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert( r < NRows && c < NCols);
      TAssert( Matrix != NULL);
      return Matrix[r][c];
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const
      {
      TAssert( index < NRows);
      TAssert( Matrix != NULL);
      TAssert(line != NULL);
      memcpy(line, Matrix[index], sizeof(double) * NCols);
      }
      //    
    virtual void GetCol(unsigned index, double *col) const;
  };
///////////////////////////////////////////////////////////////////////////////
// ,   
///////////////////////////////////////////////////////////////////////////////
class DSqrMatrix : public CDSqrMatrix, public DRectMem
  {
  friend class DMatrix;
  friend DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend DSqrMatrix &Mult(DSqrMatrix &dst, const  DUppMatrix &src1, const DLowMatrix &src2);

  public:
      //   
    DSqrMatrix() 
      {};
      //    
    DSqrMatrix(unsigned n)
      {DRectMem::Init(n, n);};
      //  
    DSqrMatrix(const CDSqrMatrix &mt)
      { DRectMem::Init(mt);};
    DSqrMatrix(const DSqrMatrix &mt) : CDSqrMatrix(), DRectMem()
      { InitRect(mt);};
    DSqrMatrix(const CDSymmMatrix &mt)
      { DRectMem::Init(mt);};
    DSqrMatrix(const DSymmMatrix &mt)
      { DRectMem::Init(mt);};
    DSqrMatrix(const DUppMatrix &mt)
      { DRectMem::Init(mt);};
    DSqrMatrix(const DLowMatrix &mt)
      { DRectMem::Init(mt);};
    DSqrMatrix(const DDiagMatrix &mt)
      { DRectMem::Init(mt);};

      // 
    virtual ~DSqrMatrix(void) 
      {};

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < NRows && c < NCols);
      TAssert(Matrix != NULL);
      return Matrix[r][c];
      };

      //  
    DSqrMatrix & operator=(const DSqrMatrix &mt)
      { InitRect(mt); return *this;};
    DSqrMatrix & operator=(const CDSqrMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DSqrMatrix & operator=(const CDSymmMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DSqrMatrix & operator=(const DSymmMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DSqrMatrix & operator=(const DUppMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DSqrMatrix & operator=(const DLowMatrix &mt)
      { DRectMem::Init(mt); return *this;};
    DSqrMatrix & operator=(const DDiagMatrix &mt)
      { DRectMem::Init(mt); return *this;};

      //  
    void Init(unsigned n)
      { DRectMem::Init(n, n);};
      //      
    void Init(double value)
      { DRectMem::Init(value);};
      //  
    void ResetDimension(unsigned n)
      { DRectMem::ResetDimension(n, n);};

      //   
    DSqrMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;}

      //    
    DSqrMatrix & operator+=(const CDSqrMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator+=(const DSqrMatrix &mt)
      { AddRect(mt); return *this;};
    DSqrMatrix & operator-=(const CDSqrMatrix &mt)
      { Sub(mt); return *this;};
    DSqrMatrix & operator-=(const DSqrMatrix &mt)
      { SubRect(mt); return *this;};

    DSqrMatrix & operator+=(const CDSymmMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator+=(const DSymmMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator-=(const CDSymmMatrix &mt)
      { Sub(mt); return *this;};
    DSqrMatrix & operator-=(const DSymmMatrix &mt)
      { Sub(mt); return *this;};

    DSqrMatrix & operator+=(const DUppMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator-=(const DUppMatrix &mt)
      { Sub(mt); return *this;};

    DSqrMatrix & operator+=(const DLowMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator-=(const DLowMatrix &mt)
      { Sub(mt); return *this;};

    DSqrMatrix & operator+=(const DDiagMatrix &mt)
      { Add(mt); return *this;};
    DSqrMatrix & operator-=(const DDiagMatrix &mt)
      { Sub(mt); return *this;};

      //  
    DSqrMatrix & MulRight(const CDSqrMatrix &mt)
      { MulRightRect(DSqrMatrix(mt)); return *this;};
    inline DSqrMatrix & operator*=(const CDSqrMatrix &mt)
      { return MulRight(mt);};
    DSqrMatrix & MulRight(const DSqrMatrix &mt)
      { MulRightRect(mt); return *this;};
    inline DSqrMatrix & operator*=(const DSqrMatrix &mt)
      { return MulRight(mt);};

    DSqrMatrix & MulLeft(const DSqrMatrix &mt)
      { MulLeftRect(mt); return *this;};
    DSqrMatrix & MulLeft(const CDSqrMatrix &mt)
      { MulLeftRect(DSqrMatrix(mt)); return *this;};

    DSqrMatrix & MulRight(const DUppMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DSqrMatrix & MulLeft(const DUppMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DSqrMatrix & operator*=(const DUppMatrix &mt)
      { return MulRight(mt);};

    DSqrMatrix & MulRight(const DLowMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DSqrMatrix & MulLeft(const DLowMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DSqrMatrix & operator*=(const DLowMatrix &mt)
      { return MulRight(mt);};

    DSqrMatrix & MulRight(const DDiagMatrix &mt)
      { DRectMem::MulRight(mt); return *this;};
    DSqrMatrix & MulLeft(const DDiagMatrix &mt)
      { DRectMem::MulLeft(mt); return *this;};
    DSqrMatrix & operator*=(const DDiagMatrix &mt)
      { return MulRight(mt);};

      // 
    void Transpose();
      // 
    bool Inverse();

      //    
    virtual unsigned Rows() const
      { return NRows;};
      //    
    virtual unsigned Cols() const
      { return NCols;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert( r < NRows && c < NCols);
      TAssert( Matrix != NULL);
      return Matrix[r][c];
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const
      {
      TAssert(index < NRows);
      TAssert(Matrix != NULL);
      TAssert(line != NULL);
      memcpy(line, Matrix[index], sizeof(double) * NCols);
      }
      //    
    virtual void GetCol(unsigned index, double *col) const;
  };
///////////////////////////////////////////////////////////////////////////////
// ,   
///////////////////////////////////////////////////////////////////////////////
class DSymmMatrix : public CDSymmMatrix, public DTriagMem
  {
  friend double TRecmtSymmRecmt(const DVector &A, const DSymmMatrix &B);

  public:
       // 
    DSymmMatrix() 
      {};
    DSymmMatrix(unsigned dimension)
      {Init(dimension);};
    DSymmMatrix(const DSymmMatrix &mt)  : CDSymmMatrix(), DTriagMem()
      {Init(mt);};
    DSymmMatrix(const CDSymmMatrix &mt)
      {Init(mt);};
    DSymmMatrix(const DDiagMatrix &mt)
      {Init(mt);};
    virtual ~DSymmMatrix(void) {};
  
      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      unsigned ind = (c >= r) ? Index(r, c) : Index(c, r);
      return Buffer[ind];
      };

      //  
    DSymmMatrix & operator=(const DSymmMatrix &mt)
      {Init(mt); return *this;};
    DSymmMatrix & operator=(const CDSymmMatrix &mt)
      {Init(mt); return *this;};
    DSymmMatrix & operator=(const DDiagMatrix &mt)
      {Init(mt); return *this;};
      
      //   
    DSymmMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;};
      // 
    void Init(unsigned dim)
      { DTriagMem::Init(dim);};
      //      
    void Init(double value)
      { DTriagMem::Init(value);};

      //    
    DSymmMatrix & operator+=(const CDSymmMatrix &mt);
    DSymmMatrix & operator-=(const CDSymmMatrix &mt);
    DSymmMatrix & operator+=(const DSymmMatrix &mt);
    DSymmMatrix & operator-=(const DSymmMatrix &mt);
    DSymmMatrix & operator+=(const DDiagMatrix &mt);
    DSymmMatrix & operator-=(const DDiagMatrix &mt);

      // 
    bool Inverse();

      //   
    DSymmMatrix & RectSymmTRecmt(const DRectMem &A, const DSymmMatrix &B);
    DSymmMatrix & TRectSymmRecmt(const DRectMem &A, const DSymmMatrix &B);
    DSymmMatrix & RectSymmTRecmt(const CDSqrMatrix &A, const DSymmMatrix &B);
    DSymmMatrix & TRectSymmRecmt(const CDSqrMatrix &A, const DSymmMatrix &B);

      //       
    DSymmMatrix & RectTRecmt(const DRectMem &A);

      //      
    virtual bool Expand(DUppMatrix &s, DDiagMatrix &d) const;

    void JacobiExpand(DDiagMatrix &S,  DMatrix &R);

      //  
    friend double TRectSymmRect(const DLineMem &A, const DSymmMatrix &B);
    
      //    
    virtual unsigned Rows() const
      { return Dimension;};
      //    
    virtual unsigned Cols() const
      { return Dimension;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      unsigned ind = (c >= r) ? Index(r, c) : Index(c, r);
      return Buffer[ind];
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const;
      //    
    virtual void GetCol(unsigned index, double *col) const
      { GetLine(index, col);};

  protected:
      // 
    void Init(const CDSymmMatrix &mt);
    void Init(const DDiagMatrix &mt);
    void Init(const DSymmMatrix &mt);
  };
///////////////////////////////////////////////////////////////////////////////
// ,  - (   )
///////////////////////////////////////////////////////////////////////////////
class DVector : public CDMatrix, public DLineMem
  {
  friend bool TDMSolution(const DVector &b, const DDiagMatrix &a, DVector &x);
  friend bool TDMSolution(const DVector &b, const DUppMatrix &a, DVector &x);
  friend bool TDMSolution(const DVector &b, const CDSymmMatrix &a, DVector &x);
  friend bool TDMSolution(const DVector &b, const DLowMatrix &a, DVector &x);
  friend double operator*(const DVector &a, const DVector &b);
  friend double TRecmtSymmRecmt(const DVector &A, const DSymmMatrix &B);

  public:

      // 
    DVector() 
      {}; 
    DVector(unsigned dim)
      { Init(dim);}
    DVector(const DVector &vec) : CDMatrix(),  DLineMem()
      { DLineMem::Init(vec.Vector, vec.Dimension);};

      // 
    virtual ~DVector() 
      {};

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};

    double operator[](unsigned index) const
      { 
      TAssert(index < Dimension); 
      return Vector[index];
      };
    double operator()(unsigned index) const
      { 
      TAssert(index < Dimension); 
      return Vector[index];
      };
    unsigned Dim(void) const
      { return Dimension;};


      //    
    double & operator[](unsigned index)
      { 
      TAssert(index < Dimension); 
      TAssert(Vector != NULL);
      return Vector[index];
      };
    double & operator()(unsigned index)
      { 
      TAssert(index < Dimension); 
      TAssert(Vector != NULL);
      return Vector[index];
      };

      // 
    void Init(unsigned dim)
      { DLineMem::Init(dim);};
      //      
    void Init(double value)
      { DLineMem::Init(value);};

      //   
    DVector & operator*=(double mltp)
      { 
      Mult(mltp); 
      return *this;
      }; 

      //  
    DVector & operator=(const CDMatrix &mt);
    DVector & operator=(const DVector &vec)
      { DLineMem::Init(vec.Vector, vec.Dimension); return *this;};

      //  
    double Abs(void) const;

      //    
    DVector & operator+=(const DVector &mt)
      { AddVect(mt); return *this;};
    DVector & operator-=(const DVector &mt)
      { SubVect(mt); return *this;};

      //  
    DVector & MulLeft(const CDMatrix &mt)
      { MulLeftRect(DMatrix(mt)); return *this;};
    DVector & MulLeft(const DMatrix &mt)
      { MulLeftRect(mt); return *this;};
    DVector & MulLeft(const DSqrMatrix &mt)
      { MulLeftRect(mt); return *this;};

    DVector & MulLeft(const DUppMatrix &mt);
    DVector & MulLeft(const DLowMatrix &mt);
    DVector & MulLeft(const DDiagMatrix &mt);
     
      //     
    DVector & TRectRectVector(const CDMatrix &a, const CDMatrix &p, const DVector &b);

    friend double TRectSymmRecmt(const DVector &a, const DSymmMatrix &b);
    
      //    
    virtual unsigned Rows() const
      { return Dimension;};
      //    
    virtual unsigned Cols() const
      { return 1;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert(c == 0 && r < Dimension);
      TAssert(Vector != NULL);
      return Vector[r]; 
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const;
      //    
    virtual void GetCol(unsigned index, double *col) const;
  };
///////////////////////////////////////////////////////////////////////////////
// ,    
///////////////////////////////////////////////////////////////////////////////
class DUppMatrix : public CDMatrix, public DTriagMem
  {
  friend class DVector;
  friend class DSymmMatrix; 
  friend class DLowMatrix; 
  friend class DDiagMatrix; 

  friend DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend DSqrMatrix &Mult(DSqrMatrix &dst, const DUppMatrix &src1, const DLowMatrix &src2);
  //friend DDiagMatrix &Mult(DDiagMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend bool TDMSolution(const DVector &b, const DUppMatrix &a, DVector &x);

  public:
      // 
    DUppMatrix() 
      {};
    DUppMatrix(unsigned dimension)
      {Init(dimension);};
    DUppMatrix(const DUppMatrix &mt): CDMatrix(), DTriagMem() 
      {Init(mt);};
    DUppMatrix(const DDiagMatrix &mt)
      {Init(mt);};
      // 
    virtual ~DUppMatrix(void) 
      {};

      //  
    unsigned Dim() const
      { return Dimension;};

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      TAssert(c >= r);
      return Buffer[Index(r, c)];
      };

      //  
    DUppMatrix & operator=(const DUppMatrix &mt)
      {Init(mt); return *this;};
    DUppMatrix & operator=(const DDiagMatrix &mt)
      {Init(mt); return *this;};
      
      // 
    void Init(unsigned dim)
      { DTriagMem::Init(dim);};
    void Init(double value)
      { DTriagMem::Init(value);};

      //   
    DUppMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;};

      //    
    DUppMatrix & operator+=(const DUppMatrix &mt);
    DUppMatrix & operator-=(const DUppMatrix &mt);
    DUppMatrix & operator+=(const DDiagMatrix &mt);
    DUppMatrix & operator-=(const DDiagMatrix &mt);

      //  
    DUppMatrix & MulRight(const DUppMatrix &mt);
    DUppMatrix & MulLeft(const DUppMatrix &mt);
    DUppMatrix & operator*=(const DUppMatrix &mt)
      { return MulRight(mt);};

    DUppMatrix & MulRight(const DDiagMatrix &mt);
    DUppMatrix & MulLeft(const DDiagMatrix &mt);
    DUppMatrix & operator*=(const DDiagMatrix &mt)
      { return MulRight(mt);};

      //    
    DUppMatrix & Transpose(const DLowMatrix &A);

      //    
    virtual unsigned Rows() const
      { return Dimension;};
      //    
    virtual unsigned Cols() const
      { return Dimension;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      return (c >= r) ? Buffer[Index(r, c)] : 0.0;
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const;
      //    
    virtual void GetCol(unsigned index, double *col) const;

  protected:
     // 
    void Init(const DUppMatrix &mt);
    void Init(const DDiagMatrix &mt);
  };
///////////////////////////////////////////////////////////////////////////////
// ,    
///////////////////////////////////////////////////////////////////////////////
class DLowMatrix : public CDMatrix, public DTriagMem
  {
  friend class DVector;
  friend class DSymmMatrix; 
  friend class DUppMatrix; 
  friend class DDiagMatrix; 

  friend DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend DSqrMatrix &Mult(DSqrMatrix &dst, const DUppMatrix &src1, const DLowMatrix &src2);
  //friend DDiagMatrix &Mult(DDiagMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend bool TDMSolution(const DVector &b, const DLowMatrix &a, DVector &x);

  public:
      // 
    DLowMatrix() 
      {};
    DLowMatrix(unsigned dimension)
      {Init(dimension);};
    DLowMatrix(const DLowMatrix &mt) : CDMatrix(),  DTriagMem()
      {Init(mt);};
    DLowMatrix(const DDiagMatrix &mt)
      {Init(mt);};
      // 
    virtual ~DLowMatrix(void) 
      {};

      //  
    unsigned Dim() const
      { return Dimension;};

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      TAssert(c <= r);
      return Buffer[Index(c, r)];
      };

      //  
    DLowMatrix & operator=(const DLowMatrix &mt)
      {Init(mt); return *this;};
    DLowMatrix & operator=(const DDiagMatrix &mt)
      {Init(mt); return *this;};
      
      //   
    DLowMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;};

      // 
    void Init(unsigned dim)
      { DTriagMem::Init(dim);};
      //      
    void Init(double value)
      { DTriagMem::Init(value);};

      //    
    DLowMatrix & operator+=(const DLowMatrix &mt);
    DLowMatrix & operator-=(const DLowMatrix &mt);
    DLowMatrix & operator+=(const DDiagMatrix &mt);
    DLowMatrix & operator-=(const DDiagMatrix &mt);

      //  
    DLowMatrix & MulRight(const DLowMatrix &mt);
    DLowMatrix & MulLeft(const DLowMatrix &mt);
    DLowMatrix & operator*=(const DLowMatrix &mt)
      { return MulRight(mt);};

    DLowMatrix & MulRight(const DDiagMatrix &mt);
    DLowMatrix & MulLeft(const DDiagMatrix &mt);
    DLowMatrix & operator*=(const DDiagMatrix &mt)
      { return MulRight(mt);};

      //    
    DLowMatrix & Transpose(const DUppMatrix &A);

      //    
    virtual unsigned Rows() const
      { return Dimension;};
      //    
    virtual unsigned Cols() const
      { return Dimension;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Buffer != NULL);
      return (c <= r) ? Buffer[Index(c, r)] : 0.0;
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const;
      //    
    virtual void GetCol(unsigned index, double *col) const;

  protected:
     // 
    void Init(const DLowMatrix &mt);
    void Init(const DDiagMatrix &mt);
  };
///////////////////////////////////////////////////////////////////////////////
// ,   
///////////////////////////////////////////////////////////////////////////////
class DDiagMatrix : public CDSymmMatrix, public DLineMem
  {
  friend class DVector;
  friend class DSymmMatrix; 

  //friend DDiagMatrix &Mult(DDiagMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
  friend bool TDMSolution(const DVector &B, const DDiagMatrix &A, DVector &X);

  public:

      // 
    DDiagMatrix()  
      {};
    DDiagMatrix(unsigned dimension)
      {Init(dimension);};
    DDiagMatrix(const DDiagMatrix &mt) : CDSymmMatrix(),  DLineMem()
      {Init(mt);};
      // 
    virtual ~DDiagMatrix(void) 
      {};

      //  
    unsigned Dim() const
      { return Dimension;};
 
      //   
    double operator[](unsigned index) const
      {
      TAssert(index < Dimension);
      TAssert(Vector != NULL);
      return Vector[index];
      };
      //   
    double operator()(unsigned index) const
      {
      TAssert(index < Dimension);
      TAssert(Vector != NULL);
      return Vector[index];
      };

      //   
    virtual double operator()(unsigned r, unsigned c) const
      { return Val(r, c);};
      //    
    double &operator()(unsigned r, unsigned c)
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Vector != NULL);
      TAssert(r == c);
      return Vector[r];
      };
    double & operator[](unsigned index)
      {
      TAssert(index < Dimension);
      TAssert(Vector != NULL);
      return Vector[index];
      };
    double & operator()(unsigned index)
      {
      TAssert(index < Dimension);
      TAssert(Vector != NULL);
      return Vector[index];
      };

      //  
    DDiagMatrix & operator=(const DDiagMatrix &mt)
      {Init(mt); return *this;};
      
      //   
    DDiagMatrix & operator*=(double mltp)
      {Mult(mltp); return *this;};

      // 
    void Init(unsigned dim)
      { DLineMem::Init(dim);};
      //      
    void Init(double value)
      { DLineMem::Init(value);};

      //    
    DDiagMatrix & operator+=(const DDiagMatrix &mt);
    DDiagMatrix & operator-=(const DDiagMatrix &mt);

      //  
    DDiagMatrix & MulRight(const DDiagMatrix &mt);
    DDiagMatrix & MulLeft(const DDiagMatrix &mt);
    DDiagMatrix & operator*=(const DDiagMatrix &mt)
      { return MulRight(mt);};

      //  (  )
    bool Inverse();

      //      
    virtual bool Expand(DUppMatrix &s, DDiagMatrix &d) const;

      //    
    virtual unsigned Rows() const
      { return Dimension;};
      //    
    virtual unsigned Cols() const
      { return Dimension;};
      //   
    virtual double Val(unsigned r, unsigned c) const
      {
      TAssert(r < Dimension && c < Dimension);
      TAssert(Vector != NULL);
      return (r == c) ? Vector[r] : 0.0;
      };
      //       
    virtual void GetArray(double *array) const;
      //    
    virtual void GetLine(unsigned index, double *line) const;
      //    
    virtual void GetCol(unsigned index, double *col) const;

  protected:
     // 
    void Init(const DDiagMatrix &mt);
  };
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
template<class SRC1, class SRC2> DMatrix &Add(DMatrix &dst, const SRC1 &src1, const SRC2 &src2)
  { dst = src1; dst += src2; return dst;};
template<class SRC1, class SRC2> DMatrix &Sub(DMatrix &dst, const SRC1 &src1, const SRC2 &src2)
  { dst = src1; dst -= src2; return dst;};

template<class SRC1, class SRC2> DSqrMatrix &Add(DSqrMatrix &dst, const SRC1 &src1, const SRC2 &src2)
  { dst = src1; dst += src2; return dst;};
template<class SRC1, class SRC2> DSqrMatrix &Sub(DSqrMatrix &dst, const SRC1 &src1, const SRC2 &src2)
  { dst = src1; dst -= src2; return dst;};

inline DVector &Add(DVector &dst, const DVector &src1, const DVector &src2)
  { dst = src1; dst += src2; return dst;};
inline DVector &Sub(DVector &dst, const DVector &src1, const DVector &src2)
  { dst = src1; dst -= src2; return dst;};

inline DSymmMatrix &Add(DSymmMatrix &dst, const CDSymmMatrix &src1, const CDSymmMatrix &src2)
  { dst = src1; dst += src2; return dst;};
inline DSymmMatrix &Sub(DSymmMatrix &dst, const CDSymmMatrix &src1, const CDSymmMatrix &src2)
  { dst = src1; dst -= src2; return dst;};

inline DUppMatrix &Add(DUppMatrix &dst, const DUppMatrix &src1, const DUppMatrix &src2)
  { dst = src1; dst += src2; return dst;};
inline DUppMatrix &Sub(DUppMatrix &dst, const DUppMatrix &src1, const DUppMatrix &src2)
  { dst = src1; dst -= src2; return dst;};

inline DLowMatrix &Add(DLowMatrix &dst, const DLowMatrix &src1, const DLowMatrix &src2)
  { dst = src1; dst += src2; return dst;};
inline DLowMatrix &Sub(DLowMatrix &dst, const DLowMatrix &src1, const DLowMatrix &src2)
  { dst = src1; dst -= src2; return dst;};

inline DDiagMatrix &Add(DDiagMatrix &dst, const DDiagMatrix &src1, const DDiagMatrix &src2)
  { dst = src1; dst += src2; return dst;};
inline DDiagMatrix &Sub(DDiagMatrix &dst, const DDiagMatrix &src1, const DDiagMatrix &src2)
  { dst = src1; dst -= src2; return dst;};
///////////////////////////////////////////////////////////////////////////////
//  
///////////////////////////////////////////////////////////////////////////////
inline void MultRect(DRectMem &dst, const DRectMem &src1, const DRectMem &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DRectMem &src1, const DUppMatrix &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DUppMatrix &src1, const DRectMem &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DRectMem &src1, const DLowMatrix &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DLowMatrix &src1, const DRectMem &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DRectMem &src1, const DDiagMatrix &src2)
  {dst.Mult(src1, src2);};
inline void MultRect(DRectMem &dst, const DDiagMatrix &src1, const DRectMem &src2)
  {dst.Mult(src1, src2);};

inline DMatrix &Mult(DMatrix &dst, const CDMatrix &src1, const CDMatrix &src2)
  {MultRect(dst, DMatrix(src1), DMatrix(src2)); return dst;};
inline DMatrix &Mult(DMatrix &dst, const CDMatrix &src1, const DMatrix &src2)
  {MultRect(dst, DMatrix(src1), src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DMatrix &src1, const CDMatrix &src2)
  {MultRect(dst, src1, DMatrix(src2)); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DMatrix &src1, const DMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};

inline DMatrix &Mult(DMatrix &dst, const DMatrix &src1, const DUppMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const CDMatrix &src1, const DUppMatrix &src2)
  {MultRect(dst, DMatrix(src1), src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DUppMatrix &src1, const DMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DUppMatrix &src1, const CDMatrix &src2)
  {MultRect(dst, src1, DMatrix(src2)); return dst;};

inline DMatrix &Mult(DMatrix &dst, const DMatrix &src1, const DLowMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const CDMatrix &src1, const DLowMatrix &src2)
  {MultRect(dst, DMatrix(src1), src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DLowMatrix &src1, const DMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DLowMatrix &src1, const CDMatrix &src2)
  {MultRect(dst, src1, DMatrix(src2)); return dst;};

inline DMatrix &Mult(DMatrix &dst, const DMatrix &src1, const DDiagMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const CDMatrix &src1, const DDiagMatrix &src2)
  {MultRect(dst, DMatrix(src1), src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DDiagMatrix &src1, const DMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DMatrix &Mult(DMatrix &dst, const DDiagMatrix &src1, const CDMatrix &src2)
  {MultRect(dst, src1, DMatrix(src2)); return dst;};

inline DSqrMatrix &Mult(DSqrMatrix &dst, const CDSqrMatrix &src1, const CDSqrMatrix &src2)
  {MultRect(dst, DSqrMatrix(src1), DSqrMatrix(src2)); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const CDSqrMatrix &src1, const DSqrMatrix &src2)
  {MultRect(dst, DSqrMatrix(src1), src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DSqrMatrix &src1, const CDSqrMatrix &src2)
  {MultRect(dst, src1, DSqrMatrix(src2)); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DSqrMatrix &src1, const DSqrMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};

inline DSqrMatrix &Mult(DSqrMatrix &dst, const DSqrMatrix &src1, const DUppMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const CDSqrMatrix &src1, const DUppMatrix &src2)
  {MultRect(dst, DSqrMatrix(src1), src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DUppMatrix &src1, const DSqrMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DUppMatrix &src1, const CDSqrMatrix &src2)
  {MultRect(dst, src1, DSqrMatrix(src2)); return dst;};

inline DSqrMatrix &Mult(DSqrMatrix &dst, const DSqrMatrix &src1, const DLowMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const CDSqrMatrix &src1, const DLowMatrix &src2)
  {MultRect(dst, DSqrMatrix(src1), src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const DSqrMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const CDSqrMatrix &src2)
  {MultRect(dst, src1, DSqrMatrix(src2)); return dst;};

inline DSqrMatrix &Mult(DSqrMatrix &dst, const DSqrMatrix &src1, const DDiagMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const CDSqrMatrix &src1, const DDiagMatrix &src2)
  {MultRect(dst, DSqrMatrix(src1), src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DDiagMatrix &src1, const DSqrMatrix &src2)
  {MultRect(dst, src1, src2); return dst;};
inline DSqrMatrix &Mult(DSqrMatrix &dst, const DDiagMatrix &src1, const CDSqrMatrix &src2)
  {MultRect(dst, src1, DSqrMatrix(src2)); return dst;};

DSqrMatrix &Mult(DSqrMatrix &dst, const DLowMatrix &src1, const DUppMatrix &src2);
DSqrMatrix &Mult(DSqrMatrix &dst, const DUppMatrix &src1, const DLowMatrix &src2);

inline DVector &Mult(DVector &dst, const CDMatrix &src1, const DVector &src2)
  {dst.MulRect(DMatrix(src1), src2); return dst;};
inline DVector &Mult(DVector &dst, const DMatrix &src1, const DVector &src2)
  {dst.MulRect(src1, src2); return dst;};
inline DVector &Mult(DVector &dst, const DSqrMatrix &src1, const DVector &src2)
  {dst.MulRect(src1, src2); return dst;};

inline DUppMatrix &Mult(DUppMatrix &dst, const DUppMatrix &src1, const DUppMatrix &src2)
  { dst = src1; dst *= src2; return dst;};
inline DUppMatrix &Mult(DUppMatrix &dst, const DUppMatrix &src1, const DDiagMatrix &src2)
  { dst = src1; dst *= src2; return dst;};
inline DUppMatrix &Mult(DUppMatrix &dst, const DDiagMatrix &src1, const DUppMatrix &src2)
  { dst = src1; dst *= src2; return dst;};

inline DLowMatrix &Mult(DLowMatrix &dst, const DLowMatrix &src1, const DLowMatrix &src2)
  { dst = src1; dst *= src2; return dst;};
inline DLowMatrix &Mult(DLowMatrix &dst, const DLowMatrix &src1, const DDiagMatrix &src2)
  { dst = src1; dst *= src2; return dst;};
inline DLowMatrix &Mult(DLowMatrix &dst, const DDiagMatrix &src1, const DLowMatrix &src2)
  { dst = src1; dst *= src2; return dst;};

inline DDiagMatrix &Mult(DDiagMatrix &dst, const DDiagMatrix &src1, const DDiagMatrix &src2)
  { dst = src1; dst *= src2; return dst;};
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
bool TDMSolution(const DVector &B, const CDSqrMatrix &A, DVector &X);
bool TDMSolution(const DVector &B, const CDSymmMatrix &A, DVector &X);
bool TDMSolution(const DVector &B, const DDiagMatrix &A, DVector &X);
bool TDMSolution(const DVector &B, const DUppMatrix &A, DVector &X);
bool TDMSolution(const DVector &B, const DLowMatrix &A, DVector &X);
bool TDMSolutionJ(const DVector &B, DSymmMatrix &A, DVector &X);
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
double operator*(const DVector &a, const DVector &b);
///////////////////////////////////////////////////////////////////////////////
//  
///////////////////////////////////////////////////////////////////////////////
double TRecmtSymmRecmt(const DVector &A, const DSymmMatrix &B);
///////////////////////////////////////////////////////////////////////////////
//    DMatrix
///////////////////////////////////////////////////////////////////////////////
inline DMatrix::DMatrix(const DSqrMatrix &mt)
  {InitRect(mt);}
inline DMatrix &DMatrix::operator=(const DSqrMatrix &mt)
  {InitRect(mt); return *this;};
inline DMatrix & DMatrix::operator+=(const DSqrMatrix &mt)
  {AddRect(mt); return *this;};
inline DMatrix & DMatrix::operator-=(const DSqrMatrix &mt)
  {SubRect(mt); return *this;};
inline DMatrix & DMatrix::operator+=(const DUppMatrix &mt)
  {Add(mt); return *this;};
inline DMatrix & DMatrix::operator-=(const DUppMatrix &mt)
  {Sub(mt); return *this;};
inline DMatrix & DMatrix::operator+=(const DLowMatrix &mt)
  {Add(mt); return *this;};
inline DMatrix & DMatrix::operator-=(const DLowMatrix &mt)
  {Sub(mt); return *this;};
inline DMatrix & DMatrix::operator+=(const DDiagMatrix &mt)
  {Add(mt); return *this;};
inline DMatrix & DMatrix::operator-=(const DDiagMatrix &mt)
  {Sub(mt); return *this;};
inline DMatrix & DMatrix::MulRight(const DSqrMatrix &mt)
  {MulRightRect(mt); return *this;};
inline DMatrix & DMatrix::MulLeft(const DSqrMatrix &mt)
  {MulLeftRect(mt); return *this;};
///////////////////////////////////////////////////////////////////////////////
// ,    
///////////////////////////////////////////////////////////////////////////////
#define ZERO_ACCURACY (1.0e-100)
///////////////////////////////////////////////////////////////////////////////
#endif

