///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ODEPHASERP_H
#define BNOTOOL_ODEPHASERP_H
///////////////////////////////////////////////////////////////////////////////
#include "IODE.h"
#include "MMode.h"
#include "IKOProp.h"
#include "AccAdapter.h"
#include "IModel.h"
///////////////////////////////////////////////////////////////////////////////
//    ,   
///////////////////////////////////////////////////////////////////////////////
class ODEPhaseSystem : public IODESystem
  {
  public:
      //   
    ODEPhaseSystem();
      // 
    virtual ~ODEPhaseSystem();
      
      // 
    void Init(const TimeJD &t0, const MotionMode &m, IKOProp &ko, AccAdapter &a);
	void SetPrecItrf(bool value);
      //   
    virtual int GetDimension() const
      { return 6;};
      //   
      //   false,    ,
      //     
    virtual bool GetRP(double t, const double *ph, double *f);
	
	bool getPrecItrf() const {return m_precItrf;}
  protected:
      //   
    const MotionMode *MMode;
      //   
    const IKOProp *KOModel;
      //  
    AccAdapter *Ad;
      //  ,     
      // 
    TimeJD T0;

      //    (. )  
    void SetTime(double x, TimeJD &t) const;
      //       
    double GetDiff(const TimeJD &t) const;
      //   
    void SetAdapter(double x, const double *ph, bool precItrf);

      //      
    void EarthNonCentralAcc(Vect3 &acc, const Vect3 &pos); 
      //     
    void PlanetGravityAcc(double mu, const Vect3 &r, const Vect3 &rp,
                          Vect3 &acc) const;
      //   
      // (  ) 
    const IGravityModel *EarthGravity;
      //  ,    
      //   
    double *Pm, *Pm1, *Ron, *Nm1;
      //   
    int Ng, Mg;
	//    J2000->GFCS   
	bool m_precItrf;
  };
///////////////////////////////////////////////////////////////////////////////
//    ,   KO  
//    
///////////////////////////////////////////////////////////////////////////////
class ODEVarSystem : public ODEPhaseSystem
  {
  public:
      //   
    ODEVarSystem();
      // 
    virtual ~ODEVarSystem()
      {};

      //   
    void SetKO(IKOPropVar &ko)
      { KOVar = &ko;};   
      // 
    void Init(const TimeJD &t0, const MotionMode &m, AccAdapter &a);

      //   
    virtual int GetDimension() const
      { TAssert(KOVar != NULL); return 6 + (6 + KOVar->GetNVars()) * 6;};
      //   
      //   false,    ,
      //     
    virtual bool GetRP(double t, const double *ph, double *f);

  protected:
      //    c   
    IKOPropVar *KOVar;

      //       
    void VarCentralAccel(TransMtx &m);

      //       
    void VarNonCentralAccel(TransMtx &m);

      //       
    void VarPlanetAccel(double mu, Vect3 &r, Vect3 &r0, TransMtx &m);
  };

///////////////////////////////////////////////////////////////////////////////
#endif
