///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,    c   
// .
//   ,      
// GetSrcFrame()
///////////////////////////////////////////////////////////////////////////////
#include "OpticDelayMeasurement.h"
#include "GreenwichFrame.h"
#include "Consts.h"
#include "PhasePoint.h"
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
//   
//-----------------------------------------------------------------------------
OpticDelayMeasurement::OpticDelayMeasurement() : OpticMeasurement()
{
	TimePrec = 0.01e-3; //       
	//      
}
//-----------------------------------------------------------------------------
//    t,       
// ,          
//-----------------------------------------------------------------------------
bool OpticDelayMeasurement::DetectPosition(ITraject &trj, TimeJD &t, const IFrame *fr, Vect6 *f) const
{
	Vect6 fo;
	Vect3 rc;
	double tau = 0, tau1 = 0;
	GreenwichFrame gr;

	if (fr == NULL){
		fr = &gr;
	}

	if (f == NULL)
		f = &fo;

	if (RecPosFrInit){
		rc = RecPosFr;
	}else{
		if (!DetectRecieverPos(t, *fr, rc)){
			return false;
		}
	}

	int count = 0;
	for (;;)
	{
		t = GetTime();
		t -= tau/86.4;

		if (!trj.Predict(t, *fr, *f)){
			return false;
		}

		Vect3 del = rc;
		del -= *f;
		tau1 = del.Len() / Clight;
		if (fabs(tau - tau1) < TimePrec){
			return true;
		}

		tau = tau1;

		if(count++ > 100){
			return false;
		}
	}
}

bool OpticDelayMeasurement::DetectPositionEx(ITraject &trj, TimeJD &t, Vect6& f) const
{
	double tau = 0, tau1 = 0;
	Vect3 rc = RecPosFr;
	
	for (;;)
	{
		t = GetTime();
		t -= tau/86.4;
		
		if (!trj.PredictEx(t, f)){
			return false;
		}
		
	//	return true;
	//	break; //TODO

		Vect3 del = rc;
		del -= f;
		tau1 = del.Len() / Clight;
		if (fabs(tau - tau1) < TimePrec){
			return true;
		}
			
		tau = tau1;
	}
}

//-----------------------------------------------------------------------------
//      f    t
//   ,   fr
//-----------------------------------------------------------------------------
bool OpticDelayMeasurement::DetectRecieverPos(const TimeJD &t, const IFrame &fr, 
											  Vect3 &f) const
{
	/*  PhasePoint3D dst;
	dst.SetFrame(fr);

	PhasePoint3D src; 
	GreenwichFrame gr;
	src.Init(gr, RecPos, MTime);

	dst.Assign(src);

	f = dst.Coords;
	*/

	PhasePoint3D dst(static_cast<const IFrame&>(GreenwichFrame()),RecPos,MTime);

	dst.Convert(fr);
	f = dst.Coords;

	return true;
}

bool OpticDelayMeasurement::PrepareDstFrame( const IFrame &fr )
{
	SetWrkFrame(fr);

	if (!DetectRecieverPos(GetTime(), fr, RecPosFr)){
		return false;
	}

	RecPosFrInit = true;
	
	return true;
}
///////////////////////////////////////////////////////////////////////////////
