///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
// ,      
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_PLANET_H
#define BNOTOOL_PLANET_H
///////////////////////////////////////////////////////////////////////////////
#include "IModel.h"
#include "ephemeris.h"
#include "SGravity.h"
#include "Atmosphere.h"
///////////////////////////////////////////////////////////////////////////////
//  ,     
///////////////////////////////////////////////////////////////////////////////
class PlanetConsts 
  {
  public:
    TPlanetId PlanetId;
    const char *PlanetName;
    Ephemeride::EnBodyID PlanetEphId;
    double MeanRot;
    double Re;
    double Flat;
    double Mu;
  };
///////////////////////////////////////////////////////////////////////////////
//     
///////////////////////////////////////////////////////////////////////////////
class SPlanet : public IPlanet
  {
  public:
      //   
    SPlanet()
      {};
      //    
    SPlanet(const PlanetConsts &pc, const char *eph_name = NULL)
      { Init(pc, eph_name);};

      //   
    void Init(const PlanetConsts &pc, const char *eph_name = NULL);

      //  
    virtual const char *Name() const
      { return PName;};
      //  
    virtual TPlanetId Id() const
      { return ID;};
      //    
    virtual const IGravityModel *GetGravityModel() const
      { return &GrModel;};
      //    
    virtual const IAtmosphereModel *GetAtmosphere() const
      { return NULL;};
      //   
    virtual IEphemeride *GetEphemeride()
      { return &Eph;};
    virtual const IEphemeride *GetEphemeride() const
      { return &Eph;};

  protected:
    const char *PName;
    TPlanetId ID;
    SGravityModel GrModel;
    PlanetEphemeride Eph;
  };
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
class MoonPlanet : public IPlanet
  {
  public:
      //    
    MoonPlanet(const char *eph_name = NULL)
      { Init(eph_name);};

      //   
    void Init(const char *eph_name = NULL);

      //  
    virtual const char *Name() const
      { return "Moon";};
      //  
    virtual TPlanetId Id() const
      { return plIdMoon;};
      //    
    virtual const IGravityModel *GetGravityModel() const
      { return &GrModel;};
      //    
    virtual const IAtmosphereModel *GetAtmosphere() const
      { return NULL;};
      //   
    virtual IEphemeride *GetEphemeride()
      { return &Eph;};
    virtual const IEphemeride *GetEphemeride() const
      { return &Eph;};

  protected:
    SGravityModel GrModel;
    MoonEphemeride Eph;
  };
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
class SunPlanet : public IPlanet
  {
  public:
      //   
    SunPlanet();

      //  
    virtual const char *Name() const
      { return "Sun";};
      //  
    virtual TPlanetId Id() const
      { return plIdSun;};
      //    
    virtual const IGravityModel *GetGravityModel() const
      { return &GrModel;};
      //    
    virtual const IAtmosphereModel *GetAtmosphere() const
      { return NULL;};
      //   
    virtual IEphemeride *GetEphemeride()
      { return &Eph;};
    virtual const IEphemeride *GetEphemeride() const
      { return &Eph;};

  protected:
    SGravityModel GrModel;
    SunEphemeride Eph;
  };
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
class EarthPlanet : public IPlanet
  {
  public:
      //   
    EarthPlanet();
      // 
    virtual ~EarthPlanet();

      //  
    virtual const char *Name() const
      { return "Earth";};
      //  
    virtual TPlanetId Id() const
      { return plIdEarth;};
      //    
    virtual const IGravityModel *GetGravityModel() const
      { return GrModel;};
      //    
    virtual const IAtmosphereModel *GetAtmosphere() const
      { return AtmModel;};
      //   
    virtual IEphemeride *GetEphemeride()
      { return &Eph;};
    virtual const IEphemeride *GetEphemeride() const
      { return &Eph;};

      //   
    bool SetGravityModel(TGravityModelId m, const char *fname = NULL);
      //   
    void SetAtmModel(TAtmosphereId m, ITableGeoSolarFlux *gsf = NULL, bool flag3h = false);

  protected:
    IGravityModel *GrModel;
    IAtmosphereModel *AtmModel;
    EarthEphemeride Eph;
  };
///////////////////////////////////////////////////////////////////////////////
#endif

