///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  - 
///////////////////////////////////////////////////////////////////////////////
#include "RLMeasurement.h"
#include "Angles.h"
#include "PhasePoint.h"
#include "GreenwichFrame.h"
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
RLMeasurement::RLMeasurement()
{
	Type = mtRLocation;

	int i;
	for (i = 0; i < 6; i++)
		Ians[i] = false;
}
//-----------------------------------------------------------------------------
//      new()
//-----------------------------------------------------------------------------
IMeasurement* RLMeasurement::Clone() const
{
	RLMeasurement *m = new RLMeasurement;
	*m = *this;
	return m;
}
//-----------------------------------------------------------------------------
//     v  
//-----------------------------------------------------------------------------
void RLMeasurement::GetRealVector(DVector& v) const
{
	v.Init(6U);

	int i;
	for (i = 0; i < 6; i++)
		v[i] = Vect[i];
}
//-----------------------------------------------------------------------------
//     v   
//-----------------------------------------------------------------------------
void RLMeasurement::GetMSEVector(DVector& v) const
{
	v.Init(6U);

	int i;
	for (i = 0; i < 6; i++)
		v[i] = Mse[i];
}
//-----------------------------------------------------------------------------
//     p   ,
//     
//-----------------------------------------------------------------------------
void RLMeasurement::GetWeightMatrix(DSymmMatrix &p) const
{
	p.Init(6U);
	p.Init(0.0);

	int i;
	for (i = 0; i < 6; i++)
	{
		if (Ians[i])
			p(i, i) = 0.0;
		else
			p(i, i) = 1.0 / (Mse[i] * Mse[i]);
	}
}
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
void RLMeasurement::SetStationCoordsAz(const GeoCoordsPrec &p, double az)
{
	Frame.Init(p, az);
}
//-----------------------------------------------------------------------------
//    
//  m1  m2 == NULL,   
//-----------------------------------------------------------------------------
void RLMeasurement::Diff(DVector &d, const DVector *m1, const DVector *m2) const
{
	d.Init(6U);

	DVector s1, s2;
	int i;

	if (m1 != NULL)
		s1 = *m1;
	else
	{
		s1.Init(6U);
		for (i = 0; i < 6; i++)
			s1(i) = Vect[i];
	}

	if (m2 != NULL)
		s2 = *m2;
	else
	{
		s2.Init(6U);
		for (i = 0; i < 6; i++)
			s2(i) = Vect[i];
	}

	for (i = 0; i < 6; i++)
	{
		if (i == 1 || i == 2)
			d(i) = AngleDiff(s1(i), s2(i));
		else
			d(i) = s1(i) - s2(i);
	}

}
//-----------------------------------------------------------------------------
//  -    
//-----------------------------------------------------------------------------
void RLMeasurement::RLCoords(const Vect6 &v, DVector &el) const
{
	el.Init(6U);

	//  D

	el(0) = v.Len();

	//  epsilon

	el(1) = acos(v.z / el(0));

	//  gamma

	double rxy = sqrt(v.x * v.x + v.y * v.y);
	el(2) = asin(v.y / rxy);
	if (v.x < 0)
	{
		if (v.y > 0)
			el(2) = Pi - el(2);
		else
			el(2) = -Pi - el(2);
	}

	//  

	el(3) = (v.x * v.vx + v.y * v.vy + v.z * v.vz) / el(0);  

	// epsilon(t)

	el(4) = (el(3) * v.z - el(0) * v.vz) / (el(0) * rxy);  //veps

	// gamma(t)

	el(5) = (v.x * v.vy - v.y * v.vx) / (rxy * rxy); //vgamma
}
//-----------------------------------------------------------------------------
//  -        
//-----------------------------------------------------------------------------
void RLMeasurement::RLCoordsPDM(const Vect6 &v, DVector &el, DMatrix &pdm) const
{
	RLCoords(v, el);
	double p = sqrt(v.x * v.x + v.y * v.y);

	pdm.Init(6, 6);

	//   
	int i;
	for (i = 0; i < 3; i++)
	{
		pdm(0, i) = v[i] / el(0);
		pdm(0, 3 + i) = 0.0;
	}

	//   eps
	pdm(1, 0) = v.z * v.x / (el(0) * el(0) * p);
	pdm(1, 1) = v.z * v.y / (el(0) * el(0) * p);
	pdm(1, 2) = -p / (el(0) * el(0));
	for (i = 0; i < 3; i++)
		pdm(1, 3 + i) = 0.0;

	//   gamma
	pdm(2, 0) = -v.y / (p * p);
	pdm(2, 1) = v.x / (p * p);
	pdm(2, 2) = 0.0;
	for (i = 0; i < 3; i++)
		pdm(2, 3 + i) = 0.0;

	//    
	pdm(3, 0) = (v.vx * el(0) - v.x * el(3)) / (el(0) * el(0));
	pdm(3, 1) = (v.vy * el(0) - v.y * el(3)) / (el(0) * el(0));
	pdm(3, 2) = (v.vz * el(0) - v.z * el(3)) / (el(0) * el(0));
	for (i = 0; i < 3; i++)
		pdm(3, 3 + i) = v[i] / el(0);

	//   veps
	pdm(4, 0) = (v.vx * v.z / (el(0) * el(0)) - 2.0 * v.x * v.z * el(3) / (el(0) * el(0) * el(0)) -
		el(3) * v.x * v.z / (el(0) * p * p) + v.vz * v.x / (p * p)) / p;
	pdm(4, 1) = (v.vy * v.z / (el(0) * el(0)) - 2.0 * v.y * v.z * el(3) / (el(0) * el(0) * el(0)) -
		el(3) * v.y * v.z / (el(0) * p * p) + v.vz * v.y / (p * p)) / p;
	pdm(4, 2) = (v.vz * v.z / (el(0) * el(0)) - 2.0 * v.z * v.z * el(3) / (el(0) * el(0) * el(0)) -
		el(3) * v.z * v.z / (el(0) * p * p) + v.vz * v.z / (p * p) + el(3) / (el(0) * p)) / p;
	pdm(4, 3) = v.z * v.x / (el(0) * el(0) * p);
	pdm(4, 4) = v.z * v.y / (el(0) * el(0) * p);
	pdm(4, 5) = -p / (el(0) * el(0));

	//   vgamma
	double p4 = p * p * p * p;
	pdm(5, 0) = ( v.vy * v.y * v.y - v.vy * v.x * v.x + 2.0 * v.x * v.y * v.vx) / p4;
	pdm(5, 1) = (-v.vx * v.x * v.x + v.vx * v.y * v.y - 2.0 * v.x * v.y * v.vy) / p4;
	pdm(5, 2) = 0.0;
	pdm(5, 3) = -v.y / (p * p);
	pdm(5, 4) = v.x / (p * p);
	pdm(5, 5) = 0.0;
}
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
bool RLMeasurement::GetXYZ(Vect6 &out, const double *v) const
{
	if (v == NULL)
		v = Vect;

	double d, sineps, coseps, singamma, cosgamma;
	sineps   = sin(v[1]);
	coseps   = cos(v[1]);
	singamma = sin(v[2]);
	cosgamma = cos(v[2]);
	d        = v[0];

	out.x = d * sineps * cosgamma;
	out.y = d * sineps * singamma;
	out.z = d * coseps;
	out.vx = v[3] * sineps * cosgamma + d * v[4] * coseps * cosgamma - d * v[5] * sineps * singamma;
	out.vy = v[3] * sineps * singamma + d * v[4] * coseps * singamma + d * v[5] * sineps * cosgamma;
	out.vz = v[3] * coseps - d * v[4] * sineps;
	
	return true;
}
//-----------------------------------------------------------------------------
//       ,
//       trj.
//    :
// t	-	,     
//           (    ,
//          )
// fr	-	 ,       
//        ,      (.   f  m)
// v	-	   .     0,
//          
// m	-	   Nx6,  N -   
//      (..    N   6 ).   
//        Rx Ry Rz Vx  Vy Vz,    -   .
//          0,     
// del	-	   .     0,
//           
//-----------------------------------------------------------------------------
bool RLMeasurement::Simulate(int matrixsize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f, DVector *v, DMatrix *m, DVector *del) const
{
	Vect6 pv;

	if (f == NULL && (v != NULL || m != NULL || del != NULL))
		f = &pv;

	t = GetTime();
	if (!trj.Predict(t, fr, *f))
		return false;

	if (v == NULL && m == NULL && del == NULL)
		return true;

	PhasePoint6D p;
	p.Init(fr, *f, t);

	p.Convert(Frame);

	DVector el(6);
	DMatrix pdm;

	RLCoordsPDM(p.CoordsVel, el, pdm);

	if (v != NULL)
	{
		v->Init(6U);
		*v = el;
		(*v)(0) *= 1.e3; // km
		(*v)(4) *= 1.e-3; // rad/s
		(*v)(5) *= 1.e-3; // rad/s
	}

	if (m != NULL)
	{
		FrameConverter3D cnv;
		fr.Get3DConverter(Frame, t, cnv);

		Vect3 rr, vv;
		m->Init(6, 6);
		int i, j;

		for (j = 0; j < 6; j++)
		{
			for (i = 0; i < 3; i++)
			{
				rr[i] = pdm(j, i);
				vv[i] = pdm(j, i + 3);
			}
			rr.MulRight(cnv.Rotation);
			vv.MulRight(cnv.Rotation);
			double a;
			switch (j)
			{
			case 0:
				a = 1.e3;
				break;
			case 4:
			case 5:
				a = 1.e-3;
				break;
			default:
				a = 1.0;
			}
			for (i = 0; i < 3; i++)
			{
				(*m)(j, i) = rr[i] * a;
				(*m)(j, i + 3) = vv[i] * a;
			}
		}
	}

	if (del != NULL)
	{
		if (v != NULL)
			Diff(*del, NULL, v);
		else
		{
			DVector wrk(6);
			wrk = el;
			wrk(0) *= 1.e3; // km
			wrk(4) *= 1.e-3; // rad/s
			wrk(5) *= 1.e-3; // rad/s
			Diff(*del, NULL, &wrk);
		}
	}

	return true;
}
///////////////////////////////////////////////////////////////////////////////
