///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  - 
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_RLMEASUREMENT_H
#define BNOTOOL_RLMEASUREMENT_H
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"
#include "LocalRLFrame.h"
///////////////////////////////////////////////////////////////////////////////
class RLMeasurement : public IMeasurement
  {
  public:
      // ""    6- 
      // 0 -  [km]
      // 1 - eps [rad]
      // 2 - gamma [rad] 
      // 3 - .  [km/s] 
      // 4 -   eps [rad/s] 
      // 5 -   gamma [rad/s] 
    double Vect[6];
      //  
    double Mse[6];
      //  
    bool Ians[6];

	    // 
    RLMeasurement();

      //     
    virtual unsigned GetSize() const
      { return 6;};
      //     v  
    virtual void GetRealVector(DVector& v) const;
      //     v   
    virtual void GetMSEVector(DVector& v) const;
      //     p   ,
      //     
    virtual void GetWeightMatrix(DSymmMatrix &p) const;

      //       ,
      //       trj.
      //    :
      // t	-	,     
      //           (    ,
      //          )
      // fr	-	 ,       
      //        ,      (.   f  m)
      // v	-	   .     0,
      //          
      // m	-	   Nx6,  N -   
      //      (..    N   6 ).   
      //        Rx Ry Rz Vx  Vy Vz,    -   .
      //          0,     
      // del	-	   .     0,
      //           
    virtual bool Simulate(int matrixsize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f = 0,
                          DVector *v = 0, DMatrix *m = 0, DVector *del = 0) const;
      //    
      //  m1  m2 == NULL,   
    virtual void Diff(DVector &d, const DVector *m1 = NULL, const DVector *m2 = NULL) const;
      //      new()
    virtual IMeasurement* Clone() const;

      //     
    void SetStationCoordsAz(const GeoCoordsPrec &p, double az);

      //  -    
    void RLCoords(const Vect6 &v, DVector &rl) const;
      //  -        
    void RLCoordsPDM(const Vect6 &v, DVector &rl, DMatrix &pdm) const;
      //     
    bool GetXYZ(Vect6 &out, const double *v = NULL) const;

      //    
    virtual bool GetIan(unsigned k) const
      { return Ians[k];};
      //   
    virtual void SetIan(unsigned k, bool f = true)
      { Ians[k] = f;};

  protected:
      //   
    LocalRLFrame Frame;
  };
///////////////////////////////////////////////////////////////////////////////
#endif
