///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_SGRAVITY_H
#define BNOTOOL_SGRAVITY_H
///////////////////////////////////////////////////////////////////////////////
#include "IModel.h"
///////////////////////////////////////////////////////////////////////////////
//     (   )
///////////////////////////////////////////////////////////////////////////////
class SGravityModel : public IGravityModel
{
public:
    //   
    SGravityModel()
    {}
    //    
    SGravityModel(double m, double r, double a)
    {
        mu = m; Re = r; alpha = a;
    }

    //   
    virtual double c(unsigned int /*n*/, unsigned int /*m*/) const
    {
        return 0;
    }

    virtual double d(unsigned int /*n*/, unsigned int /*m*/) const
    {
        return 0;
    }
        
    //   
    virtual unsigned int N() const
    {
        return 0;
    }

    virtual unsigned int M() const
    {
        return 0;
    };

    //  
    virtual TGravityModelId GetID() const
    {
        return grIdMu;
    };
};
///////////////////////////////////////////////////////////////////////////////
//    PZ90
///////////////////////////////////////////////////////////////////////////////
class EarthPZ90GravityModel : public IGravityModel
{
public:
    //   
    EarthPZ90GravityModel();

    //   
    virtual double c(unsigned int n, unsigned int m) const
    {
        return CC[m][n];
    };
    virtual double d(unsigned int n, unsigned int m) const
    {
        return DD[m][n];
    };

    virtual const double* const* get_cc() const { return CC; }

    virtual const double* const* get_dd() const { return DD; }

    //   
    virtual unsigned int N() const
    {
        return 36;
    };
    virtual unsigned int M() const
    {
        return 36;
    };

    //  
    virtual TGravityModelId GetID() const
    {
        return grIdPZ90;
    };

protected:
    // 
    double **CC;
    double **DD;
};
///////////////////////////////////////////////////////////////////////////////
//        ( )
///////////////////////////////////////////////////////////////////////////////
class EarthPZ902GravityModel : public IGravityModel
{
public:
    //   
    EarthPZ902GravityModel();

    double** getCC() const {
        return CC;
    }

    double** getDD() const {
        return DD;
    }

    //   
    virtual double c(unsigned int n, unsigned int m) const
    {
        return CC[m][n];
    };
    virtual double d(unsigned int n, unsigned int m) const
    {
        return DD[m][n];
    };

    virtual const double* const* get_cc() const { return CC; }

    virtual const double* const* get_dd() const { return DD; }

    //   
    virtual unsigned int N() const
    {
        return 36;
    };
    virtual unsigned int M() const
    {
        return 36;
    };

    //  
    virtual TGravityModelId GetID() const
    {
        return grIdPZ902s;
    };

protected:
    // 
    double **CC;
    double **DD;
};
///////////////////////////////////////////////////////////////////////////////
//        ( )
///////////////////////////////////////////////////////////////////////////////
class GravityModel : public IGravityModel
{
public:
    // ,    
    //     
    GravityModel(TGravityModelId Id, unsigned int N, unsigned int M,
        const IGravityModel *cd);
    //   
    GravityModel();

    // 
    virtual ~GravityModel()
    {
        freeAll();
    };

    //   
    virtual double c(unsigned int n, unsigned int m) const
    {
        return CC[m][n];
    };
    virtual double d(unsigned int n, unsigned int m) const
    {
        return DD[m][n];
    };

    //   
    double c(unsigned int n, unsigned int m, double value)
    {
        return (CC[m][n] = value);
    };
    double d(unsigned int n, unsigned int m, double value)
    {
        return (DD[m][n] = value);
    };

    virtual const double* const* get_cc() const { return CC; }

    virtual const double* const* get_dd() const { return DD; }

    //   
    virtual unsigned int N() const
    {
        return N_;
    };
    virtual unsigned int M() const
    {
        return M_;
    };

    //  
    virtual TGravityModelId GetID() const
    {
        return Id;
    };

    //    
    //     
    void Init(unsigned int N, unsigned int M, const IGravityModel *c_d);
    //   
    void initEmpty(TGravityModelId Id);
    //   
    void null();
    // 
    void print(char *comment, FILE *out = stdout) const;

    //  
    TGravityModelId Id;

protected:
    // 
    double **CC;
    double **DD;
    //  
    unsigned int N_;
    unsigned int M_;
    //  
    bool isInit;
    //  
    void freeAll();
    //   
    void SetSize(unsigned int N, unsigned int M);
};
///////////////////////////////////////////////////////////////////////////////
#endif
