///////////////////////////////////////////////////////////////////////////////
//   
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
#include "SList.h"
#include <string.h>
#include <assert.h>
#ifdef DEBUG_PRINT
#include "stdio.h"
#endif
///////////////////////////////////////////////////////////////////////////////
//   -   
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
TSList::TSList() 
  {
  List = NULL;
  Length  = 0;
  LastPos = -1;
  FoundPos = -2;
  FunSort  = NULL;
  PFunSort = NULL;
  LastFindSort = false;
  LastFindOk   = false;
  };
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
TSList::~TSList() 
  {
  if (List != NULL) 
    delete [] List;
  };
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
void TSList::Clear() 
  {
  LastPos  = -1;
  FoundPos = -2;
  LastFindSort = false;
  };
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
bool TSList::Insert(const void *p, int Ind) 
  {
  if (FunSort || PFunSort)
    return false;
  return LowInsert(p, Ind);
  };
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
bool TSList::Replace(const void *p,int Ind) 
  {
  if (Ind < 0 || Ind > LastPos) 
    return false;
  if (FunSort || PFunSort)
    return false;
  List[Ind] = p;
  return true;
  };
//-----------------------------------------------------------------------------
//       ,   
//-----------------------------------------------------------------------------
void TSList::Add(const void *p) 
  {
  if (LastPos < 0) 
      //  
    FoundPos = 0;
      //   
  else if (FunSort) 
    {
    if (!LastFindSort || (p != PPrev)) 
      Find(p, FoundPos, FunSort, true);
    if (LastFindOk)
      while (FunSort(p, List[FoundPos]) == 0) 
        if (++FoundPos > LastPos) 
          break;
    } 
  else if (PFunSort) 
    {
    if (!LastFindSort || (p!=PPrev)) 
      Find(p, FoundPos, *PFunSort, true);
    if (LastFindOk)
      while (PFunSort->compare(p, List[FoundPos]) == 0) 
        if (++FoundPos > LastPos) 
          break;
    } 
  else 
    FoundPos = LastPos + 1;
  LowInsert(p,FoundPos);
  };

//-----------------------------------------------------------------------------
//                 
//-----------------------------------------------------------------------------
void TSList::Insert(const void *p) 
  {
  if (LastPos < 0) 
    FoundPos = 0;
  else if (FunSort) 
    {
    if (!LastFindSort || (p!=PPrev)) 
      Find(p, FoundPos, FunSort, true);
    } 
  else if (PFunSort) 
    {
    if (!LastFindSort || (p!=PPrev)) 
      Find(p, FoundPos, *PFunSort, true);
    } 
  else 
    FoundPos = 0;
  LowInsert(p,FoundPos);
  };
//-----------------------------------------------------------------------------
//    pos
//-----------------------------------------------------------------------------
bool TSList::Delete(int pos) 
  {
  if (pos < 0 || pos > LastPos) 
    return false;
  if (pos < LastPos) 
    {
    int len1 = (LastPos-pos) * sizeof(void *);
    memmove(List+pos,List+pos+1,len1);
    };
  PPrev = NULL;
  LastPos -= 1;
  return true;
  };
//-----------------------------------------------------------------------------
//  n-   
//-----------------------------------------------------------------------------
void * TSList::operator[](int Ind) const 
  {
  if (Ind < 0 || Ind > LastPos) 
    return NULL;
  return (void *)(List[Ind]);
  };
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
bool TSList::LowInsert(const void *p, int pos) 
  {
     //      
  if (pos < 0 || pos > LastPos + 1) 
    return false;
  if (LastPos == Length - 1) 
    { //    
    SetLength(Length == 0 ? 16 : Length + (Length >> 1) + 1);
#ifdef DEBUG_PRINT
    printf("Make length %ld\n",len);
#endif
    };
  int len1 = (LastPos - pos + 1) * sizeof(void *);
  if (len1 > 0)
    memmove(List + pos + 1, List + pos, len1);
  List[pos] = p;
#ifdef DEBUG_PRINT
  printf("insert in pos %ld\n",pos);
#endif
  PPrev = NULL;
  LastPos += 1;
  return true;
  };
//-----------------------------------------------------------------------------
//  
//-----------------------------------------------------------------------------
void TSList::SetLength(int newLen) 
  {
  if (newLen == 0) 
    {
      //  
    delete [] List;
    List = 0;
    LastPos = -1;
    } 
  else 
    {
    const void **p1;
    p1 = List;
    List = new const void *[newLen];
    TAssert(List!=NULL);
    if (p1 != NULL) 
      {
      if (Length>newLen) 
        Length = newLen;
      memcpy(List, p1, Length * sizeof(void *));
      if (LastPos >= Length) 
        LastPos = Length-1;
      delete [] p1;
      };
    };
  FoundPos = -2;
  Length = newLen;
  };
//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
//   true,        
//   false,      
//        .
//-----------------------------------------------------------------------------
#define defineTSListFind(TypeFunSearch, funSearchName, funSearchRun) \
bool TSList::Find(const void *p, int &ind, TypeFunSearch funSearchName, bool bySort) \
  { \
  LastFindSort = bySort; \
  LastFindOk   = false; \
  PPrev = p; \
  int i; \
  if (LastPos==-1) \
    { \
      /*   */ \
    ind = FoundPos = 0; \
    return false; \
    }; \
  if (bySort && (FunSort || PFunSort)) \
    { \
      /*   - ,   */ \
    FoundPos = 0; \
    int h = LastPos; \
    int i1; \
    while (true) \
      { \
      i = funSearchRun(p, List[FoundPos]); \
      if (i == 0) \
        { \
        /*    */ \
        while (FoundPos > 0) \
          { \
          if (funSearchRun(p,List[FoundPos-1])) \
            break; \
          FoundPos--; \
          }; \
        ind = FoundPos; \
        LastFindOk = true; \
        return true; \
        }; \
      if (i < 0) \
        { \
          /*   */ \
        if (FoundPos < h) \
          { \
          if (FoundPos == 0) \
            { \
            ind = FoundPos = 0; \
            return false; \
            } \
          else \
            { \
            h = FoundPos; \
            FoundPos=0; \
            } \
          } \
        else \
          FoundPos -= h; \
        } \
      else \
        { \
        /*   */ \
        i1 = FoundPos + h; \
        if (i1 > LastPos) \
          { \
          if (FoundPos == LastPos) \
            { \
            ind = ++FoundPos; \
            return false; \
            } \
          else \
            { \
            h = LastPos - FoundPos; \
            FoundPos = LastPos; \
            } \
          } \
        else \
          FoundPos=i1; \
        }; \
      if (h == 0) \
        { \
        if (i < 0) \
          FoundPos -= 1; \
        ind = ++FoundPos; \
        return false; \
        }; \
      if (h == 1) \
        h = 0; \
      else if ((h&1)==0) \
        h = h >> 1; \
      else \
        h = (h + 1) >> 1; \
      }; \
    } \
  else \
    { \
    /*   */ \
    for (FoundPos = 0; FoundPos <= LastPos; FoundPos++) \
      { \
      i = funSearchRun(p, List[FoundPos]); \
      if (i == 0) \
        { \
        ind = FoundPos; \
        LastFindOk = true; \
        return true; \
        }; \
      }; \
    ind = FoundPos = LastPos+1; \
    return false; \
    }; \
  }
defineTSListFind(TCompareFunction, funSearch1,funSearch1);
defineTSListFind(TSListCompare  &,funSearch1,funSearch1.compare);

bool TSList::Find(const void *p)             
  { 
  return FunSort ? 
    Find(p, FoundPos, FunSort, true) : 
      PFunSort ? 
        Find(p, FoundPos, *PFunSort, true) : false; 
  };
bool TSList::Find(const void *p, TCompareFunction fun) 
  { 
  return Find(p, FoundPos, fun, false); 
  };
bool TSList::Find(const void *p, TSListCompare &fun) 
  { 
  return Find(p, FoundPos, fun, false); 
  };

void TSList::SetSortFunction(TCompareFunction fun) 
  {
  FunSort = fun;
  PFunSort = NULL;
  if (FunSort && (LastPos > 0)) 
    {  //    !
    int i,j;
    const void *p;
    for (i = 1; i <= LastPos; i++) 
      {
      p = List[i];
      for (j = i - 1; j >= 0 ; j--) 
        {
        if (FunSort(p,List[j])<0) 
          List[j+1] = List[j];
        else 
          break;
        };
      List[j+1] = p;
      };
    LastFindSort = false;
    LastFindOk   = false;
    FoundPos     = -2;
    };
  };

void TSList::SetSortFunction(TSListCompare *fun) 
  {
  FunSort = NULL;
  PFunSort = fun;
  if (!PFunSort) 
    return;

  if (LastPos > 0) 
    {  //    !
    int i,j;
    const void *p;
    for (i=1;i<=LastPos;i++) 
      {
      p = List[i];
      for (j=i-1;j>=0;j--) 
        {
        if (PFunSort->compare(p,List[j])<0) List[j+1] = List[j];
        else break;
        };
      List[j+1] = p;
      };
    LastFindSort = false;
    LastFindOk   = false;
    FoundPos     = -2;
    };
  };
