///////////////////////////////////////////////////////////////////////////////
//   
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//   
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOLS_SLIST_H
#define BNOTOOLS_SLIST_H
///////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
///////////////////////////////////////////////////////////////////////////////
//   .
//   <0,    ,\n
//                   >0 -  , 0 =  .\n
//                  ( <0,     , >0 -  )
typedef int (*TCompareFunction) (const void *pFromUser, const void *pFromList);
///////////////////////////////////////////////////////////////////////////////
class TSListCompare;
///////////////////////////////////////////////////////////////////////////////
//   -   
///////////////////////////////////////////////////////////////////////////////
class TSList 
  {
  public:
      //  
    TSList();                             
      // 
    virtual ~TSList();                     
      //    
    void SetLength(int pos);            
      //       ,   
    void Add(const void *p);   
      //                 
    void Insert(const void *p);
      //    
    bool Insert(const void *p, int pos);
      //    pos
    bool Delete(int pos);               
      //     
    bool Replace(const void *p, int pos); 
      //  n-   
    void * operator[](int Ind) const;     
      //       
    void SetSortFunction(TCompareFunction fun);   
      //       
    void SetSortFunction(TSListCompare   *fun);   
      //    .    -  FoundPos
    bool Find(const void *p);                     
      //        
    bool Find(const void *p, TCompareFunction fun); 
      //        
    bool Find(const void *p, TSListCompare  &fun); 
      //  
    void Clear();                                 
      //     
    const void **GetPtr() const { return List; }; 
      //   
    int Len() const { return LastPos + 1;};    
      //       
    int FoundPos;                              
  protected:
    bool LowInsert(const void *p, int pos);
      //    
    const void **List;         
      //    .
    const void *PPrev; 
      //           
    int Length;        
      //    
    TSListCompare *PFunSort;                      
      //    
    TCompareFunction FunSort; 
      //  -       Sort
    bool LastFindSort; 
      //  -    
    bool LastFindOk  ; 
      //    
    int LastPos;                               
  private:
      //    
    TSList(TSList &l) { TAssert(false); };
    void operator=(TSList &l) { TAssert(false); };
      //    . bySort=true :   
    bool Find(const void *p, int &ind, TCompareFunction fun, bool bySort=true);  
      //    . bySort=true :   
    bool Find(const void *p, int &ind, TSListCompare   &fun, bool bySort=true);  
  };

///////////////////////////////////////////////////////////////////////////////
///     
///////////////////////////////////////////////////////////////////////////////
//   compare   -1    ,
//                                     1    ,
//                                     0  .
///////////////////////////////////////////////////////////////////////////////
class TSListCompare 
  {
  public:
      // 
    virtual ~TSListCompare() 
      {} ; 
    virtual int compare(const void *pFromUser, const void *pFromList) = 0;
  };
///////////////////////////////////////////////////////////////////////////////
///     
///////////////////////////////////////////////////////////////////////////////
template <class objType> class TSListCompareDef : public TSListCompare
  {
  public:
    TSListCompareDef() : TSListCompare() {};
    virtual ~TSListCompareDef() {};
    virtual int compare(const void *p1, const void *p2) 
      { return compare((objType *)p1,(objType *)p2); };
    virtual int compare(const objType *p1, const objType *p2) = 0;
  };
///////////////////////////////////////////////////////////////////////////////
//    
///////////////////////////////////////////////////////////////////////////////
template <class objType, bool _clearOnDelete> class TSListDef : public TSList 
  {
  private:
    TSListDef(TSListDef &l) 
      { TAssert(false); }; 
    void operator=(TSListDef &l) 
      { TAssert(false); }; 
    bool clearOnDelete;
  public: 
      //  
    TSListDef() : TSList() 
      { clearOnDelete = _clearOnDelete; }; 
      //     . 
    TSListDef(bool clearOnDelete) : TSList() 
      { this->clearOnDelete = clearOnDelete; };
      // 
    ~TSListDef() 
      { 
      if (clearOnDelete) 
        for (int i = LastPos; i >= 0; i--) 
          delete ((objType *) (List[i])); 
      }; 
      //  
    void Clear(bool forceDelete = false) 
      { 
      if (clearOnDelete || forceDelete) 
        for (int i = LastPos; i >= 0; i--) 
          delete ((objType *) (List[i])); 
      TSList::Clear(); 
      }; 
      //    pos
    bool Delete(int pos, bool forceDelete = false) 
      { 
      if (clearOnDelete || forceDelete) 
        delete  ((objType *)(List[pos])); 
      return TSList::Delete(pos); 
      }; 
      //     
    bool Replace(const void *p, int pos, bool forceDelete=false) 
      { 
      if (FunSort || PFunSort)
        return false;
      if (clearOnDelete || forceDelete) 
        delete ((objType *) (List[pos])); 
      return TSList::Replace(p,pos); 
      };
      //  n-   
    objType * operator[](int Ind) const 
      { 
      if (Ind < 0 || Ind > LastPos) 
        return NULL;
      return (objType *) (List[Ind]); 
      };
      //     
    const objType **GetPtr() const 
      { return (const objType **) List; }; 
      //    
    void SetLength(int newLen) 
      { 
      if (clearOnDelete) 
        { 
        while (LastPos>=newLen) 
          delete ((objType *) (List[LastPos--])); 
        }; 
      TSList::SetLength(newLen); 
      }; 
      //       
    void SetSortFunction(TSListCompareDef<objType> *fun) 
      { TSList::SetSortFunction((TSListCompare *)fun); };
      //       
    void SetSortFunction(int fun(const objType *pFromUser, const objType *pFromList))
      { TSList::SetSortFunction((TCompareFunction)&fun);};
      //        
    bool Find(const void *p, TSListCompareDef<objType> *fun) 
      { return TSList::Find(p,(TSListCompare &)fun); }
      //        
    bool Find(const void *p, int *fun(const objType *pFromUser, const objType *pFromList)) 
      { return TSList::Find(p,(TCompareFunction)fun); }
  };

#endif
