///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
#include "SFramePars.h"
////////////////////////////////////////////////////////////////////////////////
//        
//   
////////////////////////////////////////////////////////////////////////////////
TFrameParameters::TFrameParameters()
  { 
  InitFlag = false; 
  };
//------------------------------------------------------------------------------
//     
//------------------------------------------------------------------------------
void TFrameParameters::Init(const TimeJD &t)
  { 
  TFix = t; 
  InitFlag = true;
  }
//------------------------------------------------------------------------------
bool TFrameParameters::IsEqual(const IFrameParameters &c) const
  {
  if (c.GetFrameType() != FrameType)
    return false;

  const TFrameParameters &ct = (const TFrameParameters &)c;
  return ct.TFix == TFix;
  }
////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
bool NZEFrameParameters::IsEqual(const IFrameParameters &c) const
  {
  if (c.GetFrameType() != FrameType)
    return false;

  const NZEFrameParameters &ct = (const NZEFrameParameters &)c;
  return ct.Point == Point;
  }
////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
bool RNBFrameParameters::IsEqual(const IFrameParameters &c) const
  {
  if (c.GetFrameType() != FrameType)
    return false;

  const RNBFrameParameters &ct = (const RNBFrameParameters &)c;
  if (!(ct.Point == Point))
    return false;

  return ct.Matrix == Matrix;
  }
//------------------------------------------------------------------------------
//      
//      J2000
//------------------------------------------------------------------------------
void RNBFrameParameters::Init(const Vect3 &p, const Vect3 &v)
  {
  Vect3 ex, ey, ez;

  ex = p;
  ex /= ex.Len();

  Matrix[0] = ex.x;
  Matrix[1] = ex.y;
  Matrix[2] = ex.z;

  ez = p;
  ez.MulVect(v);
  ez /= ez.Len();

  Matrix[6] = ez.x;
  Matrix[7] = ez.y;
  Matrix[8] = ez.z;

  ey = ez;
  ey.MulVect(ex);

  Matrix[3] = ey.x;
  Matrix[4] = ey.y;
  Matrix[5] = ey.z;

  Point = p;
  InitFlag = true;
  }
//------------------------------------------------------------------------------
//      
//      J2000
//------------------------------------------------------------------------------
void RNBFrameParameters::Init(const Vect6 &rv)
  {
  Vect3 p = rv;
  Vect3 v;
  rv.GetVelocity(v);
  Init(p, v);
  }
////////////////////////////////////////////////////////////////////////////////
//    J2000
////////////////////////////////////////////////////////////////////////////////
bool LocalJ2000FrameParameters::IsEqual(const IFrameParameters &c) const
  {
  if (c.GetFrameType() != FrameType)
    return false;

  const LocalJ2000FrameParameters &ct = (const LocalJ2000FrameParameters &)c;
  return ct.GPoint == GPoint;
  }
////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
bool LocalRLFrameParameters::IsEqual(const IFrameParameters &c) const
  {
  if (c.GetFrameType() != FrameType)
    return false;

  const LocalRLFrameParameters &ct = (const LocalRLFrameParameters &)c;

  if (ct.Az != Az)
    return false;

  return ct.GPoint == GPoint;
  }
////////////////////////////////////////////////////////////////////////////////
