///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//  TLE- 
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_TLEMEASUREMENT_H
#define BNOTOOL_TLEMEASUREMENT_H
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"
#include "msgp.h"
///////////////////////////////////////////////////////////////////////////////
class TLEMeasurement : public IMeasurement
{
public:
	// ""    6- 
	// 0 - n ( ) [rad/min]
	// 1 - e ()
	// 2 -    [rad] 
	// 3 -  [rad] 
	// 4 -   [rad] 
	// 5 -   [rad] 
	double Vect[6];
	//  
	double PVect[6];

	bool Ians[6];

	// 
	TLEMeasurement();

	//     
	virtual unsigned GetSize() const
	{ return 6;};
	//    
	virtual const TimeJD &GetTime() const
	{ return MTime;};
	//     v  
	virtual void GetRealVector(DVector& v) const;
	//     v   
	virtual void GetMSEVector(DVector& v) const;
	//     ,
	//    .
	//   0,    .
	virtual TSiteId GetStationID() const
	{ return TSiteId(); };
	//     ,
	//    
	virtual void SetStationID(unsigned id) 
	{};

	//  TLE-    TLE
	void SetByTLESet(const TLESet &tle);

	//       ,
	//       trj.
	//    :
	// t	-	,     
	//           (    ,
	//          )
	// fr	-	 ,       
	//        ,      (.   f  m)
	// v	-	   .     0,
	//          
	// m	-	   Nx6,  N -   
	//      (..    N   6 ).   
	//        Rx Ry Rz Vx  Vy Vz,    -   .
	//          0,     
	// del	-	   .     0,
	//           
	virtual bool Simulate(int matrixSize, ITraject &trj, TimeJD &t, const IFrame &fr, Vect6 *f = 0,
		DVector *v = 0, DMatrix *m = 0, DVector *del = 0) const;

	//  m1  m2 == NULL,   
	virtual void Diff(DVector &d, const DVector *m1 = NULL, const DVector *m2 = NULL) const;
	//      new()
	virtual IMeasurement* Clone() const;

	//    
	virtual bool GetIan(unsigned k) const
	{ return Ians[k];};
	//   
	virtual void SetIan(unsigned k, bool f = true)
      { Ians[k] = f;};
	//     p   ,
	//     
	virtual void GetWeightMatrix(DSymmMatrix &p) const;

	//      
	//  TLE-   
	static void CreateTLEVector(const Vect6 &rv, double *q);
	
	//    
	static void ToAux(const double *src, double *aux);

	//      
	static void FromAux(const double *aux, double *src);

	//      
	static void NormalizeAux(double *v);
protected:
	//   TLE-   
	void CreateTLESet(const double *q, TLESet &tle) const;
	//      MSGP-
	bool CreatePDM(const double *q, const double *eps, DSqrMatrix &pdm) const;
	//      0-2Pi
	static void Normalize(double *v);
	
	//  -  
	void PrintTLE(const char *str, const double *tle) const;
	//  -   
	void PrintVector(const char *str, const Vect6 &v) const;
	//  M   
	bool CorrectM(const Vect6 &rv, double *q) const;

private:
	void CalcTleXYZResiduals(const double* TLEElements, TimeJD date, Vect6& xyzMETE) const;
	void CalcTleXYZResiduals(const double* TLEElements1, const double* TLEElements2, TimeJD date, Vect6& residuals) const;
};
///////////////////////////////////////////////////////////////////////////////
#endif
