///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
#include "TMeasurementPacket.h"
///////////////////////////////////////////////////////////////////////////////
//     
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
IMeasurementRef::IMeasurementRef()
  {
  Ref = NULL;
  ManualFlag = false;
  AutoFlag = false;
  DeleteFlag = false;
  }
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
IMeasurementRef::~IMeasurementRef()
  {
  if (Ref != NULL)
   delete Ref;
  Ref = NULL;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
bool IMeasurementRef::IsActive() const
  {
  return !ManualFlag && !AutoFlag && !DeleteFlag;
  }

bool IMeasurementRef::IsDeleted() const
{
	return DeleteFlag;
  }
///////////////////////////////////////////////////////////////////////////////
// ,       
///////////////////////////////////////////////////////////////////////////////
//   true,        
//   false   
//-----------------------------------------------------------------------------
bool TMeasurementPacket::IsActive() const
  {
  IMeasurementEntry *f;

  for (f = first(); f != NULL; f = f->next())
    {
    if (f->IsActive())
      return true;
    }

  return false;
  }
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
unsigned TMeasurementPacket::GetActiveSize() const
  {
  IMeasurementEntry *e;
  unsigned ret = 0;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->IsActive())
      ret++;
    }

  return ret;
  }
//-----------------------------------------------------------------------------
//       .
//    ,     
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetFirstActive()
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->IsActive())
      {
      Current = e;
      return;
      }
    }

  Current = NULL;
  }
//-----------------------------------------------------------------------------
//       .
//    ,     
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetLastActive()
  {
  IMeasurementEntry *e;

  for (e = last(); e != NULL; e = e->prev())
    {
    if (e->IsActive())
      {
      Current = e;
      return;
      }
    }

  Current = NULL;
  }
//-----------------------------------------------------------------------------
//      ,     
//   t.
//    ,     
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetByTime(const TimeJD &t)
  {
  IMeasurementEntry *e;
  TimeJD ta, tc;
  ta = t;

  for (e = first(); e != NULL; e = e->next())
    {
    tc = e->Ref->GetTime();
    if (tc >= ta)
      {
      Current = e;
      return;
      }
    }

  Current = NULL;
  }
//-----------------------------------------------------------------------------
//       ,   
//     t.
//    ,     
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetByTimeActive(const TimeJD &t)
  {
  IMeasurementEntry *e;
  TimeJD ta, tc;
  ta = t;

  for (e = first(); e != NULL; e = e->next())
    {
    tc = e->Ref->GetTime();
    if (tc >= ta)
      break;
    }

  while (e != NULL && !e->IsActive())
    e = e->next();

  Current = e;
  }
//-----------------------------------------------------------------------------
//          m.
//      ,    
// .
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetCurrent(const IMeasurement &m)
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &m)
      {
      Current = e;
      return;
      }
    }

  Current = NULL;
  }
//-----------------------------------------------------------------------------
//          
//  id.
//      ,    
// .
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetCurrent(TMeasId id)
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      {
      Current = e;
      return;
      }
    }

  Current = NULL;
  }
//-----------------------------------------------------------------------------
//    ,     
//     .
//      ,
//        .
//-----------------------------------------------------------------------------
void TMeasurementPacket::GoNext()
  {
  if (Current == NULL)
    return;
  Current = Current->next();
  }
//-----------------------------------------------------------------------------
//    ,     
//    .
//      ,
//        .
//-----------------------------------------------------------------------------
void TMeasurementPacket::GoPrev()
  {
  if (Current == NULL)
    return;
  Current = Current->prev();
  }
//-----------------------------------------------------------------------------
//      ,     
//     .
//    ,       
//  .
//-----------------------------------------------------------------------------
void TMeasurementPacket::GoNextActive()
  {
  if (Current == NULL)
    return;

  IMeasurementEntry *e = Current->next();

  while (e != NULL && !e->IsActive())
    e = e->next();

  Current = e;
  }
//-----------------------------------------------------------------------------
//      ,    
//      .
//    ,       
//  .
//-----------------------------------------------------------------------------
void TMeasurementPacket::GoPrevActive()
  {
  if (Current == NULL)
    return;

  IMeasurementEntry *e = Current->prev();

  while (e != NULL && !e->IsActive())
    e = e->prev();

  Current = e;
  }
//-----------------------------------------------------------------------------
//      ,   
//  id.      ,    
// NULL.      .
//-----------------------------------------------------------------------------
const IMeasurement* TMeasurementPacket::Find(TMeasId id) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      return e->Ref;
    }

  return NULL;
  }
//-----------------------------------------------------------------------------
//   true,      
// (..   )
//-----------------------------------------------------------------------------
bool TMeasurementPacket::IsActive(const IMeasurement &mr) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &mr)
      break;
    }

  return e != NULL ? e->IsActive() : false;
  }
//-----------------------------------------------------------------------------
//   true,     id   
// (..   )
//-----------------------------------------------------------------------------
bool TMeasurementPacket::IsActive(TMeasId id) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      break;
    }

  return e != NULL ? e->IsActive() : false;
  }
//-----------------------------------------------------------------------------
//        m  .
//    true,     , ,
//    .
//-----------------------------------------------------------------------------
bool TMeasurementPacket::GetManualFlag(const IMeasurement &mr) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &mr)
      break;
    }

  return e != NULL ? e->ManualFlag : false;
  }
//-----------------------------------------------------------------------------
//    f     m  
// .  f  true,    , , 
//    .
//-----------------------------------------------------------------------------
bool TMeasurementPacket::GetManualFlag(TMeasId id) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      break;
    }
  return e != NULL ? e->ManualFlag : false;
  }
//-----------------------------------------------------------------------------
bool TMeasurementPacket::GetDeleteFlag(TMeasId id) const
{
	IMeasurementEntry *e;

	for (e = first(); e != NULL; e = e->next())
	{
		if (e->Ref->GetID() == id)
			break;
	}
	return e != NULL ? e->DeleteFlag : false;
}

bool TMeasurementPacket::GetDeleteFlag(const IMeasurement &mr) const
{
	IMeasurementEntry *e;

	for (e = first(); e != NULL; e = e->next())
	{
		if (e->Ref == &mr)
			break;
	}

	return e != NULL ? e->DeleteFlag : false;
}

void TMeasurementPacket::SetDeleteFlag(const IMeasurement &mr, bool f)
{
	IMeasurementEntry *e;

	for (e = first(); e != NULL; e = e->next())
	{
		if (e->Ref == &mr)
			break;
	}

	if (e != NULL)
		e->DeleteFlag = f;
}

//-----------------------------------------------------------------------------
//    f     m  
// .  f  true,    , , 
//    .
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetManualFlag(const IMeasurement &mr, bool f)
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &mr)
      break;
    }

  if (e != NULL)
    e->ManualFlag = f;
  }
//-----------------------------------------------------------------------------
//    f     
//  id  .  f  true,   
//  , ,    .
//-----------------------------------------------------------------------------
void TMeasurementPacket::SetManualFlag(TMeasId id, bool f)
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      break;
    }
  if (e != NULL)
    e->ManualFlag = f;
  }
//-----------------------------------------------------------------------------
//        m 
//  .    true,     , 
// ,    .
//-----------------------------------------------------------------------------
bool TMeasurementPacket::GetAutoFlag(const IMeasurement &mr) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &mr)
      break;
    }

  return e != NULL ? e->AutoFlag : false;
  }
//-----------------------------------------------------------------------------
//        
//  id  .
//    true,     , ,
//    .
//-----------------------------------------------------------------------------
bool TMeasurementPacket::GetAutoFlag(TMeasId id) const
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref->GetID() == id)
      break;
    }
  return e != NULL ? e->AutoFlag : false;
  }
//-----------------------------------------------------------------------------
//          
//  
//-----------------------------------------------------------------------------
void TMeasurementPacket::ClearAutoFlags()
  {
  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    e->AutoFlag = false;
  }
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
void TMeasurementPacket::Add(const IMeasurement *cm, bool man_flag)
  {
  TimeJD tm, t;
  tm = cm->GetTime();

  IMeasurementEntry *n = new IMeasurementEntry;
  n->Ref = cm->Clone();
  n->ManualFlag = man_flag;

  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    t = e->Ref->GetTime();
    if (tm < t)
      {
      insert_prev(e, n, 0);
      return;
      }
    }

  insert_last(n, 0);
  }
//-----------------------------------------------------------------------------
//         
//  m.   -  
//-----------------------------------------------------------------------------
void TMeasurementPacket::FilterMeasurement(const IMeasurement &m, const DVector &del)
  {
  if (Filter == NULL)
    return;

  IMeasurementEntry *e;

  for (e = first(); e != NULL; e = e->next())
    {
    if (e->Ref == &m)
      break;
    }

  if (e != NULL && !e->ManualFlag)
    e->AutoFlag = Filter->IsDeactivate(*e->Ref, del);
  }
//-----------------------------------------------------------------------------
//           
//  
//-----------------------------------------------------------------------------
void TMeasurementPacket::Create(IMeasurementPacket &src, const TimeJD &t1,
                                const TimeJD &t2)
  {
  TimeJD tl, tr, tt;

  clear();

  tl = t1;
  tr = t2;

  if (tr < tl)
    {
    tt = tl;
    tl = tr;
    tr = tt;
    }

  src.SetFirst();

  while (src.GetCurrent() != NULL && src.GetCurrent()->GetTime() < tl)
    src.GoNext();

  const IMeasurement *m;

  while ((m = src.GetCurrent()) != NULL && m->GetTime() <= tr)
    {
    Add(m, GetManualFlag(*m));
    src.GoNext();
    }
  }
//-----------------------------------------------------------------------------
//           
//  
//-----------------------------------------------------------------------------
bool TMeasurementPacket::IsEnough(unsigned kmin)
  {
  unsigned k = 0;

  SetFirstActive();

  const IMeasurement *m;

  while((m = GetCurrent()) != NULL)
    {
    k += m->GetSize();
    if (k > kmin)
      return true;
    GoNextActive();
    }

  return false;
  }
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
void TMeasurementPacket::DeleteInActive()
  {
  IMeasurementEntry *e, *en;

  e = first();
  while (e != NULL)
    {
    en = e->next();
    if (!e->IsActive())
      delete_node(e);
    e = en;  
    }
  }
///////////////////////////////////////////////////////////////////////////////
