///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//        
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_TRSCANER_H
#define BNOTOOL_TRSCANER_H
///////////////////////////////////////////////////////////////////////////////
#include "twll.h"
#include "IPredict.h"
///////////////////////////////////////////////////////////////////////////////
//         
//  
///////////////////////////////////////////////////////////////////////////////
//   ,     
///////////////////////////////////////////////////////////////////////////////
class IFloatCPoint
  {
  public:
      // ,   
    virtual ~IFloatCPoint() 
      {} ;
      // ,   
      //       
    virtual double CFunction(const PhasePoint6D &src) const = 0;
      // ,       
      //   true,    
    virtual bool CReaction(const PhasePoint6D &src)
      { return true;};
  };
///////////////////////////////////////////////////////////////////////////////
//      
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<IFloatCPoint> IFloatCPointEntry;
class IFloatCPointList : public Twll_List_Class<IFloatCPointEntry>
  {
  public:
  };
///////////////////////////////////////////////////////////////////////////////
//   ,     
///////////////////////////////////////////////////////////////////////////////
class IRelCPoint
  {
  public:
      // ,   
    virtual ~IRelCPoint() 
      {} ;
      // ,   
      //       
    virtual int CFunction(const PhasePoint6D &src) const = 0;
      // ,       
      //   true,    
    virtual bool CReaction(const PhasePoint6D &src)
      { return true;};
  };
///////////////////////////////////////////////////////////////////////////////
//      
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<IRelCPoint> IRelCPointEntry;
class IRelCPointList : public Twll_List_Class<IRelCPointEntry>
  {
  public:
  };
///////////////////////////////////////////////////////////////////////////////
//   ,      
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<TimeJD> TimeJDEntry;
class TimeJDList : public Twll_List_Class<TimeJDEntry>
  {
  public:
      //  dir > 0 -  
    void Sort(int dir);
      //  
    TimeJDEntry *Find(const TimeJD &t) const;
  };
class TimeCPointSet
  {
  public:
      //   
    virtual ~TimeCPointSet() 
      {} ;
      //   
    TimeCPointSet();

      //   
    void Set(const TimeJD &t);
      //  
    void Delete(const TimeJD &t);

      //      
    bool HasPoint(const TimeJD &t) const;
      //       ?
    bool HasPoints(const TimeJD &t1, const TimeJD &t2);
      //  ,    
    TimeJD GetPoint(const TimeJD &t1, const TimeJD &t2);
    
      // ,       
      //   true,    
    virtual bool CReaction(const PhasePoint6D &src)
      { return true;};

  protected:
      //   
    TimeJDList TList;
      //   
    int CurDir;
  };
typedef Twll_Entry_Class<TimeCPointSet> TimeCPointEntry;
class TimeCPointList : public Twll_List_Class<TimeCPointEntry>
  {
  public:
      //       ?
    bool HasPoints(const TimeJD &t1, const TimeJD &t2);
      //  ,    
    TimeJD GetPoint(const TimeJD &t1, const TimeJD &t2);
  };
///////////////////////////////////////////////////////////////////////////////
//   ,   
///////////////////////////////////////////////////////////////////////////////
class ITimeStepSet
  {
  public:
      // 
    virtual ~ITimeStepSet() 
      {} ;
      // 
    ITimeStepSet()
      { Dir = 0; Step = 0.0;};
      //  
    TimeJD TStart;
      //    
    double Step;

      // 
    void Init(const TimeJD &t1, const TimeJD &t2);
      //      
    bool HasPoint(const TimeJD &t) const;
      //       ?
    bool HasPoints(const TimeJD &t1, const TimeJD &t2);
      //  ,    
    TimeJD &GetPoint(const TimeJD &t1, const TimeJD &t2);
      // ,      
    bool Reaction(const PhasePoint6D &src);
  
  protected:
      // ,       
      //   true,    
    virtual bool CReaction(const PhasePoint6D &src)
      { return true;};

      //  
    int Dir;
      //  
    int NSteps;
      //    
    TimeJD TNext;
  };
typedef Twll_Entry_Class<ITimeStepSet> ITimeStepEntry;
class ITimeStepList : public Twll_List_Class<ITimeStepEntry>
  {
  public:
      //       ?
    bool HasPoints(const TimeJD &t1, const TimeJD &t2);
      //  ,    
    TimeJD GetPoint(const TimeJD &t1, const TimeJD &t2);
      // 
    void Init(const TimeJD &t1, const TimeJD &t2);
  };
///////////////////////////////////////////////////////////////////////////////
//      
///////////////////////////////////////////////////////////////////////////////
class TrScaner
  {
  public:
      //   
    TrScaner();
      // 
    virtual ~TrScaner();

      //    ,   
    void SetPoints(IFloatCPointList *flist)
      { FList = flist;};
      //    ,   
    void SetPoints(IRelCPointList *rlist)
      { RList = rlist;}; 
      //    ,    
    void SetPoints(TimeCPointList *tlist)
      { TList = tlist;};
      //    ,     
    void SetPoints(ITimeStepList *slist)
      { SList = slist;};

      //      
      //         
      //   
    bool Run(IPredict &prg, const TimeJD &t1, const TimeJD &t2, double step, double eps);

  protected:
      //   ,   
    IFloatCPointList *FList;
      //   ,   
    IRelCPointList *RList; 
      //   ,    
    TimeCPointList *TList;
      //  ,     
    ITimeStepList *SList;

      //      
      //     
    int Nf, Nr;
      //      
    double *Fl, *Fr, *Fc;
      //      
    int *Rl, *Rr, *Rc;
      //    
    PhasePoint6D Pl, Pr, Pc;

      //  
      //   
    void Clear();
      //   
    void SetFunctions(const PhasePoint6D &p, double *f, int *r);
      //    
    bool CheckTPoints(const PhasePoint6D &p) const;
      //     
    TimeJD SelectStep(const TimeJD &t1, const TimeJD &t2, double snom);
      //      
    bool HasCPoints(const double *fl, const int *rl, const double *fr, const int *rr);
      //    
    void DoStep(PhasePoint6D &pl, double *fl, int *rl, 
                const PhasePoint6D &pr, const double *fr, const int *rr); 
      //        
    bool FindCPoint(IPredict &prg, double eps);              
      //     ,    
    bool CallReaction();
  };
///////////////////////////////////////////////////////////////////////////////
#endif
