#include "TrjNum.h"
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
TrjNum::TrjNum() : InitTime(0, 0.0)
{
	Kb = Kappa = 0.0;
}
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
void TrjNum::SetKOProp(double kb, double kappa)
{
	Kb = kb;
	Kappa = kappa;
}
//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
bool TrjNum::Init(const DVector &u0)
{
	//        SetTime() !!!!
	TAssert(InitTime != TimeJD(0, 0.0));

	unsigned n = Size();
	TAssert(u0.Dim() == n);

	Predictor.SetKOParams(Kb, Kappa);

	PhasePoint6D ph;
	J2000Frame j2000;
	Vect6 v;

	unsigned i;
	for (i = 0; i < n; i++)
		v[i] = u0[i];

	ph.Init(j2000, v, InitTime);

	Predictor.SetInitPoint(ph);
	
	return true;
}
//-----------------------------------------------------------------------------
//         t.
//  fr   ,     
//   f (..   )
//-----------------------------------------------------------------------------
bool TrjNum::Predict(const TimeJD &t, const IFrame &fr, Vect6 &f)
{
	if (!Predictor.Run(t))
		return false;

	PhasePoint6D pr, res;

	Predictor.GetDestPoint(pr);
	res.SetFrame(fr);
	res.Assign(pr);

	f = res.CoordsVel;

	return true;
}
//-----------------------------------------------------------------------------
//       u  
//   t.
//-----------------------------------------------------------------------------
bool TrjNum::Move(const TimeJD &t, DVector &u)
{
	if (!Predictor.Run(t))
		return false;

	unsigned n = Size();

	u.Init(n);

	PhasePoint6D ph;

	Predictor.GetDestPoint(ph);
	unsigned i;

	for (i = 0; i < n; i++)
		u[i] = ph.CoordsVel[i];

	return true; 
}
///////////////////////////////////////////////////////////////////////////////
