///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//          
//   .     
///////////////////////////////////////////////////////////////////////////////
#include "TrjNumVar.h"
///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
TrjNumVar::TrjNumVar() : InitTime(0, 0.0)
{
	Kb = Kappa = 0.0;
	IsKbVar = IsKappaVar = false;
}
//-----------------------------------------------------------------------------
//       
//-----------------------------------------------------------------------------
void TrjNumVar::SetKOProp(double kb, double kappa, bool is_kb_var, 
						  bool is_kappa_var)
{
	Kb = kb;
	Kappa = kappa;
	IsKbVar = is_kb_var;
	IsKappaVar = is_kappa_var;
}

//     
//-----------------------------------------------------------------------------
unsigned TrjNumVar::Size() const
{
	unsigned n = 6;
	if (IsKbVar)
		n++; 
	if (IsKappaVar)
		n++; 

	return n;
}
//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
bool TrjNumVar::Init(const DVector &u0)
{
	//        SetTime() !!!!
	TAssert(InitTime != TimeJD(0, 0.0));

	//    
	unsigned n = Size();
	TAssert(u0.Dim() == n);

	//  Kb    
	if (IsKbVar)
		Kb = u0[6];

	if (IsKappaVar)
		Kappa = IsKbVar ? u0[7] : u0[6];

	Predictor.SetKOParams(Kb,Kappa,IsKbVar,IsKappaVar);

	PhasePoint6D ph;
	J2000Frame j2000;
	Vect6 v;

	int i;
	for (i = 0; i < 6; i++)
		v[i] = u0[i];

	ph.Init(j2000, v, InitTime);

	Predictor.SetInitPoint(ph);

	return true;
}
//-----------------------------------------------------------------------------
//         t.
//  fr   ,     
//   f (..   )
//-----------------------------------------------------------------------------
bool TrjNumVar::Predict(const TimeJD &t, const IFrame &fr, Vect6 &f)
{
	if (!GetPredictor()->Run(t))
		return false;

	PhasePoint6D pr, res;

	GetPredictor()->GetDestPoint(pr);
	res.SetFrame(fr);
	res.Assign(pr);

	f = res.CoordsVel;

	return true;
}
//-----------------------------------------------------------------------------
//         t 
//         .
//  fr   ,     
//   f (..   )   m.
//-----------------------------------------------------------------------------
bool TrjNumVar::PredictMatrix(const TimeJD &t, const IFrame& fr, Vect6 &f, DMatrix &m)
{
	if (!Predict(t, fr, f))
		return false;

	J2000Frame j2000;

	if (j2000.IsEqual(fr))
	{
		GetPredictor()->GetDestMatrix(m);
		m.Transpose();

		return true;
	}

	//          
	TAssert(false);

	return false; 
}
//-----------------------------------------------------------------------------
//       u  
//   t.
//-----------------------------------------------------------------------------
bool TrjNumVar::Move(const TimeJD &t, DVector &u)
{
	if (!GetPredictor()->Run(t))
		return false;

	unsigned n = Size();
	u.Init(n);

	PhasePoint6D ph;

	GetPredictor()->GetDestPoint(ph);
	int i;

	for (i = 0; i < 6; i++)
		u[i] = ph.CoordsVel[i];

	if (IsKbVar)
		u[6] = Kb;

	if (IsKappaVar)
	{
		if (IsKbVar)
			u[7] = Kappa;
		else
			u[6] = Kappa;
	}

	return true; 
}
///////////////////////////////////////////////////////////////////////////////
