#ifndef ASCNODEORBIT_H
#define ASCNODEORBIT_H

#include "KepData.h"
#include "TM.h"
#include "StateVector.h"
#include "Propagator.h"

class AscNodeOrbit
{
public:
	AscNodeOrbit(){};
	bool calculate(const SInitOrbit& orbit, bool vzmode);
	void ToXml(std::ostream& outs) const;
	double getLatitude() const {return m_latitude;}
	double getLongitude() const {return m_longitude;}
	double getAltitude() const {return m_altitude;}
	double getDrPeriod() const {return m_drperiod;}
	double getDrPeriodDt() const {return m_drperiodDt;}
	KepData getKdJ2000() const {return m_kdj2000;}
	KepData getKdEci() const {return m_kdeci;}
	TimeJD getDate() const {return m_date;}
	Vect6 getJ2000() const {return m_j2000;}
private:
	bool calcAscNode(Propagator& prop, bool vzmode, PhasePoint6D& point);
	bool getOrbPeriod(Propagator& prop, bool vzmode, double &tp, double &dt);
	double getKeplerianTime(double TA,double e,double w) const;
	KepData getKeplerData(const Vect6& pos);
	bool jumpToArgOfLatitude(Propagator& prop, double dstU, SInitOrbit& result) const;
	
	double m_latitude;
	double m_longitude;
	double m_altitude;
	double m_drperiod;
	double m_drperiodDt;
	KepData m_kdj2000;
	KepData m_kdeci;
	Vect6 m_j2000;
	TimeJD m_date;
};

#endif
