#include "KeplerTrj.h"
#include <iostream>

///////////////////////////////////////////////////////////////////////////////
// 
//-----------------------------------------------------------------------------
KeplerTrj::KeplerTrj() : InitTime(0, 0.0)
{
	Kb = Kappa = 0.0;
	IsKbVar = IsKappaVar = false;
}
//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
bool KeplerTrj::Init(const DVector &u0)
{
	//        SetTime() !!!!
	TAssert(InitTime != TimeJD(0, 0.0));

	//    
	unsigned n = 6;
	if (IsKbVar)
		n++; 
	if (IsKappaVar)
		n++; 
	TAssert(u0.Dim() == n);

	//  Kb    
	if (IsKbVar)
		Kb = u0[6];

	if (IsKappaVar)
		Kappa = IsKbVar ? u0[7] : u0[6];

	PhasePoint6D ph;
	J2000Frame j2000;
	Vect6 v;

	int i;
	for (i = 0; i < 6; i++)
		v[i] = u0[i];

	ph.Init(j2000, v, InitTime);

	Predictor.SetInitPoint(ph);

	return true;
}
//-----------------------------------------------------------------------------
//         t.
//  fr   ,     
//   f (..   )
//-----------------------------------------------------------------------------
bool KeplerTrj::Predict(const TimeJD &t, const IFrame &fr, Vect6 &f)
{
	if (!Predictor.Run(t))
		return false;

	PhasePoint6D pr(666);
	Predictor.GetDestPoint(pr);
	pr.Convert(fr);
	f = pr.CoordsVel;

	return true;
}

/*bool KeplerTrj::PredictEx(const TimeJD &t, Vect6 &f)
{
if (m_cacheTime == t && m_cache){
f = m_cachePos;
return true;
}

if (!Predictor.Run(t)){
return false;
}

Predictor.GetDestPointEX(f);

m_cacheTime = t;
m_cache = true;
m_cachePos = f;

return true;
}*/

//-----------------------------------------------------------------------------
//         t 
//         .
//  fr   ,     
//   f (..   )   m.
//-----------------------------------------------------------------------------
bool KeplerTrj::PredictMatrix(const TimeJD &t, const IFrame& fr, Vect6 &f, DMatrix &m)
{
	if (!Predict(t, fr, f))
		return false;

	J2000Frame j2000;

	if (j2000.IsEqual(fr))
	{
		// Predictor.GetDestMatrix(m);
		// m.Transpose();
		return true;
	}

	//          
	TAssert(false);

	return false; 
}
//-----------------------------------------------------------------------------
//       u  
//   t.
//-----------------------------------------------------------------------------
bool KeplerTrj::Move(const TimeJD &t, DVector &u)
{
	if (!Predictor.Run(t))
		return false;

	unsigned n = 6;
	if (IsKbVar)
		n++; 
	if (IsKappaVar)
		n++; 

	u.Init(n);

	PhasePoint6D ph;

	Predictor.GetDestPoint(ph);
	int i;

	for (i = 0; i < 6; i++)
		u[i] = ph.CoordsVel[i];

	if (IsKbVar)
		u[6] = Kb;

	if (IsKappaVar)
	{
		if (IsKbVar)
			u[7] = Kappa;
		else
			u[6] = Kappa;
	}

	return true; 
}

unsigned KeplerTrj::Size() const
{
	return 6;
}
