#ifndef TEXTFUNCTIONS_H
#define TEXTFUNCTIONS_H 

#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#include <list>
#include <stdio.h>
#include <iomanip>//j
#include "Converter.h"
#include "MException.h"

#define SPACES " \t\r\n"

static std::string ToStr(int arg)
{
	std::ostringstream ss;
	ss << arg;
	return ss.str();
}

static std::string ToStr(size_t arg)
{
	std::ostringstream ss;
	ss << arg;
	return ss.str();
}

static std::string ToStr(double arg)
{
	std::ostringstream ss;
	ss << arg;
	return ss.str();
}

static void string_replace(std::string& str, const std::string& from, const std::string& to) {
	if(from.empty())
		return;
	size_t start_pos = 0;
	while((start_pos = str.find(from, start_pos)) != std::string::npos) {
		str.replace(start_pos, from.length(), to);
		start_pos += to.length(); // In case 'to' contains 'from', like replacing 'x' with 'yx'
	}
}

static std::string ToStr(double value, int precision)
{
	std::ostringstream ss;
	ss << std::setprecision(precision);
	ss << value; 
	return ss.str();
}

static std::string trim_right(const std::string & s, const std::string & t = SPACES)
{ 
	std::string d (s); 
	std::string::size_type i (d.find_last_not_of (t));
	if (i == std::string::npos){
		return "";
	}else{
		return d.erase (d.find_last_not_of (t) + 1) ; 
	}
} 

static std::string trim_left(const std::string & s, const std::string & t = SPACES) 
{ 
	std::string d (s); 
	return d.erase (0, s.find_first_not_of (t)) ; 
}  

template <typename T, typename T2> bool GetMapItem(const std::map<T,T2>& map, const T& key, T2& ret)
{
	typename std::map<T,T2>::const_iterator it;
	it = map.find(key);
	if(it != map.end()){
		ret = it->second;
		return true;
	}else{
		return false;
	}
}

template <typename T, typename T2> bool GetMapItem(std::map<T,T2>& map, const T& key, T2*& ret)
{
	typename std::map<T,T2>::iterator it;
	it = map.find(key);
	if(it != map.end()){
		ret = &it->second;
		return true;
	}else{
		ret = 0;
		return false;
	}
}

template <typename T, typename T2> bool GetMapItem(const std::map<T,T2>& map, const T& key, const T2*& ret)
{
	typename std::map<T,T2>::const_iterator it;
	it = map.find(key);
	if(it != map.end()){
		ret = &it->second;
		return true;
	}else{
		ret = 0;
		return false;
	}
}

template <typename T, typename T2> bool GetMapItem(const std::map<T,T2>& map, const T& key)
{
	return map.find(key) != map.end();
}

static std::string trim(const std::string & s, const std::string & t = SPACES)
{ 
	std::string d (s); 
	return trim_left (trim_right (d, t), t) ; 
}

template < typename ContainerT > void splitstrf(const std::string& str, ContainerT& tokens, const std::string& delimiters = " ", const bool trimEmpty = false)
{
	std::string::size_type pos, lastPos = 0;
	while(true){
		pos = str.find_first_of(delimiters, lastPos);
		if(pos == std::string::npos){
			pos = str.length();

			if(pos != lastPos || !trimEmpty)
				tokens.push_back(typename ContainerT::value_type(str.data()+lastPos, (typename ContainerT::value_type::size_type)pos-lastPos ));

			break;
		}else{
			if(pos != lastPos || !trimEmpty)
				tokens.push_back(typename ContainerT::value_type(str.data()+lastPos, (typename ContainerT::value_type::size_type)pos-lastPos ));
		}

		lastPos = pos + 1;
	}
};

static std::vector<std::string> splitstr(const std::string& str_i, const char * const delims_i)
{
	std::vector<std::string> ret_vect;
	if (str_i.empty()) return ret_vect;

	splitstrf(str_i, ret_vect, std::string(delims_i));

	return ret_vect;
}

static bool ReadFile(std::string path, std::vector<std::string> &file)
{
	std::string line;
	file.clear();
	std::ifstream infile(path.c_str(), std::ios::in);
	if (infile.is_open()){
		while (getline(infile, line, '\n')){
			file.push_back(line);
		}
		infile.close();
		return true;
	}else{
		return false;
	}
}

static bool ReadFile(std::string path, std::string &str)
{
	FILE *fp;
	long len;
	char *buf;
	fp=fopen(path.c_str(),"rb");
	if (!fp){
		return 0;
	}
	fseek(fp,0,SEEK_END); //go to end
	len=ftell(fp); //get position at end (length)
	fseek(fp,0,SEEK_SET); //go to beg.
	buf=new char[len+1]; //malloc buffer
	buf[len] = 0;
	size_t res = fread(buf,len,1,fp); //read into buffer
	fclose(fp);
	str = std::string(buf);
	delete[] buf;
	
	if (!res){
		return false;
	}

/*	std::ifstream infile(path.c_str(), std::ios::in);
	if (infile.is_open()){
		std::string line;
		while (getline(infile, line)){
			str+=line;
			if (infile.good()){
				str+="\n";
			}
		}
		infile.close();
		return true;
	}else{
		return false;
	}*/
	return true;
}

static bool WriteFile(std::string path, std::string str)
{
	std::string line;
	std::ofstream file(path.c_str(), std::ios::out);
	
	if (file.is_open()){
		file << str.c_str();	
		file.close();
		return true;
	}else{
		std::cerr << "Error while writing to file [" + path + "]";
		return false;
	}
}

template <typename T, typename Tid> void GetAsMap(const std::vector<T>& vec, std::map<Tid, T>& res)
{
	for (typename std::vector<T>::const_iterator it = vec.begin(); it != vec.end(); it++){
		res[(*it).GetId()] = *it;
	}
}

static void checkBounds(int value, int* min, int* max, bool strict = false)
{
	if(!strict && min && value < *min || strict && min && value <= *min){
		throw (Exp() << "value [" + ToStr(value) + "] less than " + ToStr(*min));
	}

	if(!strict && max && value > *max || strict && max && value >= *max){
		throw (Exp() << "value [" + ToStr(value) + "] greater than " + ToStr(*max));
	}
} 

static void checkBounds(double value, double* min, double* max, bool strict = false)
{
	if(!strict && min && value < *min || strict && min && value <= *min){
		throw (Exp() << "value [" + ToStr(value) + "] less than " + ToStr(*min));
	}

	if(!strict && max && value > *max || strict && max && value >= *max){
		throw (Exp() << "value [" + ToStr(value) + "] greater than " + ToStr(*max));
	}
} 

static void parseStrA(std::string data, int pos1, int pos2, int& ret, int* min, int* max, bool strict)
{
	std::string part;
	if(pos1 == -1 && pos2 == -1){
		part = data;
	}else{
		part = pos2 >= 0 ? data.substr(pos1, pos2) : data.substr(pos1);
	}

	if(!Converter::ToNumber(part, ret)){
		std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
		throw (Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part);
	}

	if(min || max){
		try{
			checkBounds(ret, min, max, strict);
		}catch(Exp& e){
			std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
			throw (Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part << ". " << e.what());
		}
	}
}

static void parseStrA(std::string data, int pos1, int pos2, double& ret, double* min, double* max, bool strict)
{
	std::string part;
	if(pos1 == -1 && pos2 == -1){
		part = data;
	}else{
		part = pos2 >= 0 ? data.substr(pos1, pos2) : data.substr(pos1);
	}
	
	if(!Converter::ToNumber(part, ret)){
		std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
		throw (Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part);
	}

	if(min || max){
		try{
			checkBounds(ret, (double*)min, (double*)max, strict);
		}catch(Exp& e){
			std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
			throw (Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part << ". " << e.what());
		}
	}
}

static void parseStr(std::string data, int pos1, int pos2, int& ret, int min, int max, bool strict = false)
{
	parseStrA(data, pos1, pos2, ret, &min, &max, strict);
}

static void parseStr(std::string data, int pos1, int pos2, double& ret, double min, double max, bool strict = false)
{
	parseStrA(data, pos1, pos2, ret, &min, &max, strict);
}

static void parseStr(std::string data, int pos1, int pos2, int& ret)
{
	parseStrA(data, pos1, pos2, ret, 0, 0, false);
}

static void parseStr(std::string data, int pos1, int pos2, double& ret)
{
	parseStrA(data, pos1, pos2, ret, 0, 0, false);
}

static void parseStr(std::string data, int& ret, int min, int max, bool strict = false)
{
	parseStrA(data, -1, -1, ret, &min, &max, false);
}

static void parseStr(std::string data, double& ret, double min, double max, bool strict = false)
{
	parseStrA(data, -1, -1, ret, &min, &max, false);
}

static void parseStr(std::string data, int& ret)
{
	parseStrA(data, -1, -1, ret, 0, 0, false);
}

static void parseStr(std::string data, double& ret)
{
	parseStrA(data, -1, -1, ret, 0, 0, false);
}
#endif
