#ifndef TIMEFUNCTIONS_H
#define TIMEFUNCTIONS_H

#include "Converter.h"
#include "TM.h"
#include <vector>
#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include "globals.h"

/*void		IncJ(double& JD, double sec);
double		MoveJ(double JD, double sec);
double		LocalTimeToJ(int Cyear, int Cmonth, int Cday,int Chour,int Cminute,int Csecond, int Cms);
double		LocalTimeToJTz(int Cyear, int Cmonth, int Cday,int Chour,int Cminute,int Csecond, int Cms, double tz);
double		LocalTimeSTToJ(Time st);
double		LocalTimeSTToJTz(Time st, double tz);
double		SystemTimeToJ(int Cyear, int Cmonth, int Cday,double Chour,int Cminute,int Csecond, int Cms);
double		TimeInterval(double J1, double J2);
void		JToSystemTime(double J,int &Cyear,int &Cmonth,int &Cday,int& Chour,int& Cminute,int& Csecond,int& Cms);
std::string PrintFullTime(double J, double tz);
*/

static void GetPrecK(int msprec, int& k1, int& k2)
{
	if (msprec == 0){
		k1 = 1000;
		k2 = 1;
	}else if (msprec == 1){
		k1 = 100;
		k2 = 10;
	}else if (msprec == 2){
		k1 = 10;
		k2 = 100;
	}else if (msprec == 3){
		k1 = 1;
		k2 = 1000;
	}else if (msprec == 4){
		k1 = 1;
		k2 = 10000;
	}else if (msprec == 5){
		k1 = 1;
		k2 = 100000;
	}else{
		k1 = 1;
		k2 = 1000;
	}
}

static bool StrToDate(const std::string& date, TimeGD &gd)
{
	if (date.size() == 10 || date.size() >= 19){
		bool ret = true;
		std::string part0 = date.substr(0,10);
		
		ret = ret && Converter::ToNumber(part0.substr(0, 2),gd.Day);
		ret = ret && Converter::ToNumber(part0.substr(3, 2),gd.Month);
		ret = ret && Converter::ToNumber(part0.substr(6, 4),gd.Year);
				
		gd.Hour = 0;
		gd.Min = 0;
		gd.Sec = 0;
		gd.Fraction = 0;

		if (date.size() >= 19){
			std::string part1 = date.substr(11);

			ret = ret && Converter::ToNumber(part1.substr(0, 2),gd.Hour);
			ret = ret && Converter::ToNumber(part1.substr(3, 2),gd.Min);
			ret = ret && Converter::ToNumber(part1.substr(6, 2),gd.Sec);
			
			if(part1.size() > 8){
				int pos = part1.find_first_of(".");
				if (pos != std::string::npos){
					std::string fracStr = part1.substr(pos);
					ret = ret && Converter::ToNumber(fracStr,gd.Fraction);
				}
			}
		}

		return ret;
	}else{
		return false;
	}
}

static int isleap(int year)
{
	return (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0);
}

static int mthdays(int month, int year)
{
	static const int days[2][13] = {
		{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
		{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	};
	int leap = isleap(year);

	return days[leap][month];
}

static std::string DateToStr(const TimeGD& date, int prec = 3)
{
	int d = date.Hour;
	int m = date.Min;
	int s = date.Sec;
	int day = date.Day;
	int month = date.Month;
	int year = date.Year;
	int k2 = (int)pow(10.0, prec);
	int msec = Round(date.Fraction * k2);

	if (msec >= k2){
		s++;
		msec -= k2;
	}

	if (s >= 60){
		m++;
		s -= 60;
	}

	if (m >= 60){
		d++;
		m -= 60;
	}

	if (d >= 24 && m == 0 && s == 0 && msec == 0){
		day++;
		d -= 24;
	}

	int daysInMonth = mthdays(date.Month, date.Year);
	
	if(day > daysInMonth){
		month++;
		day -= daysInMonth;
	}

	if(month >= 13){
		year++;
		month -= 13;
	}

	std::ostringstream ss;

	ss << std::setfill('0');
	ss << std::setw(2) << day	 << "/";
	ss << std::setw(2) << month	 << "/";
	ss << std::setw(4) << year	 << " ";
	ss << std::setw(2) << d << ":";
	ss << std::setw(2) << m << ":";;
	ss << std::setw(2) << s;
	
	if (prec > 0){
		ss << ".";
		ss << std::setw(prec) << msec;
	}

	return ss.str();
}

static bool StrToDate(const std::string& date, int& year, int& day, int& month, int& hour, int& minute, int& second, double& fraction)
{
	if (date.size() == 10 || date.size() >= 19){
		bool ret = true;
		std::string part0 = date.substr(0,10);

		ret = ret && Converter::ToNumber(part0.substr(0, 2),day);
		ret = ret && Converter::ToNumber(part0.substr(3, 2),month);
		ret = ret && Converter::ToNumber(part0.substr(6, 4),year);

		hour = 0;
		minute = 0;
		second = 0;
		fraction = 0;

		if (date.size() >= 19){
			std::string part1 = date.substr(11);

			ret = ret && Converter::ToNumber(part1.substr(0, 2),hour);
			ret = ret && Converter::ToNumber(part1.substr(3, 2),minute);
			ret = ret && Converter::ToNumber(part1.substr(6, 2),second);

			if(part1.size() > 8){
				int pos = part1.find_first_of(".");
				if (pos != std::string::npos){
					std::string fracStr = part1.substr(pos);
					ret = ret && Converter::ToNumber(fracStr,fraction);
				}
			}
		}

		return ret;
	}else{
		return false;
	}
}

static TimeGD ShiftDateX(const TimeGD& date, double seconds)
{
	int jd;
	double jf;

	convertTimeGJ(date.Year, date.Month, date.Day, date.Hour, date.Min, date.Sec, date.Fraction, jd, jf);

	int nd = (int)(seconds/86400.0);
	jd += nd;
	jf += (seconds/86400.0-nd);

	TimeGD ret;
	convertTimeJG(jd,jf,ret.Year, ret.Month, ret.Day, ret.Hour, ret.Min, ret.Sec, ret.Fraction);

	return ret;
}

static double DaysIntervalX(const TimeGD& date1, const TimeGD& date2)
{
	int jd1,jd2;
	double jf1,jf2;
	convertTimeGJ(date1.Year, date1.Month, date1.Day, date1.Hour, date1.Min, date1.Sec, date1.Fraction, jd1, jf1);
	convertTimeGJ(date2.Year, date2.Month, date2.Day, date2.Hour, date2.Min, date2.Sec, date2.Fraction, jd2, jf2);
	double interval = jd2-jd1 + jf2-jf1;
	return interval;
}

static TimeGD ShiftDate(const TimeGD& date, double seconds)
{
	TimeJD jd = date;
	jd += TmDay(seconds/86400.0);
	return jd;
}

static TimeJD ShiftDate(const TimeJD& date, double seconds)
{
	TimeJD jd = date;
	jd += TmDay(seconds/86400.0);
	return jd;
}

static double DaysInterval(const TimeGD& date1, const TimeGD& date2)
{
	TimeJD jd1 = date1;
	TimeJD jd2 = date2;
	TmDay diff = jd2 - jd1;

	return diff.Day().GetDouble();
}

static double DaysInterval(const TimeJD& date1, const TimeJD& date2)
{
	return (date2 - date1).Day().GetDouble();
}

static void DegToDMS(double deg, int &rd, int &rm, double &rs)
{
	int sign = 1;
	if (deg < 0) {
		sign = -1;
		deg = fabs(deg);
	}

	int h,m;
	double s;

	h = (int)deg;
	m = (int)((deg-h)*60);
	s = ((deg-h)*60-m)*60.0;
	
	rd = h;
	rm = m;
	rs = s;
}

static void DegToHMS(double deg, int &rd, int &rm, double &rs)
{
	if (deg < 0)deg = 360+deg;
	if (deg >= 360)deg = deg-360;

	int h,m;
	double s;
	deg = deg/360.0*24;

	h = (int)deg;
	m = (int)((deg-h)*60);
	s = ((deg-h)*60-m)*60.0;
	
	rd = h;
	rm = m;
	rs = s;
}

static void JToSystemTime(double J,int &Cyear,int &Cmonth,int &Cday,int& Chour,int& Cminute,int& Csecond,int& Cms)
{
	int month, day;
	long year, a, c, d, x, y, jd;
	int BC;
	double dd;

#define RTOH (12.0/3.1415926535897932384626433832795)

	if( J < 1721425.5 ) 
		BC = 1;
	else
		BC = 0;

	jd = (long) (J + 0.5); 

	a = (100*jd + 3204500L)/3652425L;

	c = jd + 1486;
	if( jd >= 2299160.5 )
		c += a - a/4;
	else
		c += 38;

	d = (100*c - 12210L)/36525L;
	x = (36525L * d)/100L;

	y = ((c-x)*100L)/3061L;
	day = (int) (c - x - ((306L*y)/10L));
	month = (int) (y - 1);
	if( y > 13 )
		month -= 12;

	year = d - 4715;
	if( month > 2 )
		year -= 1;

	a = (jd + 1) % 7;

	dd = day + J - jd + 0.5;

	int cyear = year;

	if( BC )
	{
		year = -year + 1;
		cyear = -year;
		Cyear = year;
	}
	else
		Cyear =  year;

	Cday = (int) dd;
	Cmonth = month;

	a = (long) dd;
	dd = dd - a;

	int h, m;
	long sint, sfrac;
	double s;

	s = 2.0*3.1415926535897932384626433832795*dd * RTOH;
	if( s < 0.0 )
		s += 24.0;
	h = (int) s;
	s -= h;
	s *= 60;
	m = (int) s;
	s -= m;
	s *= 60;

	sfrac = (long) (1000.0 * s + 0.5);
	if( sfrac >= 60000L )
	{
		sfrac -= 60000L;
		m += 1;
		if( m >= 60 )
		{
			m -= 60;
			h += 1;
		}
	}
	sint = sfrac / 1000;
	sfrac -= sint * 1000;

	Chour	= h;
	Cminute	= m;
	Csecond	= sint;
	Cms	    = sfrac;
}


static double SystemTimeToJ(int Cyear, int Cmonth, int Cday,double Chour,int Cminute,int Csecond, int Cms)
{
	long y, a, b, c, e, m;
	double J;

	y = Cyear + 4800;
	if( Cyear < 0 )
		y += 1;

	m = Cmonth;
	if( m <= 2 )
	{
		m += 12;
		y -= 1;
	}

	e = (306 * (m+1))/10;
	a = y/100;	

	if( Cyear <= 1582L )
	{
		if( Cyear == 1582L )
		{
			if( Cmonth < 10 )
				goto julius;
			if( Cmonth > 10)
				goto gregor;
			if( Cday >= 15 )
				goto gregor;
		}
julius:
		b = -38;
	}
	else
	{
gregor:
		b = (a/4) - a;
	}

	c = (36525L * y)/100; 

	J = b + c + e + Cday - 32167.5 + (3600.0*Chour + 60.0*Cminute + Csecond +Cms/1000.0)/86400.0;

	return  J;
}

static double ToJD(const TimeGD& gd)
{
	return SystemTimeToJ(gd.Year,gd.Month,gd.Day,gd.Hour,gd.Min,gd.Sec,Round(gd.Fraction*1000));
}

static TimeGD ToGD(double jd)
{
	int Cyear, Cmonth, Cday, Chour, Cminute, Csecond, Cms;
	JToSystemTime(jd, Cyear, Cmonth, Cday, Chour, Cminute, Csecond, Cms);
	return TimeGD(Cday,Cmonth,Cyear,Chour,Cminute,Csecond,Cms/1000.0);
}

#endif

